@@ -1,4 +1,4 @@
-static char const rcsid[] = "$Id: blast_seq.c,v 1.66 2005/04/27 20:00:36 dondosha Exp $";
+static char const rcsid[] = "$Id: blast_seq.c,v 1.73 2005/11/09 14:49:49 camacho Exp $";
 /*
 * ===========================================================================
 *
@@ -44,22 +44,23 @@ static char const rcsid[] = "$Id: blast_seq.c,v 1.66 2005/04/27 20:00:36 dondosh
  */
 
 /** Converts a SeqLocPtr to a BlastSeqLoc, used for formatting.
- * @param mask_slp SeqLocPtr to be converted [in]
+ * @param slp SeqLocPtr to be converted [in]
+ * @param head_loc BlastSeqLoc returned from last call [in]
  * @return pointer to BlastSeqLoc
  */
 static BlastSeqLoc* 
-s_BlastSeqLocFromSeqLoc(SeqLocPtr mask_slp)
+s_BlastSeqLocFromSeqLoc(SeqLocPtr slp, BlastSeqLoc* head_loc)
 {
-   BlastSeqLoc* last_loc = NULL,* head_loc = NULL;
+   BlastSeqLoc* last_loc = head_loc;
 
-   if (mask_slp == NULL)
+   if (slp == NULL)
       return NULL;
 
-   if (mask_slp->choice == SEQLOC_PACKED_INT)
-      mask_slp = (SeqLocPtr) mask_slp->data.ptrvalue;
+   if (slp->choice == SEQLOC_PACKED_INT)
+      slp = (SeqLocPtr) slp->data.ptrvalue;
 
-   for ( ; mask_slp; mask_slp = mask_slp->next) {
-      SeqIntPtr si = (SeqIntPtr) mask_slp->data.ptrvalue;
+   for ( ; slp; slp = slp->next) {
+      SeqIntPtr si = (SeqIntPtr) slp->data.ptrvalue;
       if (!head_loc) {
          last_loc = head_loc = BlastSeqLocNew(&last_loc, si->from, si->to);
       } else {
@@ -70,42 +71,70 @@ s_BlastSeqLocFromSeqLoc(SeqLocPtr mask_slp)
 }
 
 BlastMaskLoc* 
-BlastMaskLocFromSeqLoc(SeqLoc* mask_locs, SeqLoc* query_locs)
+BlastMaskLocFromSeqLoc(SeqLoc* mask_seqlocs, SeqLoc* query_seqlocs, 
+                       EBlastProgramType program_number)
 {
-    const Int4 kNumSeqs = ValNodeLen(query_locs);
-    BlastMaskLoc* blast_mask;
-    Int4 tmp_index=0;
-    SeqLocPtr current_query_loc;
-    SeqLocPtr mask_var;
+    const Int4 kNumSeqs = ValNodeLen(query_seqlocs);
+    BlastMaskLoc* retval = NULL;
+    Int4 query_index = 0;
+    const unsigned int kNumContexts = BLAST_GetNumberOfContexts(program_number);
+    SeqLocPtr current_query_loc = NULL;
 
-    if (!mask_locs)
+    if (!mask_seqlocs)
         return NULL;
 
-    blast_mask = BlastMaskLocNew(kNumSeqs);
+    retval = BlastMaskLocNew(kNumSeqs*kNumContexts);
 
-    for (mask_var = mask_locs, current_query_loc = query_locs; 
-         mask_var && current_query_loc; 
-         current_query_loc = current_query_loc->next) {
-        SeqLocPtr current_mask = (SeqLocPtr) mask_var->data.ptrvalue;
-        /* If mask is empty, advance to the next link in the mask chain.
-           If mask Seq-id does not match sequence Seq-id, stay with this mask
-           for the next link in the sequence Seq-loc chain. */
-        if (!current_mask) {
-            mask_var = mask_var->next;
-        } else if (SeqIdMatch(SeqLocId(current_mask),
-                              SeqLocId(current_query_loc))) {
-            blast_mask->seqloc_array[tmp_index] = 
-                s_BlastSeqLocFromSeqLoc(current_mask);
-            BlastSeqLoc_RestrictToInterval(&blast_mask->seqloc_array[tmp_index], 
+    for (current_query_loc = query_seqlocs, query_index = 0; 
+         current_query_loc; 
+         current_query_loc = current_query_loc->next, query_index++) {
+
+        const int kCtxIndex = kNumContexts * query_index; /* context index */
+        SeqLocPtr mask_slp = NULL;
+
+        for (mask_slp = mask_seqlocs; mask_slp; mask_slp = mask_slp->next)
+        {
+           SeqLocPtr current_mask = (SeqLocPtr) mask_slp->data.ptrvalue;
+           /* If mask is empty, advance to the next link in the mask chain.
+              If mask Seq-id does not match sequence Seq-id, stay with this mask
+              for the next link in the sequence Seq-loc chain. */
+           if (current_mask && 
+               SeqIdMatch(SeqLocId(current_mask), SeqLocId(current_query_loc))) 
+           {
+               retval->seqloc_array[kCtxIndex] = 
+                     s_BlastSeqLocFromSeqLoc(current_mask,
+                                             retval->seqloc_array[kCtxIndex]);
+           }
+        }
+         
+        if (retval->seqloc_array[kCtxIndex])
+        {
+            const Boolean kIsNa = Blast_QueryIsNucleotide(program_number) &&
+                !Blast_QueryIsTranslated(program_number) &&
+                !Blast_ProgramIsPhiBlast(program_number);
+            BlastSeqLoc_RestrictToInterval(&retval->seqloc_array[kCtxIndex], 
                                            SeqLocStart(current_query_loc), 
                                            SeqLocStop(current_query_loc));
-            mask_var = mask_var->next;
+            if (kIsNa) {
+                /* N.B.: Unlike in the C++ APIs, this logic is only applied to
+                 * non-translated nucleotide queries. See comment for
+                 * BlastMaskLocDNAToProtein */
+                Uint1 strand = SeqLocStrand(current_query_loc);
+                if (strand == Seq_strand_minus) {
+                    retval->seqloc_array[kCtxIndex+1] = 
+                        retval->seqloc_array[kCtxIndex];
+                    retval->seqloc_array[kCtxIndex] = NULL;
+                } else if (strand == Seq_strand_plus) {
+                    retval->seqloc_array[kCtxIndex+1] = NULL;
+                } else {
+                    retval->seqloc_array[kCtxIndex+1] = 
+                        BlastSeqLocListDup(retval->seqloc_array[kCtxIndex]);
+                }
+            }
         }
-        /* Advance the index in BlastMaskLoc in any case. */
-        tmp_index++;
     }
     
-    return blast_mask;
+    return retval;
 }
 
 SeqLoc*
@@ -120,41 +149,38 @@ Blast_ValNodeMaskListFree(SeqLoc* mask_loc)
 
 SeqLocPtr BlastMaskLocToSeqLoc(EBlastProgramType program_number, 
                                const BlastMaskLoc* mask_loc, 
-                               const SeqLoc* seq_loc)
+                               SeqLoc* query_loc)
 {
-   SeqLocPtr mask_head = NULL, last_mask = NULL;
+   SeqLocPtr retval = NULL, retval_tail = NULL;
    Int4 index;
-   const Boolean k_translate = (program_number == eBlastTypeBlastx || 
-                                program_number == eBlastTypeTblastx ||
-                                program_number == eBlastTypeRpsTblastn);
-   const Uint1 k_num_frames = (k_translate ? NUM_FRAMES : 1);
+   const Boolean k_translate = Blast_QueryIsTranslated(program_number);
+   const Uint1 k_num_frames = BLAST_GetNumberOfContexts(program_number);
    SeqLoc* slp;
 
    if (mask_loc == NULL || mask_loc->seqloc_array == NULL)
       return NULL;
 
-   for (index=0, slp = (SeqLoc*)seq_loc; slp; ++index, slp = slp->next)
+   for (index=0, slp = query_loc; slp; ++index, slp = slp->next)
    {
-      Int4 frame_index = index*k_num_frames;
+      const int kCtxIndex = k_num_frames * index; /* context index */
       Int4 tmp_index;
-      for (tmp_index=frame_index; tmp_index<(frame_index+k_num_frames); tmp_index++)
+      Int4 slp_from = SeqLocStart(slp);
+      SeqIdPtr seqid = SeqLocId(slp);
+      for (tmp_index=kCtxIndex; tmp_index<(kCtxIndex+k_num_frames); tmp_index++)
       {
          BlastSeqLoc* loc = NULL;
-         SeqIdPtr seqid = SeqLocId(slp);
-         SeqLocPtr mask_slp_head = NULL, mask_slp_last = NULL;
+         SeqLocPtr mask_slp_head = NULL, mask_slp_tail = NULL;
          for (loc = mask_loc->seqloc_array[tmp_index]; loc; loc = loc->next)
          {
-            SSeqRange* di = loc->ssr;
             SeqIntPtr si = SeqIntNew();
-            si->from = di->left;
-            si->to = di->right;
+            si->from = loc->ssr->left + slp_from;
+            si->to = loc->ssr->right + slp_from;
             si->id = SeqIdDup(seqid);
-            if (!mask_slp_last)
-               mask_slp_last = 
-                  ValNodeAddPointer(&mask_slp_head, SEQLOC_INT, si);
-            else 
-               mask_slp_last = 
-                  ValNodeAddPointer(&mask_slp_last, SEQLOC_INT, si);
+            /* Append the pointer, but also keep track of the tail of the list
+             * so that appending to the list is a constant operation */
+            mask_slp_tail = ValNodeAddPointer
+                ( (mask_slp_tail ? &mask_slp_tail : &mask_slp_head), 
+                  SEQLOC_INT, si);
          }
 
          if (mask_slp_head) {
@@ -169,15 +195,15 @@ SeqLocPtr BlastMaskLocToSeqLoc(EBlastProgramType program_number,
             else
                 tmp_choice = 0;
 
-            if (!last_mask) {
-               last_mask = ValNodeAddPointer(&mask_head, tmp_choice, new_mask_slp);
-            } else {
-               last_mask = ValNodeAddPointer(&last_mask, tmp_choice, new_mask_slp);
-            }
+            /* Append the pointer, but also keep track of the tail of the list
+             * so that appending to the list is a constant operation */
+            retval_tail = ValNodeAddPointer
+                ( (retval_tail ? &retval_tail : &retval), 
+                  tmp_choice, new_mask_slp);
         }
       }
    }
-   return mask_head;
+   return retval;
 }
 
 /** Set field values for one element of the context array of a
@@ -330,7 +356,7 @@ s_QueryInfoSetUp(SeqLocPtr slp, EBlastProgramType program,
  * @param buffer Buffer to fill.
  */
 static Int2 
-s_SeqLocReadSequence(SeqLocPtr slp, Uint1 encoding, Uint1** buffer)
+s_SeqLocReadSequence(SeqLocPtr slp, EBlastEncoding encoding, Uint1** buffer)
 {
    Uint1* buffer_var = *buffer;
    Int4 size, index;
@@ -342,18 +368,22 @@ s_SeqLocReadSequence(SeqLocPtr slp, Uint1 encoding, Uint1** buffer)
                           buffer_var, NULL);
 
    switch (encoding) {
-   case BLASTP_ENCODING: 
+   case eBlastEncodingProtein: 
       for (index = 0; index < size; index++) 
           buffer_var[index] = AMINOACID_TO_NCBISTDAA[buffer_var[index]];
       break;
-   case NCBI4NA_ENCODING:
+   case eBlastEncodingNcbi4na:
       for (index = 0; index < size; index++) 
          buffer_var[index] = IUPACNA_TO_NCBI4NA[buffer_var[index]];
       break;
-   case BLASTNA_ENCODING:
+   case eBlastEncodingNucleotide:
       for (index = 0; index < size; index++) 
          buffer_var[index] = IUPACNA_TO_BLASTNA[buffer_var[index]];
       break;
+   default:
+       /* This function should not be called for any other encodings - 
+          return an error status. */
+       return -1;
    }
 
    *buffer = buffer_var + size;
@@ -370,12 +400,13 @@ s_SeqLocReadSequence(SeqLocPtr slp, Uint1 encoding, Uint1** buffer)
  * @param buffer Buffer to populate. Must be already allocated. [in] [out]
  */
 static Int2 
-s_SeqLocFillSequenceBuffer(SeqLocPtr slp, Uint1 encoding, 
+s_SeqLocFillSequenceBuffer(SeqLocPtr slp, EBlastEncoding encoding, 
     Boolean add_sentinel_bytes, Boolean both_strands, Uint1* buffer)
 {
    Uint1* buffer_var;
-   Uint1 sentinel = 
-      (encoding == BLASTNA_ENCODING ? NCBI4NA_TO_BLASTNA[NULLB] : NULLB);
+   Uint1 sentinel = (encoding == eBlastEncodingNucleotide 
+                     ? NCBI4NA_TO_BLASTNA[NULLB] 
+                     : NULLB);
    Uint1 seq_code, strand;
 
    buffer_var = buffer;
@@ -385,7 +416,7 @@ s_SeqLocFillSequenceBuffer(SeqLocPtr slp, Uint1 encoding,
       ++buffer_var;
    }
 
-   if (encoding == BLASTP_ENCODING) {
+   if (encoding == eBlastEncodingProtein) {
       seq_code = Seq_code_ncbistdaa;
       strand = Seq_strand_unknown;
    } else {
@@ -457,7 +488,7 @@ Int2 BLAST_GeneticCodeFind(Int4 gc, Uint1** genetic_code)
  * @param query_info The query information structure, pre-initialized,
  *                   but filled here [in]
  * @param query_options Query setup options, containing the genetic code for
- *                      translation [in]
+ *                      translation. N.B.: its strand_option field is ignored [in]
  * @param num_frames How many frames to get for this sequence? [in]
  * @param encoding In what encoding to retrieve the sequence? [in]
  * @param buffer_out Buffer to hold plus strand or protein [out]
@@ -465,8 +496,8 @@ Int2 BLAST_GeneticCodeFind(Int4 gc, Uint1** genetic_code)
  */
 static Int2 
 s_GetSequence(SeqLocPtr slp, BlastQueryInfo* query_info, 
-   const QuerySetUpOptions* query_options, Uint1 num_frames, Uint1 encoding, 
-   Uint1* *buffer_out, Int4 *buffer_length)
+   const QuerySetUpOptions* query_options, Uint1 num_frames, 
+   EBlastEncoding encoding, Uint1* *buffer_out, Int4 *buffer_length)
 {
    Int2		status=0; /* return value. */
    Int4 total_length; /* Total length of all queries/frames/strands */
@@ -486,7 +517,7 @@ s_GetSequence(SeqLocPtr slp, BlastQueryInfo* query_info,
       /* allow two extra bytes for sentinels or a trailing
          null appended by the low-level sequence conversion */
       total_length = (*buffer_length) + 2;
-      if (encoding == NCBI4NA_ENCODING) {
+      if (encoding == eBlastEncodingNcbi4na) {
          /* Searches with translated subjects (tblastn, tblastx) */
          add_sentinel_bytes = FALSE;
       }
@@ -576,30 +607,28 @@ Int2 BLAST_SetUpQuery(EBlastProgramType program_number,
    Int4 buffer_length;
    Int2 status;
    Uint1 num_frames;
-   Uint1 encoding;
-   BlastMaskLoc* lcase_mask = NULL;
+   EBlastEncoding encoding;
 
    if (query_slp == NULL || query_options == NULL ||
        query_info == NULL || query_blk == NULL)
       return -1;
 
-   lcase_mask = BlastMaskLocFromSeqLoc(masking_locs, query_slp);
 
    if ((status = s_QueryInfoSetUp(query_slp, program_number, query_info)))
       return status;
 
    if (program_number == eBlastTypeBlastn || 
        program_number == eBlastTypePhiBlastn) {
-      encoding = BLASTNA_ENCODING;
+      encoding = eBlastEncodingNucleotide;
       num_frames = 2;
    } else if (program_number == eBlastTypeBlastp ||
               program_number == eBlastTypeRpsBlast ||
               program_number == eBlastTypeTblastn ||
               program_number == eBlastTypePhiBlastp) {
-      encoding = BLASTP_ENCODING;
+      encoding = eBlastEncodingProtein;
       num_frames = 1;
    } else { /* blastx or rpstblastn, which is also essentially blastx */
-      encoding = NCBI4NA_ENCODING;
+      encoding = eBlastEncodingNcbi4na;
       num_frames = NUM_FRAMES;
    }
 
@@ -613,10 +642,11 @@ Int2 BLAST_SetUpQuery(EBlastProgramType program_number,
                                     0, query_blk, TRUE)))
       return status;
 
-   if (lcase_mask) {
-       if (program_number == eBlastTypeBlastx || 
-           program_number == eBlastTypeTblastx ||
-           program_number == eBlastTypeRpsTblastn)
+   if (masking_locs) {
+       BlastMaskLoc* lcase_mask = BlastMaskLocFromSeqLoc(masking_locs, 
+                                                         query_slp,
+                                                         program_number);
+       if (Blast_QueryIsTranslated(program_number))
            BlastMaskLocDNAToProtein(lcase_mask, *query_info);
        (*query_blk)->lcase_mask = lcase_mask;
        (*query_blk)->lcase_mask_allocated = TRUE;
@@ -633,18 +663,18 @@ Int2 BLAST_SetUpSubject(EBlastProgramType program_number,
                                       sequence in two sequences case */
    Int4 buffer_length=0; /* Length of subject sequence for two sequences 
                             case */
-   Uint1 encoding;
+   EBlastEncoding encoding;
    const Boolean kNucleotide = (program_number == eBlastTypeBlastn || 
                                 program_number == eBlastTypePhiBlastn);
    const Boolean kTranslated = (program_number == eBlastTypeTblastn ||
                                 program_number == eBlastTypeTblastx);
 
    if (kNucleotide)
-      encoding = BLASTNA_ENCODING;
+      encoding = eBlastEncodingNucleotide;
    else if (kTranslated) {
-      encoding = NCBI4NA_ENCODING;
+      encoding = eBlastEncodingNcbi4na;
    } else {
-      encoding = BLASTP_ENCODING;
+      encoding = eBlastEncodingProtein;
    }
 
    if ((status = s_GetSequence(subject_slp, NULL, NULL, 1, encoding,
