@@ -1,4 +1,4 @@
-/* $Id: aa_ungapped.c,v 1.36 2004/08/05 20:41:01 dondosha Exp $
+/* $Id: aa_ungapped.c,v 1.45 2005/11/16 14:27:03 madden Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -6,7 +6,7 @@
  *
  *  This software/database is a "United States Government Work" under the
  *  terms of the United States Copyright Act.  It was written as part of
- *  the author's offical duties as a United States Government employee and
+ *  the author's official duties as a United States Government employee and
  *  thus cannot be copyrighted.  This software/database is freely available
  *  to the public for use. The National Library of Medicine and the U.S.
  *  Government have not placed any restriction on its use or reproduction.
@@ -28,10 +28,13 @@
  * @todo FIXME Need description
  */
 
+#ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: aa_ungapped.c,v 1.36 2004/08/05 20:41:01 dondosha Exp $";
+    "$Id: aa_ungapped.c,v 1.45 2005/11/16 14:27:03 madden Exp $";
+#endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <algo/blast/core/aa_ungapped.h>
+#include "blast_extend_priv.h"
 
 Int2 BlastAaWordFinder(BLAST_SequenceBlk* subject,
                        BLAST_SequenceBlk* query,
@@ -39,8 +42,7 @@ Int2 BlastAaWordFinder(BLAST_SequenceBlk* subject,
                        Int4** matrix,
                        const BlastInitialWordParameters* word_params,
                        Blast_ExtendWord* ewp,
-                       Uint4* query_offsets,
-                       Uint4* subject_offsets,
+                       BlastOffsetPair* NCBI_RESTRICT offset_pairs,
                        Int4 offset_array_size,
                        BlastInitHitList* init_hitlist, 
                        BlastUngappedStats* ungapped_stats)
@@ -56,7 +58,7 @@ Int2 BlastAaWordFinder(BLAST_SequenceBlk* subject,
                                         matrix,
                                         word_params->cutoff_score,
                                         word_params->x_dropoff,
-                                        query_offsets, subject_offsets,
+                                        offset_pairs,
                                         offset_array_size,
                                         init_hitlist, ungapped_stats);
     }
@@ -67,26 +69,28 @@ Int2 BlastAaWordFinder(BLAST_SequenceBlk* subject,
                                         matrix,
                                         word_params->cutoff_score,
                                         word_params->x_dropoff,
-                                        query_offsets, subject_offsets,
+                                        offset_pairs,
                                         offset_array_size,
                                         init_hitlist, ungapped_stats);
     }
 
+  Blast_InitHitListSortByScore(init_hitlist);
+
   return status;
 }
 
-Int2 BlastAaWordFinder_TwoHit(const BLAST_SequenceBlk* subject,
-			      const BLAST_SequenceBlk* query,
-			      const LookupTableWrap* lookup_wrap,
-			      BLAST_DiagTable* diag,
-			      Int4 ** matrix,
-			      Int4 cutoff,
-			      Int4 dropoff,
-			      Uint4 * NCBI_RESTRICT query_offsets,
-			      Uint4 * NCBI_RESTRICT subject_offsets,
-			      Int4 array_size,
-			      BlastInitHitList* ungapped_hsps, 
-               BlastUngappedStats* ungapped_stats)
+Int2 
+BlastAaWordFinder_TwoHit(const BLAST_SequenceBlk* subject,
+                         const BLAST_SequenceBlk* query,
+                         const LookupTableWrap* lookup_wrap,
+                         BLAST_DiagTable* diag,
+                         Int4 ** matrix,
+                         Int4 cutoff,
+                         Int4 dropoff,
+                         BlastOffsetPair* NCBI_RESTRICT offset_pairs,
+                         Int4 array_size,
+                         BlastInitHitList* ungapped_hsps, 
+                         BlastUngappedStats* ungapped_stats)
 {
    BlastLookupTable* lookup=NULL;
    BlastRPSLookupTable* rps_lookup=NULL;
@@ -105,6 +109,7 @@ Int2 BlastAaWordFinder_TwoHit(const BLAST_SequenceBlk* subject,
    DiagStruct* diag_array;
    Boolean right_extend;
    Int4 hits_extended = 0;
+   Uint4 subject_offset, query_offset;
 
    ASSERT(diag != NULL);
 
@@ -131,28 +136,31 @@ Int2 BlastAaWordFinder_TwoHit(const BLAST_SequenceBlk* subject,
       /* scan the subject sequence for hits */
       if (rps_lookup)
          hits = BlastRPSScanSubject(lookup_wrap, subject, &first_offset, 
-                                query_offsets, subject_offsets, array_size);
+                                    offset_pairs, array_size);
       else
          hits = BlastAaScanSubject(lookup_wrap, subject, &first_offset, 
-                                query_offsets, subject_offsets, array_size);
+                                   offset_pairs, array_size);
 
       totalhits += hits;
       /* for each hit, */
       for (i = 0; i < hits; ++i)
       {
+         query_offset = offset_pairs[i].qs_offsets.q_off;
+         subject_offset = offset_pairs[i].qs_offsets.s_off;
+         
          /* calculate the diagonal associated with this query-subject
             pair, and find the distance to the last hit on this diagonal */
 
          diag_coord = 
-            (subject_offsets[i]  - query_offsets[i]) & diag_mask;
+            (query_offset  - subject_offset) & diag_mask;
          
          last_hit = diag_array[diag_coord].last_hit - diag_offset;
-         diff = subject_offsets[i] - last_hit;
+         diff = subject_offset - last_hit;
 
          if (diff >= window) {
             /* We are beyond the window for this diagonal; start a new hit */
-            diag_array[diag_coord].last_hit = subject_offsets[i] + diag_offset;
-         } 
+            diag_array[diag_coord].last_hit = subject_offset + diag_offset;
+         }
          else {
             /* If the difference is negative, or is less than the 
                wordsize (i.e. last hit and this hit overlap), give up */
@@ -165,7 +173,7 @@ Int2 BlastAaWordFinder_TwoHit(const BLAST_SequenceBlk* subject,
                sequences, do the ungapped extension */
 
             right_extend = TRUE;
-            if ((Int4)(subject_offsets[i] + diag_offset) > 
+            if ((Int4)(subject_offset + diag_offset) >= 
                 diag_array[diag_coord].diag_level) {
 
                /* Extend this pair of hits. The extension to the left must 
@@ -174,7 +182,7 @@ Int2 BlastAaWordFinder_TwoHit(const BLAST_SequenceBlk* subject,
                hsp_len = 0;
                score = BlastAaExtendTwoHit(matrix, subject, query,
                                         last_hit + wordsize, 
-                                        subject_offsets[i], query_offsets[i], 
+                                        subject_offset, query_offset, 
                                         dropoff, &hsp_q, &hsp_s, 
                                         &hsp_len, use_pssm,
                                         wordsize, &right_extend, &s_last_off);
@@ -184,7 +192,7 @@ Int2 BlastAaWordFinder_TwoHit(const BLAST_SequenceBlk* subject,
                /* if the hsp meets the score threshold, report it */
                if (score >= cutoff)
                   BlastSaveInitHsp(ungapped_hsps, hsp_q, hsp_s,
-                        query_offsets[i], subject_offsets[i], hsp_len, score);
+                        query_offset, subject_offset, hsp_len, score);
 
                /* whether or not the score was high enough, remember
                   the rightmost subject word offset examined. Future hits must
@@ -202,13 +210,13 @@ Int2 BlastAaWordFinder_TwoHit(const BLAST_SequenceBlk* subject,
                diag_array[diag_coord].last_hit = 0;
             else
                diag_array[diag_coord].last_hit = 
-                        subject_offsets[i] + diag_offset;
+                        subject_offset + diag_offset;
          } 
       }/* end for - done with this batch of hits, call scansubject again. */
    } /* end while - done with the entire sequence. */
  
    /* increment the offset in the diagonal array */
-   DiagUpdate(diag, subject->length + window); 
+   BlastDiagUpdate(diag, subject->length + window); 
  
    Blast_UngappedStatsUpdate(ungapped_stats, totalhits, hits_extended, 
                              ungapped_hsps->total);
@@ -222,8 +230,7 @@ Int2 BlastAaWordFinder_OneHit(const BLAST_SequenceBlk* subject,
 			      Int4 ** matrix,
 			      Int4 cutoff,
 			      Int4 dropoff,
-			      Uint4 * NCBI_RESTRICT query_offsets,
-			      Uint4 * NCBI_RESTRICT subject_offsets,
+                              BlastOffsetPair* NCBI_RESTRICT offset_pairs,
 			      Int4 array_size,
                BlastInitHitList* ungapped_hsps, 
                BlastUngappedStats* ungapped_stats)
@@ -268,41 +275,42 @@ Int2 BlastAaWordFinder_OneHit(const BLAST_SequenceBlk* subject,
       /* scan the subject sequence for hits */
       if (rps_lookup)
          hits = BlastRPSScanSubject(lookup_wrap, subject, &first_offset, 
-                                query_offsets, subject_offsets, array_size);
+                                    offset_pairs, array_size);
       else
          hits = BlastAaScanSubject(lookup_wrap, subject, &first_offset,
-				query_offsets, subject_offsets, array_size);
+				   offset_pairs, array_size);
 
       totalhits += hits;
       /* for each hit, */
       for (i = 0; i < hits; ++i) {
+         Uint4 subject_offset = offset_pairs[i].qs_offsets.s_off;
+         Uint4 query_offset = offset_pairs[i].qs_offsets.q_off;
          diag_coord = 
-            (subject_offsets[i]  - query_offsets[i]) & diag_mask;
-           
-         diff = subject_offsets[i] - 
+            (subject_offset  - query_offset) & diag_mask;
+         diff = subject_offset - 
             (diag_array[diag_coord].diag_level - diag_offset);
 
          /* do an extension, but only if we have not already extended
             this far */
-         if (diff > 0) {
+         if (diff >= 0) {
             ++hits_extended;
             score=BlastAaExtendOneHit(matrix, subject, query,
-                     subject_offsets[i], query_offsets[i], dropoff,
-                     &hsp_q, &hsp_s, &hsp_len, use_pssm, &s_last_off);
+                     subject_offset, query_offset, dropoff,
+                     &hsp_q, &hsp_s, &hsp_len, wordsize, use_pssm, &s_last_off);
 
             /* if the hsp meets the score threshold, report it */
             if (score >= cutoff) {
                BlastSaveInitHsp(ungapped_hsps, hsp_q, hsp_s, 
-                  query_offsets[i], subject_offsets[i], hsp_len, score);
+                  query_offset, subject_offset, hsp_len, score);
             }
             diag_array[diag_coord].diag_level = 
-                  s_last_off - (lookup->wordsize - 1) + diag_offset;
+                  s_last_off - (wordsize - 1) + diag_offset;
          }
       } /* end for */
    } /* end while */
 
    /* increment the offset in the diagonal array (no windows used) */
-   DiagUpdate(diag, subject->length); 
+   BlastDiagUpdate(diag, subject->length); 
  
    Blast_UngappedStatsUpdate(ungapped_stats, totalhits, hits_extended, 
                              ungapped_hsps->total);
@@ -315,7 +323,7 @@ Int4 BlastAaExtendRight(Int4 ** matrix,
 			Int4 s_off,
 			Int4 q_off,
 			Int4 dropoff,
-			Int4* displacement,
+			Int4* length,
                         Int4 maxscore,
                         Int4* s_last_off)
 {
@@ -336,11 +344,14 @@ Int4 BlastAaExtendRight(Int4 ** matrix,
         best_i = i;
      }
 
+      /* The comparison below is really >= and is different than the old code (e.g., blast.c:BlastWordExtend_prelim).
+         In the old code the loop continued as long as sum > X (X being negative).  The loop control here
+         is different and we *break out* when the if statement below is true. */
      if (score <= 0 || (maxscore - score) >= dropoff)
         break;
   }
 
-  *displacement = best_i;
+  *length = best_i + 1;
   *s_last_off = s_off + i;
   return maxscore;
 }
@@ -351,14 +362,14 @@ Int4 BlastAaExtendLeft(Int4 ** matrix,
 		       Int4 s_off,
 		       Int4 q_off,
 		       Int4 dropoff,
-		       Int4* displacement)
+		       Int4* length,
+                       Int4 maxscore)
 {
    Int4 i, n, best_i;
-   Int4 score = 0;
-   Int4 maxscore = 0;
+   Int4 score = maxscore;
    
    Uint1* s,* q;
-   
+
    n = MIN( s_off , q_off );
    best_i = n + 1;
 
@@ -372,11 +383,14 @@ Int4 BlastAaExtendLeft(Int4 ** matrix,
          maxscore = score;
          best_i = i;
       }
+      /* The comparison below is really >= and is different than the old code (e.g., blast.c:BlastWordExtend_prelim).
+         In the old code the loop continued as long as sum > X (X being negative).  The loop control here
+         is different and we *break out* when the if statement below is true. */
       if ((maxscore - score) >= dropoff)
          break;
    }
-   
-   *displacement = n - best_i;
+
+   *length = n - best_i + 1;
    return maxscore;
 }
 
@@ -386,7 +400,7 @@ Int4 BlastPSSMExtendRight(Int4 ** matrix,
 			Int4 s_off,
 			Int4 q_off,
 			Int4 dropoff,
-			Int4* displacement,
+			Int4* length,
                         Int4 maxscore,
 			Int4* s_last_off)
 {
@@ -405,11 +419,14 @@ Int4 BlastPSSMExtendRight(Int4 ** matrix,
         best_i = i;
      }
 
+      /* The comparison below is really >= and is different than the old code (e.g., blast.c:BlastWordExtend_prelim).
+         In the old code the loop continued as long as sum > X (X being negative).  The loop control here
+         is different and we *break out* when the if statement below is true. */
      if (score <= 0 || (maxscore - score) >= dropoff)
         break;
   }
 
-  *displacement = best_i;
+  *length = best_i + 1;
   *s_last_off = s_off + i;
   return maxscore;
 }
@@ -419,11 +436,11 @@ Int4 BlastPSSMExtendLeft(Int4 ** matrix,
 		       Int4 s_off,
 		       Int4 q_off,
 		       Int4 dropoff,
-		       Int4* displacement)
+		       Int4* length,
+                       Int4 maxscore)
 {
    Int4 i, n, best_i;
-   Int4 score = 0;
-   Int4 maxscore = 0;
+   Int4 score = maxscore;
    Uint1* s;
    
    n = MIN( s_off , q_off );
@@ -437,11 +454,14 @@ Int4 BlastPSSMExtendLeft(Int4 ** matrix,
          maxscore = score;
          best_i = i;
       }
+      /* The comparison below is really >= and is different than the old code (e.g., blast.c:BlastWordExtend_prelim).
+         In the old code the loop continued as long as sum > X (X being negative).  The loop control here
+         is different and we *break out* when the if statement below is true. */
       if ((maxscore - score) >= dropoff)
          break;
    }
    
-   *displacement = n - best_i;
+   *length = n - best_i + 1;
    return maxscore;
 }
 
@@ -454,32 +474,67 @@ Int4 BlastAaExtendOneHit(Int4 ** matrix,
 			 Int4* hsp_q,
 			 Int4* hsp_s,
 			 Int4* hsp_len,
+                         Int4 word_size,
                          Boolean use_pssm,
                          Int4* s_last_off)
 {
-  Int4 left_score, right_score;
-  Int4 left_disp, right_disp;
+  Int4 score=0, left_score, total_score, sum=0;
+  Int4 left_disp=0, right_disp=0;
+  Int4 q_left_off=q_off, q_right_off=q_off+word_size, q_best_left_off=q_off; 
+  Int4 s_left_off, s_right_off;
+  Int4 init_hit_width = 0;
+  Int4 i; /* loop variable. */
+  Uint1 *q = query->sequence;
+  Uint1 *s = subject->sequence;
+
+  for (i = 0; i < word_size; i++) {
+      if (use_pssm)
+         sum += matrix[ q_off+i ][ s[s_off+i] ];
+      else
+         sum += matrix[ q[q_off+i] ][ s[s_off+i] ];
+
+      if (sum > score)
+      {
+           score = sum;
+           q_best_left_off = q_left_off;
+           q_right_off = q_off + i;
+      }
+      else if (sum <= 0)
+      {
+           sum = 0;
+           q_left_off = q_off + i + 1;
+      }
+  }
+
+  init_hit_width = q_right_off - q_left_off + 1;
+
+  q_left_off = q_best_left_off;  
+
+  s_left_off = q_left_off + (s_off-q_off);
+  s_right_off = q_right_off + (s_off-q_off);
 
   if (use_pssm) {
      left_score = BlastPSSMExtendLeft(matrix, subject, 
-                           s_off, q_off, dropoff, &left_disp);
-     right_score = BlastPSSMExtendRight(matrix, subject, query->length, 
-                           s_off+1, q_off+1, dropoff, &right_disp, 
+                           s_left_off-1, q_left_off-1, dropoff, &left_disp, score);
+
+     total_score = BlastPSSMExtendRight(matrix, subject, query->length, 
+                           s_right_off+1, q_right_off+1, dropoff, &right_disp, 
                            left_score, s_last_off);
   }
   else {
      left_score = BlastAaExtendLeft(matrix, subject, query, 
-                            s_off, q_off, dropoff, &left_disp);
-     right_score = BlastAaExtendRight(matrix, subject, query, 
-                            s_off+1, q_off+1, dropoff, &right_disp, 
+                            s_left_off-1, q_left_off-1, dropoff, &left_disp, score);
+
+     total_score = BlastAaExtendRight(matrix, subject, query, 
+                            s_right_off+1, q_right_off+1, dropoff, &right_disp, 
                             left_score, s_last_off);
   }
   
-  *hsp_q   = q_off - left_disp;
-  *hsp_s   = s_off - left_disp;
-  *hsp_len = left_disp + right_disp + 2;
-  
-  return right_score;
+  *hsp_q   = q_left_off - left_disp;
+  *hsp_s   = s_left_off - left_disp;
+  *hsp_len = left_disp + right_disp + init_hit_width;
+
+  return total_score;
 }
 
 Int4 
@@ -504,8 +559,10 @@ BlastAaExtendTwoHit(Int4 ** matrix,
    Uint1 *s = subject->sequence;
    Uint1 *q = query->sequence;
 
-   /* find the position (up to word_size-1 letters to the
+   /* find one beyond the position (up to word_size-1 letters to the
       right) that gives the largest starting score */
+  /* Use "one beyond" to make the numbering consistent with how it's done
+     for BlastAaExtendOneHit and the "Extend" functions called here. */
    for (i = 0; i < word_size; i++) {
       if (use_pssm)
          score += matrix[ q_right_off+i ][ s[s_right_off+i] ];
@@ -514,23 +571,23 @@ BlastAaExtendTwoHit(Int4 ** matrix,
 
       if (score > left_score) {
          left_score = score;
-         right_d = i;
+         right_d = i + 1; /* Position is one beyond the end of the word. */
       }
    }
    q_right_off += right_d;
    s_right_off += right_d;
 
-   right_d = -1;
+   right_d = 0;
    *right_extend = FALSE;
    *s_last_off = s_right_off;
 
    /* first, try to extend left, from the second hit to the first hit. */
    if (use_pssm)
       left_score = BlastPSSMExtendLeft(matrix, subject,
-    			     s_right_off, q_right_off, dropoff, &left_d);
+    			     s_right_off-1, q_right_off-1, dropoff, &left_d, 0);
    else
       left_score = BlastAaExtendLeft(matrix, subject, query,
-    			     s_right_off, q_right_off, dropoff, &left_d);
+    			     s_right_off-1, q_right_off-1, dropoff, &left_d, 0);
 
    /* Extend to the right only if left extension reached the first hit. */
    if (left_d >= (s_right_off - s_left_off)) {
@@ -538,28 +595,29 @@ BlastAaExtendTwoHit(Int4 ** matrix,
       *right_extend = TRUE;
       if (use_pssm)
          right_score = BlastPSSMExtendRight(matrix, subject, query->length,
-                                     s_right_off + 1, q_right_off + 1,
+                                     s_right_off, q_right_off,
                                      dropoff, &right_d, left_score, s_last_off);
       else 
          right_score = BlastAaExtendRight(matrix, subject, query,
-                                     s_right_off + 1, q_right_off + 1,
+                                     s_right_off, q_right_off,
                                      dropoff, &right_d, left_score, s_last_off);
    }
 
+
    *hsp_q = q_right_off - left_d;
    *hsp_s = s_right_off - left_d;
-   *hsp_len = left_d + right_d + 2;
+   *hsp_len = left_d + right_d;
    return MAX(left_score,right_score);
 }
 
-Int4 DiagUpdate(BLAST_DiagTable* diag, Int4 length)
+Int4 BlastDiagUpdate(BLAST_DiagTable* diag, Int4 length)
 {
   if (diag == NULL)
     return 0;
 
   if (diag->offset >= INT4_MAX/2)
     {
-      DiagClear(diag);
+      BlastDiagClear(diag);
     }
   else
     {
@@ -569,7 +627,7 @@ Int4 DiagUpdate(BLAST_DiagTable* diag, Int4 length)
 }
 
 
-Int4 DiagClear(BLAST_DiagTable* diag)
+Int4 BlastDiagClear(BLAST_DiagTable* diag)
 {
   Int4 i,n;
   DiagStruct* diag_struct_array;
@@ -579,6 +637,8 @@ Int4 DiagClear(BLAST_DiagTable* diag)
 
   n=diag->diag_array_length;
 
+  diag->offset = diag->window;
+
   diag_struct_array = diag->hit_level_array;
 
   for(i=0;i<n;i++) {
