@@ -1,4 +1,4 @@
-/* $Id: blast_diagnostics.c,v 1.5 2004/11/02 18:19:22 madden Exp $
+/* $Id: blast_diagnostics.c,v 1.8 2005/11/16 14:27:03 madden Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -6,7 +6,7 @@
  *
  *  This software/database is a "United States Government Work" under the
  *  terms of the United States Copyright Act.  It was written as part of
- *  the author's offical duties as a United States Government employee and
+ *  the author's official duties as a United States Government employee and
  *  thus cannot be copyrighted.  This software/database is freely available
  *  to the public for use. The National Library of Medicine and the U.S.
  *  Government have not placed any restriction on its use or reproduction.
@@ -34,7 +34,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: blast_diagnostics.c,v 1.5 2004/11/02 18:19:22 madden Exp $";
+    "$Id: blast_diagnostics.c,v 1.8 2005/11/16 14:27:03 madden Exp $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <algo/blast/core/blast_diagnostics.h>
@@ -94,6 +94,9 @@ void Blast_UngappedStatsUpdate(BlastUngappedStats* ungapped_stats,
 void 
 Blast_DiagnosticsUpdate(BlastDiagnostics* global, BlastDiagnostics* local)
 {
+    if (!local)
+        return;
+
    if (global->mt_lock) 
       MT_LOCK_Do(global->mt_lock, eMT_Lock);
 
@@ -128,6 +131,7 @@ Blast_DiagnosticsUpdate(BlastDiagnostics* global, BlastDiagnostics* local)
       global->cutoffs->x_drop_gap = local->cutoffs->x_drop_gap;
       global->cutoffs->x_drop_gap_final = local->cutoffs->x_drop_gap_final;
       global->cutoffs->ungapped_cutoff = local->cutoffs->ungapped_cutoff;
+      global->cutoffs->cutoff_score = local->cutoffs->cutoff_score;
    }
 
    if (global->mt_lock) 
