@@ -1,4 +1,4 @@
-/* $Id: blast_engine.c,v 1.189 2005/04/27 19:51:37 dondosha Exp $
+/* $Id: blast_engine.c,v 1.203 2005/11/22 13:44:13 coulouri Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -6,7 +6,7 @@
  *
  *  This software/database is a "United States Government Work" under the
  *  terms of the United States Copyright Act.  It was written as part of
- *  the author's offical duties as a United States Government employee and
+ *  the author's official duties as a United States Government employee and
  *  thus cannot be copyrighted.  This software/database is freely available
  *  to the public for use. The National Library of Medicine and the U.S.
  *  Government have not placed any restriction on its use or reproduction.
@@ -23,8 +23,6 @@
  *
  * ===========================================================================
  *
- * Author: Ilya Dondoshansky
- *
  */
 
 /** @file blast_engine.c
@@ -57,7 +55,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: blast_engine.c,v 1.189 2005/04/27 19:51:37 dondosha Exp $";
+    "$Id: blast_engine.c,v 1.203 2005/11/22 13:44:13 coulouri Exp $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <algo/blast/core/blast_engine.h>
@@ -73,6 +71,11 @@ static char const rcsid[] =
 #include <algo/blast/core/phi_gapalign.h>
 #include <algo/blast/core/phi_lookup.h>
 
+NCBI_XBLAST_EXPORT const int   kBlastMajorVersion = 2;
+NCBI_XBLAST_EXPORT const int   kBlastMinorVersion = 2;
+NCBI_XBLAST_EXPORT const int   kBlastPatchVersion = 13;
+NCBI_XBLAST_EXPORT const char* kBlastReleaseDate = "Nov-27-2005";
+
 /** Structure to be passed to s_BlastSearchEngineCore, containing pointers 
     to various preallocated structures and arrays. */
 typedef struct BlastCoreAuxStruct {
@@ -248,9 +251,7 @@ s_BlastSearchEngineCore(EBlastProgramType program_number, BLAST_SequenceBlk* que
       (ext_params->options->ePrelimGapExt == eGreedyWithTracebackExt);
 
    const Boolean kTranslatedSubject = 
-       (program_number == eBlastTypeTblastn ||
-        program_number == eBlastTypeTblastx ||
-        program_number == eBlastTypeRpsTblastn);
+        (Blast_SubjectIsTranslated(program_number) || program_number == eBlastTypeRpsTblastn);
    const Boolean kNucleotide = (program_number == eBlastTypeBlastn ||
                                 program_number == eBlastTypePhiBlastn);
 
@@ -258,7 +259,7 @@ s_BlastSearchEngineCore(EBlastProgramType program_number, BLAST_SequenceBlk* que
       first_context = 0;
       last_context = 5;
       if (score_options->is_ooframe) {
-         BLAST_GetAllTranslations(orig_sequence, NCBI2NA_ENCODING,
+         BLAST_GetAllTranslations(orig_sequence, eBlastEncodingNcbi2na,
             orig_length, db_options->gen_code_string, &translation_buffer,
             &frame_offsets, &subject->oof_sequence);
          subject->oof_sequence_allocated = TRUE;
@@ -270,7 +271,7 @@ s_BlastSearchEngineCore(EBlastProgramType program_number, BLAST_SequenceBlk* que
           frame_offsets_a = frame_offsets =
               ContextOffsetsToOffsetArray(query_info_in);
       } else {
-         BLAST_GetAllTranslations(orig_sequence, NCBI2NA_ENCODING,
+         BLAST_GetAllTranslations(orig_sequence, eBlastEncodingNcbi2na,
             orig_length, db_options->gen_code_string, &translation_buffer,
             &frame_offsets, NULL);
          frame_offsets_a = frame_offsets;
@@ -298,8 +299,7 @@ s_BlastSearchEngineCore(EBlastProgramType program_number, BLAST_SequenceBlk* que
    }
 
    /* Substitute query info by concatenated database info for RPS BLAST search */
-   if (program_number == eBlastTypeRpsBlast || 
-       program_number == eBlastTypeRpsTblastn) {
+   if (Blast_ProgramIsRpsBlast(program_number)) {
       BlastRPSLookupTable* lut = (BlastRPSLookupTable*) lookup->lut;
       query_info = (BlastQueryInfo*) calloc(1, sizeof(BlastQueryInfo));
       query_info->num_queries = lut->num_profiles;
@@ -377,6 +377,16 @@ s_BlastSearchEngineCore(EBlastProgramType program_number, BLAST_SequenceBlk* que
             aux_struct->GetGappedScore(program_number, query, query_info, 
                subject, gap_align, score_params, ext_params, hit_params, 
                init_hitlist, &hsp_list, gapped_stats);
+
+            /* Removes redundant HSPs. */
+             Blast_HSPListPurgeHSPsWithCommonEndpoints(program_number, hsp_list);
+
+             /* For nucleotide search, if match score is = 2, the odd scores
+                are rounded down to the nearest even number. */
+             Blast_HSPListAdjustOddBlastnScores(hsp_list, score_options->gapped_calculation, gap_align->sbp);
+
+             Blast_HSPListSortByScore(hsp_list);
+
             if (score_options->is_ooframe && kTranslatedSubject)
                subject->length = prot_length;
          } else {
@@ -417,10 +427,8 @@ s_BlastSearchEngineCore(EBlastProgramType program_number, BLAST_SequenceBlk* que
       status = BLAST_LinkHsps(program_number, hsp_list, query_info,
                   subject->length, gap_align->sbp, hit_params->link_hsp_params, 
                   score_options->gapped_calculation);
-   } else if (program_number != eBlastTypePhiBlastn &&
-              program_number != eBlastTypePhiBlastp &&
-              program_number != eBlastTypeRpsBlast &&
-          program_number != eBlastTypeRpsTblastn) {
+   } else if (!Blast_ProgramIsPhiBlast(program_number) &&
+              !Blast_ProgramIsRpsBlast(program_number)) {
        /* Calculate e-values for all HSPs. Skip this step
           for PHI and RPS BLAST, since calculating the E values 
           requires precomputation that has not been done yet */
@@ -510,7 +518,7 @@ s_FillReturnCutoffsInfo(BlastRawCutoffs* return_cutoffs,
  */
 static Int2 
 s_BlastSetUpAuxStructures(const BlastSeqSrc* seq_src,
-   LookupTableWrap* lookup_wrap,	
+   LookupTableWrap* lookup_wrap,    
    const BlastInitialWordParameters* word_params,
    const BlastExtensionOptions* ext_options,
    const BlastHitSavingOptions* hit_options,
@@ -608,8 +616,7 @@ s_RPSPreliminarySearchEngine(EBlastProgramType program_number,
    BLAST_SequenceBlk* one_query = NULL;
    Int4 index;
 
-   if (program_number != eBlastTypeRpsBlast &&
-       program_number != eBlastTypeRpsTblastn)
+   if ( !Blast_ProgramIsRpsBlast(program_number))
       return -1;
 
    /* modify scoring and gap alignment structures for
@@ -734,8 +741,7 @@ BLAST_PreliminarySearchEngine(EBlastProgramType program_number,
 
    /* For RPS BLAST, there is no loop over subject sequences, so the preliminary
       search engine is done in a separate function. */
-   if (program_number == eBlastTypeRpsBlast || 
-       program_number == eBlastTypeRpsTblastn) {
+   if (Blast_ProgramIsRpsBlast(program_number)) {
       status =         
          s_RPSPreliminarySearchEngine(program_number, query, query_info, 
             seq_src, score_params, lookup_wrap, aux_struct, word_params, 
@@ -754,7 +760,7 @@ BLAST_PreliminarySearchEngine(EBlastProgramType program_number,
 
    /* Encoding is set so there are no sentinel bytes, and protein/nucleotide
       sequences are retieved in ncbistdaa/ncbi2na encodings respectively. */
-   seq_arg.encoding = BLASTP_ENCODING; 
+   seq_arg.encoding = eBlastEncodingProtein; 
 
    db_length = BlastSeqSrcGetTotLen(seq_src);
 
@@ -793,7 +799,6 @@ BLAST_PreliminarySearchEngine(EBlastProgramType program_number,
             seq_arg.seq, lookup_wrap, gap_align, score_params, word_params, 
             ext_params, hit_params, db_options, diagnostics, aux_struct, 
             &hsp_list);
- 
       if (status)
          break;
 
@@ -833,7 +838,9 @@ BLAST_PreliminarySearchEngine(EBlastProgramType program_number,
          } 
          
          /* Save the results. */
-         BlastHSPStreamWrite(hsp_stream, &hsp_list);
+         status = BlastHSPStreamWrite(hsp_stream, &hsp_list);
+         if (status != 0)
+            break;
       }
       
       BlastSeqSrcReleaseSequence(seq_src, (void*) &seq_arg);
@@ -855,15 +862,20 @@ BLAST_PreliminarySearchEngine(EBlastProgramType program_number,
 
 Int2 
 Blast_RunPreliminarySearch(EBlastProgramType program, 
-   BLAST_SequenceBlk* query, BlastQueryInfo* query_info, 
-   const BlastSeqSrc* seq_src, const BlastScoringOptions* score_options,
-   BlastScoreBlk* sbp, LookupTableWrap* lookup_wrap,
+   BLAST_SequenceBlk* query, 
+   BlastQueryInfo* query_info, 
+   const BlastSeqSrc* seq_src, 
+   const BlastScoringOptions* score_options,
+   BlastScoreBlk* sbp, 
+   LookupTableWrap* lookup_wrap,
    const BlastInitialWordOptions* word_options, 
    const BlastExtensionOptions* ext_options,
    const BlastHitSavingOptions* hit_options,
    const BlastEffectiveLengthsOptions* eff_len_options,
-   const PSIBlastOptions* psi_options, const BlastDatabaseOptions* db_options, 
-   BlastHSPStream* hsp_stream, BlastDiagnostics* diagnostics)
+   const PSIBlastOptions* psi_options, 
+   const BlastDatabaseOptions* db_options, 
+   BlastHSPStream* hsp_stream, 
+   BlastDiagnostics* diagnostics)
 {
    Int2 status = 0;
    BlastScoringParameters* score_params = NULL;/**< Scoring parameters */
@@ -951,8 +963,7 @@ Blast_RunFullSearch(EBlastProgramType program_number,
    /* Prohibit any subsequent writing to the HSP stream. */
    BlastHSPStreamClose(hsp_stream);
 
-   if (program_number == eBlastTypePhiBlastn || 
-       program_number == eBlastTypePhiBlastp) {
+   if (Blast_ProgramIsPhiBlast(program_number)) {
        PHIPatternSpaceCalc(query_info, diagnostics);
        pattern_blk = ((SPHIPatternSearchBlk*) lookup_wrap->lut);
    } 
