@@ -1,4 +1,4 @@
-/* $Id: blast_message.c,v 1.17 2005/02/07 15:18:39 bealer Exp $
+/* $Id: blast_message.c,v 1.19 2005/11/16 14:27:03 madden Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -6,7 +6,7 @@
  *
  *  This software/database is a "United States Government Work" under the
  *  terms of the United States Copyright Act.  It was written as part of
- *  the author's offical duties as a United States Government employee and
+ *  the author's official duties as a United States Government employee and
  *  thus cannot be copyrighted.  This software/database is freely available
  *  to the public for use. The National Library of Medicine and the U.S.
  *  Government have not placed any restriction on its use or reproduction.
@@ -31,7 +31,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: blast_message.c,v 1.17 2005/02/07 15:18:39 bealer Exp $";
+    "$Id: blast_message.c,v 1.19 2005/11/16 14:27:03 madden Exp $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <algo/blast/core/blast_message.h>
@@ -49,7 +49,7 @@ Blast_MessageFree(Blast_Message* blast_msg)
 }
 
 Int2 
-Blast_MessageWrite(Blast_Message* *blast_msg, BlastSeverity severity, 
+Blast_MessageWrite(Blast_Message* *blast_msg, EBlastSeverity severity, 
                    Int4 code,	Int4 subcode, const char *message)
 {
 	if (blast_msg == NULL)
@@ -85,13 +85,13 @@ Blast_Perror(Int2 error_code)
     case BLASTERR_IDEALSTATPARAMCALC:
         retval->message = strdup("Failed to calculate ideal Karlin-Altschul "
                                  "parameters");
-        retval->severity = BLAST_SEV_ERROR;
+        retval->severity = eBlastSevError;
         break;
     case BLASTERR_REDOALIGNMENTCORE_NOTSUPPORTED:
         retval->message = strdup("Composition based statistics or "
                                  "Smith-Waterman not supported for your "
                                  "program type");
-        retval->severity = BLAST_SEV_ERROR;
+        retval->severity = eBlastSevError;
         break;
     case 0:
         retval = Blast_MessageFree(retval);
@@ -101,7 +101,7 @@ Blast_Perror(Int2 error_code)
             char buf[512];
             snprintf(buf, sizeof(buf) - 1, "Unknown error code %d", error_code);
             retval->message = strdup(buf);
-            retval->severity = BLAST_SEV_ERROR;
+            retval->severity = eBlastSevError;
         }
         break;
     }
@@ -113,6 +113,12 @@ Blast_Perror(Int2 error_code)
  * ===========================================================================
  *
  * $Log: blast_message.c,v $
+ * Revision 1.19  2005/11/16 14:27:03  madden
+ * Fix spelling in CRN
+ *
+ * Revision 1.18  2005/06/20 13:09:36  madden
+ * Rename BlastSeverity enums in line with C++ tookit convention
+ *
  * Revision 1.17  2005/02/07 15:18:39  bealer
  * - Fix doxygen file-level comments.
  *
