@@ -1,4 +1,4 @@
-/* $Id: blast_options.h,v 1.117 2005/04/27 19:48:34 dondosha Exp $
+/* $Id: blast_options.h,v 1.125 2005/11/29 17:27:40 camacho Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -6,7 +6,7 @@
  *
  *  This software/database is a "United States Government Work" under the
  *  terms of the United States Copyright Act.  It was written as part of
- *  the author's offical duties as a United States Government employee and
+ *  the author's official duties as a United States Government employee and
  *  thus cannot be copyrighted.  This software/database is freely available
  *  to the public for use. The National Library of Medicine and the U.S.
  *  Government have not placed any restriction on its use or reproduction.
@@ -144,9 +144,6 @@ extern "C" {
                                      value exceeds this number are discarded */
 #define BLAST_HITLIST_SIZE 500 /**< Number of database sequences to save hits 
                                   for */
-#define BLAST_PRELIM_HITLIST_SIZE 500 /**< Number of database sequences to save 
-                                         hits for in the preliminary stage. */
-
 /** Types of the lookup table */
 #define MB_LOOKUP_TABLE 1  /**< megablast lookup table (includes both
                                 contiguous and discontiguous megablast) */
@@ -165,6 +162,10 @@ extern "C" {
                                    groups of three nucleotide bases to protein
                                    letters */
 
+/** Value used to indicate that no IMPALA-style scaling should be performed
+ * when scaling a PSSM */
+extern const double kPSSM_NoImpalaScaling;
+
 /** Options needed to construct a lookup table 
  * Also needed: query sequence and query length.
  */
@@ -182,7 +183,6 @@ typedef struct LookupTableOptions {
    char* phi_pattern;  /**< PHI-BLAST pattern */
    Int4 max_num_patterns; /**< Maximal number of patterns allowed for 
                              PHI-BLAST */
-   Boolean use_pssm; /**< Use a PSSM rather than a (protein) query to construct lookup table */
    Boolean variable_wordsize; /**< Should the partial bytes be examined for 
                              determining whether exact match is long enough? */
 } LookupTableOptions;
@@ -308,8 +308,6 @@ typedef struct BlastHitSavingOptions {
 
    Int4 hitlist_size;/**< Maximal number of database sequences to return
                         results for */
-   Int4 prelim_hitlist_size; /**< Maximal number of database sequences to 
-                               save hits after preliminary alignment */
    Int4 hsp_num_max; /**< Maximal number of HSPs to save for one database 
                         sequence */
    Int4 total_hsp_limit; /**< Maximal total number of HSPs to keep */
@@ -318,12 +316,6 @@ typedef struct BlastHitSavingOptions {
    Int4 culling_limit; /**< If the query range of an HSP is contained in
                             at least this many higher-scoring HSPs, throw
                             away the HSP as redundant (turned off if zero) */
-   /* PSI-BLAST Hit saving options */
-   Int4 required_start;  /**< Start of the region required to be part of the
-                            alignment */
-   Int4 required_end;    /**< End of the region required to be part of the
-                            alignment */
-   double original_expect_value; /**< Needed for PSI-BLAST??? */
 
    /********************************************************************/
    /* Merge all these in a structure for clarity? */
@@ -351,7 +343,7 @@ typedef struct BlastScoringOptions {
    char* matrix_path; /**< Directory path to where matrices are stored. */
    Int2 reward;      /**< Reward for a match */
    Int2 penalty;     /**< Penalty for a mismatch */
-   Boolean gapped_calculation; /**< identical to the one in hit saving opts */
+   Boolean gapped_calculation; /**< gap-free search if FALSE */
    Int4 gap_open;    /**< Extra penalty for starting a gap */
    Int4 gap_extend;  /**< Penalty for each gap residue */
    Int4 decline_align; /**< Cost for declining alignment (PSI-BLAST) */
@@ -738,13 +730,12 @@ Int2 LookupTableOptionsNew(EBlastProgramType program, LookupTableOptions* *optio
  * @param word_size Number of matched residues in an initial word [in]
  * @param variable_wordsize Are only full bytes of a compressed sequence 
  *        checked to find initial words? [in]
- * @param use_pssm Use PSSM rather than (protein) query to build lookup table.
  */
 NCBI_XBLAST_EXPORT
 Int2 
 BLAST_FillLookupTableOptions(LookupTableOptions* options, 
    EBlastProgramType program, Boolean is_megablast, Int4 threshold,
-   Int4 word_size, Boolean variable_wordsize, Boolean use_pssm);
+   Int4 word_size, Boolean variable_wordsize);
 
 
 /** Deallocates memory for LookupTableOptions*.
@@ -874,6 +865,30 @@ Int2 BLAST_ValidateOptions(EBlastProgramType program_number,
                            Blast_Message* *blast_msg);
 
 
+
+/** Get thresholds for word-finding suggested by Stephen Altschul.
+ *
+ * @param program_number Type of blast program: blastn, blastp, blastx, 
+ *                      tblastn, tblastx) [in]
+ * @param matrixName matrix, e.g., BLOSUM62 [in]
+ * @param threshold returns suggested value [in|out]
+ * @return zero on success
+ */
+Int2 BLAST_GetSuggestedThreshold(EBlastProgramType program_number, 
+                                 const char* matrixName, 
+                                 Int4* threshold);
+
+/** Get window sizes for two hit algorithm suggested by Stephen Altschul.
+ *
+ * @param program_number Type of blast program: blastn, blastp, blastx, 
+ *                      tblastn, tblastx) [in]
+ * @param matrixName matrix, e.g., BLOSUM62 [in]
+ * @param window_size returns suggested value [in|out]
+ * @return zero on success
+ */
+Int2 BLAST_GetSuggestedWindowSize(EBlastProgramType program_number, 
+                                 const char* matrixName, 
+                                 Int4* window_size);
 #ifdef __cplusplus
 }
 #endif
