@@ -1,4 +1,4 @@
-/*  $Id: blast_seqsrc.c,v 1.23 2004/10/06 14:48:25 dondosha Exp $
+/*  $Id: blast_seqsrc.c,v 1.30 2005/04/18 16:11:43 camacho Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -29,15 +29,24 @@
  */
 
 /** @file blast_seqsrc.c
- * Definition of ADT to retrieve sequences for the BLAST engine
+ * Definition of ADT to retrieve sequences for the BLAST engine and
+ * low level details of the implementation of the BlastSeqSrc framework.
  */
 
+#ifndef SKIP_DOXYGEN_PROCESSING
+#ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: blast_seqsrc.c,v 1.23 2004/10/06 14:48:25 dondosha Exp $";
+    "$Id: blast_seqsrc.c,v 1.30 2005/04/18 16:11:43 camacho Exp $";
+#endif /* SKIP_DOXYGEN_PROCESSING */
+#endif
 
 #include <algo/blast/core/blast_seqsrc.h>
+#include <algo/blast/core/blast_seqsrc_impl.h>
 
-/** Complete type definition of Blast Sequence Source ADT */
+/** Complete type definition of Blast Sequence Source ADT.
+ * The members of this structure should only be accessed by BlastSeqSrc
+ * implementations using the _BlastSeqSrcImpl_* functions.
+ */
 struct BlastSeqSrc {
 
     BlastSeqSrcConstructor NewFnPtr;       /**< Constructor */
@@ -54,19 +63,18 @@ struct BlastSeqSrc {
     GetBoolFnPtr      GetIsProt;      /**< Find if database is a protein or 
                                          nucleotide */
 
-   /* Functions to get information about individual sequences */
+   /* Functions that deal with individual sequences */
     GetSeqBlkFnPtr    GetSequence;    /**< Retrieve individual sequence */
     GetInt4FnPtr      GetSeqLen;      /**< Retrieve given sequence length */
+    ReleaseSeqBlkFnPtr ReleaseSequence; /**< Deallocate individual sequence 
+                                         (if applicable) */
 
    /* Functions to iterate over sequences in the database */
-    GetNextChunkFnPtr GetNextChunk;   /**< Get next chunk of seq indices */
     AdvanceIteratorFnPtr IterNext;    /**< Gets next oid from the iterator */
-    GetErrorFnPtr       GetError;       /**< Gets a saved error message, if
-                                         supported. */
-    GetSeqBlkFnPtr    RetSequence;    /**< Deallocate individual sequence 
-                                         buffer if necessary. */
    
     void*             DataStructure;  /**< ADT holding the sequence data */
+
+    char*             InitErrorStr;   /**< initialization error string */
 };
 
 BlastSeqSrc* BlastSeqSrcNew(const BlastSeqSrcNewInfo* bssn_info)
@@ -74,18 +82,18 @@ BlastSeqSrc* BlastSeqSrcNew(const BlastSeqSrcNewInfo* bssn_info)
     BlastSeqSrc* retval = NULL;
 
     if (!bssn_info) {
-        return NULL;
+        return (BlastSeqSrc*) NULL;
     }
 
     if ( !(retval = (BlastSeqSrc*) calloc(1, sizeof(BlastSeqSrc)))) {
-        return NULL;
+        return (BlastSeqSrc*) NULL;
     }
 
     /* Save the constructor and invoke it */
     if ((retval->NewFnPtr = bssn_info->constructor)) {
         retval = (*retval->NewFnPtr)(retval, bssn_info->ctor_argument);
     } else {
-      sfree(retval);
+        sfree(retval);
     }
 
     return retval;
@@ -99,11 +107,15 @@ BlastSeqSrc* BlastSeqSrcFree(BlastSeqSrc* seq_src)
         return (BlastSeqSrc*) NULL;
     }
 
+    if (seq_src->InitErrorStr) {
+        sfree(seq_src->InitErrorStr);
+    }
+
     /* This could leave a memory leak if destructor function pointer is not
      * initialized! It is the implementation's resposibility to provide this */
     if ( !(destructor_fnptr = (*seq_src->DeleteFnPtr))) {
         sfree(seq_src);
-        return NULL;
+        return (BlastSeqSrc*) NULL;
     }
 
     return (BlastSeqSrc*) (*destructor_fnptr)(seq_src);
@@ -112,14 +124,14 @@ BlastSeqSrc* BlastSeqSrcFree(BlastSeqSrc* seq_src)
 BlastSeqSrc* BlastSeqSrcCopy(const BlastSeqSrc* seq_src)
 {
     BlastSeqSrcCopier copy_fnptr = NULL;
-    BlastSeqSrc* retval;
+    BlastSeqSrc* retval = NULL;
 
     if (!seq_src) {
-        return NULL;
+        return (BlastSeqSrc*) NULL;
     }
 
     if ( !(retval = (BlastSeqSrc*) BlastMemDup(seq_src, sizeof(BlastSeqSrc)))) {
-        return NULL;
+        return (BlastSeqSrc*) NULL;
     }
 
     /* If copy function is not provided, just return a copy of the structure */
@@ -130,12 +142,101 @@ BlastSeqSrc* BlastSeqSrcCopy(const BlastSeqSrc* seq_src)
     return (BlastSeqSrc*) (*copy_fnptr)(retval);
 }
 
-/** How many database sequences to process in one database chunk. */
-const unsigned int kBlastSeqSrcDefaultChunkSize = 1024;
+char* BlastSeqSrcGetInitError(const BlastSeqSrc* seq_src)
+{
+    if ( !seq_src || !seq_src->InitErrorStr ) {
+        return NULL;
+    }
+
+    return strdup(seq_src->InitErrorStr);
+}
+
+Int4
+BlastSeqSrcGetNumSeqs(const BlastSeqSrc* seq_src)
+{
+    ASSERT(seq_src);
+    ASSERT(seq_src->GetNumSeqs);
+    return (*seq_src->GetNumSeqs)(seq_src->DataStructure, NULL);
+}
+
+Int4
+BlastSeqSrcGetMaxSeqLen(const BlastSeqSrc* seq_src)
+{
+    ASSERT(seq_src);
+    ASSERT(seq_src->GetMaxSeqLen);
+    return (*seq_src->GetMaxSeqLen)(seq_src->DataStructure, NULL);
+}
+
+Int4
+BlastSeqSrcGetAvgSeqLen(const BlastSeqSrc* seq_src)
+{
+    ASSERT(seq_src);
+    ASSERT(seq_src->GetAvgSeqLen);
+    return (*seq_src->GetAvgSeqLen)(seq_src->DataStructure, NULL);
+}
+
+Int8
+BlastSeqSrcGetTotLen(const BlastSeqSrc* seq_src)
+{
+    ASSERT(seq_src);
+    ASSERT(seq_src->GetTotLen);
+    return (*seq_src->GetTotLen)(seq_src->DataStructure, NULL);
+}
+
+const char*
+BlastSeqSrcGetName(const BlastSeqSrc* seq_src)
+{
+    ASSERT(seq_src);
+    ASSERT(seq_src->GetName);
+    return (*seq_src->GetName)(seq_src->DataStructure, NULL);
+}
+
+Boolean
+BlastSeqSrcGetIsProt(const BlastSeqSrc* seq_src)
+{
+    ASSERT(seq_src);
+    ASSERT(seq_src->GetIsProt);
+    return (*seq_src->GetIsProt)(seq_src->DataStructure, NULL);
+}
+
+Int2
+BlastSeqSrcGetSequence(const BlastSeqSrc* seq_src, 
+                       void* sequence)
+{
+    ASSERT(seq_src);
+    ASSERT(seq_src->GetSequence);
+    ASSERT(sequence);
+    return (*seq_src->GetSequence)(seq_src->DataStructure, sequence);
+}
+
+Int4
+BlastSeqSrcGetSeqLen(const BlastSeqSrc* seq_src, void* oid)
+{
+    ASSERT(seq_src);
+    ASSERT(seq_src->GetSeqLen);
+    return (*seq_src->GetSeqLen)(seq_src->DataStructure, oid);
+}
+
+void
+BlastSeqSrcReleaseSequence(const BlastSeqSrc* seq_src,
+                           void* sequence)
+{
+    ASSERT(seq_src);
+    ASSERT(seq_src->ReleaseSequence);
+    ASSERT(sequence);
+    (*seq_src->ReleaseSequence)(seq_src->DataStructure, sequence);
+}
 
 /******************** BlastSeqSrcIterator API *******************************/
 
-BlastSeqSrcIterator* BlastSeqSrcIteratorNew(unsigned int chunk_sz)
+BlastSeqSrcIterator* BlastSeqSrcIteratorNew()
+{
+    return BlastSeqSrcIteratorNewEx(0);
+}
+
+const unsigned int kBlastSeqSrcDefaultChunkSize = 1024;
+
+BlastSeqSrcIterator* BlastSeqSrcIteratorNewEx(unsigned int chunk_sz)
 {
     BlastSeqSrcIterator* itr = NULL;
 
@@ -148,7 +249,7 @@ BlastSeqSrcIterator* BlastSeqSrcIteratorNew(unsigned int chunk_sz)
     }
 
     /* Should employ lazy initialization? */
-    itr->oid_list = (unsigned int*)malloc(chunk_sz * sizeof(unsigned int));
+    itr->oid_list = (int*)malloc(chunk_sz * sizeof(unsigned int));
     if (!itr->oid_list) {
         sfree(itr);
         return NULL;
@@ -173,23 +274,31 @@ BlastSeqSrcIterator* BlastSeqSrcIteratorFree(BlastSeqSrcIterator* itr)
     return NULL;
 }
 
-Int4 BlastSeqSrcIteratorNext(const BlastSeqSrc* seq_src, BlastSeqSrcIterator* itr)
+Int4 BlastSeqSrcIteratorNext(const BlastSeqSrc* seq_src, 
+                             BlastSeqSrcIterator* itr)
 {
     ASSERT(seq_src);
     ASSERT(itr);
     ASSERT(seq_src->IterNext);
 
-    return (*seq_src->IterNext)((void*) seq_src, itr);
+    return (*seq_src->IterNext)(seq_src->DataStructure, itr);
 }
 
 /*****************************************************************************/
 
-#define DEFINE_MEMBER_FUNCTIONS(member_type, member, data_structure_type) \
-DEFINE_ACCESSOR(member_type, member, data_structure_type) \
-DEFINE_MUTATOR(member_type, member, data_structure_type)
+/* The following macros implement the "member functions" of the BlastSeqSrc
+ * structure so that its various fields can be accessed by the implementors of
+ * the BlastSeqSrc interface */
 
-#define DEFINE_ACCESSOR(member_type, member, data_structure_type) \
-member_type Get##member(const data_structure_type var) \
+#ifndef SKIP_DOXYGEN_PROCESSING
+
+#define DEFINE_BLAST_SEQ_SRC_MEMBER_FUNCTIONS(member_type, member) \
+DEFINE_BLAST_SEQ_SRC_ACCESSOR(member_type, member) \
+DEFINE_BLAST_SEQ_SRC_MUTATOR(member_type, member)
+
+#define DEFINE_BLAST_SEQ_SRC_ACCESSOR(member_type, member) \
+member_type \
+_BlastSeqSrcImpl_Get##member(const BlastSeqSrc* var) \
 { \
     if (var) \
         return var->member; \
@@ -197,27 +306,30 @@ member_type Get##member(const data_structure_type var) \
         return (member_type) NULL; \
 }
 
-#define DEFINE_MUTATOR(member_type, member, data_structure_type) \
-void Set##member(data_structure_type var, member_type arg) \
+#define DEFINE_BLAST_SEQ_SRC_MUTATOR(member_type, member) \
+void \
+_BlastSeqSrcImpl_Set##member(BlastSeqSrc* var, member_type arg) \
 { if (var) var->member = arg; }
 
+#endif
+
 /* Note there's no ; after these macros! */
-DEFINE_MEMBER_FUNCTIONS(BlastSeqSrcConstructor, NewFnPtr, BlastSeqSrc*)
-DEFINE_MEMBER_FUNCTIONS(BlastSeqSrcDestructor, DeleteFnPtr, BlastSeqSrc*)
-DEFINE_MEMBER_FUNCTIONS(BlastSeqSrcCopier, CopyFnPtr, BlastSeqSrc*)
+DEFINE_BLAST_SEQ_SRC_MEMBER_FUNCTIONS(BlastSeqSrcConstructor, NewFnPtr)
+DEFINE_BLAST_SEQ_SRC_MEMBER_FUNCTIONS(BlastSeqSrcDestructor, DeleteFnPtr)
+DEFINE_BLAST_SEQ_SRC_MEMBER_FUNCTIONS(BlastSeqSrcCopier, CopyFnPtr)
 
-DEFINE_MEMBER_FUNCTIONS(void*, DataStructure, BlastSeqSrc*)
-DEFINE_MEMBER_FUNCTIONS(GetInt4FnPtr, GetNumSeqs, BlastSeqSrc*)
-DEFINE_MEMBER_FUNCTIONS(GetInt4FnPtr, GetMaxSeqLen, BlastSeqSrc*)
-DEFINE_MEMBER_FUNCTIONS(GetInt4FnPtr, GetAvgSeqLen, BlastSeqSrc*)
-DEFINE_MEMBER_FUNCTIONS(GetInt8FnPtr, GetTotLen, BlastSeqSrc*)
+DEFINE_BLAST_SEQ_SRC_MEMBER_FUNCTIONS(void*, DataStructure)
+DEFINE_BLAST_SEQ_SRC_MEMBER_FUNCTIONS(char*, InitErrorStr)
+DEFINE_BLAST_SEQ_SRC_MEMBER_FUNCTIONS(GetInt4FnPtr, GetNumSeqs)
+DEFINE_BLAST_SEQ_SRC_MEMBER_FUNCTIONS(GetInt4FnPtr, GetMaxSeqLen)
+DEFINE_BLAST_SEQ_SRC_MEMBER_FUNCTIONS(GetInt4FnPtr, GetAvgSeqLen)
+DEFINE_BLAST_SEQ_SRC_MEMBER_FUNCTIONS(GetInt8FnPtr, GetTotLen)
 
-DEFINE_MEMBER_FUNCTIONS(GetStrFnPtr, GetName, BlastSeqSrc*)
-DEFINE_MEMBER_FUNCTIONS(GetBoolFnPtr, GetIsProt, BlastSeqSrc*)
+DEFINE_BLAST_SEQ_SRC_MEMBER_FUNCTIONS(GetStrFnPtr, GetName)
+DEFINE_BLAST_SEQ_SRC_MEMBER_FUNCTIONS(GetBoolFnPtr, GetIsProt)
 
-DEFINE_MEMBER_FUNCTIONS(GetSeqBlkFnPtr, GetSequence, BlastSeqSrc*)
-DEFINE_MEMBER_FUNCTIONS(GetInt4FnPtr, GetSeqLen, BlastSeqSrc*)
-DEFINE_MEMBER_FUNCTIONS(GetNextChunkFnPtr, GetNextChunk, BlastSeqSrc*)
-DEFINE_MEMBER_FUNCTIONS(AdvanceIteratorFnPtr, IterNext, BlastSeqSrc*)
-DEFINE_MEMBER_FUNCTIONS(GetErrorFnPtr, GetError, BlastSeqSrc*)
-DEFINE_MEMBER_FUNCTIONS(GetSeqBlkFnPtr, RetSequence, BlastSeqSrc*)
+DEFINE_BLAST_SEQ_SRC_MEMBER_FUNCTIONS(GetSeqBlkFnPtr, GetSequence)
+DEFINE_BLAST_SEQ_SRC_MEMBER_FUNCTIONS(GetInt4FnPtr, GetSeqLen)
+DEFINE_BLAST_SEQ_SRC_MEMBER_FUNCTIONS(ReleaseSeqBlkFnPtr, ReleaseSequence)
+
+DEFINE_BLAST_SEQ_SRC_MEMBER_FUNCTIONS(AdvanceIteratorFnPtr, IterNext)
