@@ -1,4 +1,4 @@
-/* $Id: blast_setup.c,v 1.117 2005/04/27 19:54:21 dondosha Exp $
+/* $Id: blast_setup.c,v 1.127 2005/10/03 12:57:03 madden Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -34,7 +34,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] =
-    "$Id: blast_setup.c,v 1.117 2005/04/27 19:54:21 dondosha Exp $";
+    "$Id: blast_setup.c,v 1.127 2005/10/03 12:57:03 madden Exp $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <algo/blast/core/blast_setup.h>
@@ -46,51 +46,43 @@ static char const rcsid[] =
 /* See description in blast_setup.h */
 Int2
 Blast_ScoreBlkKbpGappedCalc(BlastScoreBlk * sbp,
-                        const BlastScoringOptions * scoring_options,
-                        EBlastProgramType program, BlastQueryInfo * query_info)
+                            const BlastScoringOptions * scoring_options,
+                            EBlastProgramType program, 
+                            const BlastQueryInfo * query_info,
+                            Blast_Message** error_return)
 {
     Int2 index = 0;
 
     if (sbp == NULL || scoring_options == NULL)
         return 1;
 
-    /* At this stage query sequences are nucleotide only for blastn */
+    /* Allocate and fill values for a gapped Karlin block, given the scoring
+       options, then copy it for all the query contexts. */
+    for (index = query_info->first_context;
+         index <= query_info->last_context; index++) {
+        Int2 retval = 0;
+        sbp->kbp_gap_std[index] = Blast_KarlinBlkNew();
 
-    if (program == eBlastTypeBlastn) {
-
-        /* for blastn, duplicate the ungapped Karlin 
-           structures for use in gapped alignments */
-
-        for (index = query_info->first_context;
-             index <= query_info->last_context; index++) {
-            if (sbp->kbp_std[index] != NULL) {
-                sbp->kbp_gap_std[index] = Blast_KarlinBlkNew();
-                Blast_KarlinBlkCopy(sbp->kbp_gap_std[index],
-                                    sbp->kbp_std[index]);
-            }
+        /* At this stage query sequences are nucleotide only for blastn */
+        if (program == eBlastTypeBlastn) {
+            retval = 
+                Blast_KarlinBlkNuclGappedCalc(sbp->kbp_gap_std[index],
+                    scoring_options->gap_open, scoring_options->gap_extend, 
+                    scoring_options->reward, scoring_options->penalty, 
+                    sbp->kbp_std[index], &(sbp->round_down), error_return);
+        } else {
+            retval = 
+                Blast_KarlinBlkGappedCalc(sbp->kbp_gap_std[index],
+                    scoring_options->gap_open, scoring_options->gap_extend,
+                    scoring_options->decline_align, sbp->name, error_return);
         }
+        if (retval)
+            return retval;
 
-    } else {
-        for (index = query_info->first_context;
-             index <= query_info->last_context; index++) {
-            Int2 retval = 0;
-            sbp->kbp_gap_std[index] = Blast_KarlinBlkNew();
-            retval = Blast_KarlinBlkGappedCalc(sbp->kbp_gap_std[index],
-                                               scoring_options->gap_open,
-                                               scoring_options->gap_extend,
-                                               scoring_options->decline_align,
-                                               sbp->name, NULL);
-            /* FIXME: retval is not expressive enough and the Blast_Message is
-             * being passed NULL, so it's not possible to pass a meaningful
-             * error code to the calling context! */
-            if (retval != 0) {
-                return retval;
-            }
-
-            /* For right now, copy the contents from kbp_gap_std to 
-             * kbp_gap_psi (as in old code - BLASTSetUpSearchInternalByLoc) */
+        /* For right now, copy the contents from kbp_gap_std to 
+         * kbp_gap_psi (as in old code - BLASTSetUpSearchInternalByLoc) */
+        if (program != eBlastTypeBlastn) {
             sbp->kbp_gap_psi[index] = Blast_KarlinBlkNew();
-            
             Blast_KarlinBlkCopy(sbp->kbp_gap_psi[index], 
                                 sbp->kbp_gap_std[index]);
         }
@@ -280,7 +272,7 @@ s_PHIScoreBlkFill(BlastScoreBlk* sbp, const BlastScoringOptions* options,
        sprintf(buffer, "Matrix %s not allowed in PHI-BLAST\n", options->matrix);
    }
    if (status) 
-       Blast_MessageWrite(blast_message, BLAST_SEV_WARNING, 2, 1, buffer);
+       Blast_MessageWrite(blast_message, eBlastSevWarning, 2, 1, buffer);
    else {
        /* Put a copy the Karlin block into the kbp_std array */
        sbp->kbp_std[0] = (Blast_KarlinBlk*) 
@@ -323,15 +315,9 @@ Blast_ScoreBlkMatrixInit(EBlastProgramType program_number,
         }
  
      } else {
-        char* p = NULL;
-
         sbp->read_in_matrix = TRUE;
         BLAST_ScoreSetAmbigRes(sbp, 'X');
-        sbp->name = strdup(scoring_options->matrix);
-        /* protein matrices are in all caps by convention */
-        for (p = sbp->name; *p != NULLB; p++) {
-            *p = toupper(*p);
-        }
+        sbp->name = BLAST_StrToUpper(scoring_options->matrix);
     }
     status = Blast_ScoreBlkMatrixFill(sbp, scoring_options->matrix_path);
     if (status) {
@@ -343,7 +329,7 @@ Blast_ScoreBlkMatrixInit(EBlastProgramType program_number,
 
 Int2 
 BlastSetup_ScoreBlkInit(BLAST_SequenceBlk* query_blk, 
-                        BlastQueryInfo* query_info, 
+                        const BlastQueryInfo* query_info, 
                         const BlastScoringOptions* scoring_options, 
                         EBlastProgramType program_number, 
                         BlastScoreBlk* *sbpp, 
@@ -374,13 +360,12 @@ BlastSetup_ScoreBlkInit(BLAST_SequenceBlk* query_blk,
     }
 
     /* Fills in block for gapped blast. */
-    if (program_number == eBlastTypePhiBlastn ||
-        program_number == eBlastTypePhiBlastp) {
+    if (Blast_ProgramIsPhiBlast(program_number)) {
        status = s_PHIScoreBlkFill(sbp, scoring_options, blast_message);
     } else {
        if ((status = Blast_ScoreBlkKbpUngappedCalc(program_number, sbp, 
                         query_blk->sequence, query_info)) != 0) {
-          Blast_MessageWrite(blast_message, BLAST_SEV_ERROR, 2, 1, 
+          Blast_MessageWrite(blast_message, eBlastSevError, 2, 1, 
              "Could not calculate ungapped Karlin-Altschul parameters due "
              "to an invalid query sequence. Please verify the query "
              "sequence(s) and/or filtering options");
@@ -388,12 +373,9 @@ BlastSetup_ScoreBlkInit(BLAST_SequenceBlk* query_blk,
        }
 
        if (scoring_options->gapped_calculation) {
-          status = Blast_ScoreBlkKbpGappedCalc(sbp, scoring_options, 
-                                           program_number, query_info);
-          if (status) {
-             Blast_MessageWrite(blast_message, BLAST_SEV_ERROR, 2, 1, 
-                                "Unable to initialize scoring block");
-          }
+          status = 
+              Blast_ScoreBlkKbpGappedCalc(sbp, scoring_options, program_number,
+                                          query_info, blast_message);
        }
     }
 
@@ -403,57 +385,62 @@ BlastSetup_ScoreBlkInit(BLAST_SequenceBlk* query_blk,
 Int2 BLAST_MainSetUp(EBlastProgramType program_number,
     const QuerySetUpOptions *qsup_options,
     const BlastScoringOptions *scoring_options,
-    const BlastHitSavingOptions *hit_options,
     BLAST_SequenceBlk *query_blk,
-    BlastQueryInfo *query_info,
+    const BlastQueryInfo *query_info,
     double scale_factor,
     BlastSeqLoc **lookup_segments, 
-    BlastMaskInformation* maskInfo,
+    BlastMaskLoc **mask,
     BlastScoreBlk **sbpp, 
     Blast_Message **blast_message)
 {
     Boolean mask_at_hash = FALSE; /* mask only for making lookup table? */
     Int2 status = 0;            /* return value */
     BlastMaskLoc *filter_maskloc = NULL;   /* Local variable for mask locs. */
-    SBlastFilterOptions* filter_options = NULL;
+
+    SBlastFilterOptions* filter_options = qsup_options->filtering_options;
+    Boolean filter_options_allocated = FALSE;
 
 
-    if (maskInfo)
+    if (mask)
+        *mask = NULL;
+
+    if (filter_options == NULL && qsup_options->filter_string)
     {
-       maskInfo->filter_slp = NULL;
-       maskInfo->mask_at_hash = FALSE;
-    }
-
-    if (qsup_options->filtering_options == NULL && qsup_options->filter_string)
-    {
-         status = BlastFilteringOptionsFromString(program_number, qsup_options->filter_string, &filter_options, blast_message);
-         if (status)
+         status = BlastFilteringOptionsFromString(program_number, 
+                                                  qsup_options->filter_string, 
+                                                  &filter_options, 
+                                                  blast_message);
+         if (status) {
+            filter_options = SBlastFilterOptionsFree(filter_options);
             return status;
+         }
+         filter_options_allocated = TRUE;
     }
+    ASSERT(filter_options);
 
     status = BlastSetUp_GetFilteringLocations(query_blk, 
                                               query_info, 
                                               program_number, 
-                                              filter_options ? filter_options : qsup_options->filtering_options, 
-                                              &filter_maskloc, 
+                                              filter_options,
+                                              & filter_maskloc, 
                                               blast_message);
 
-
     if (status) {
+        if (filter_options_allocated)
+            filter_options = SBlastFilterOptionsFree(filter_options);
         return status;
     } 
 
-    mask_at_hash = SBlastFilterOptionsMaskAtHash(filter_options ? filter_options : qsup_options->filtering_options);
+    mask_at_hash = SBlastFilterOptionsMaskAtHash(filter_options);
 
-    filter_options = SBlastFilterOptionsFree(filter_options);
+    if (filter_options_allocated) {
+        filter_options = SBlastFilterOptionsFree(filter_options);
+    }
 
-    if (!mask_at_hash)
-    {
-        status = BlastSetUp_MaskQuery(query_blk, query_info, filter_maskloc, 
-                                      program_number);
-        if (status != 0) {
-            return status;
-        }
+
+    if (!mask_at_hash) {
+        BlastSetUp_MaskQuery(query_blk, query_info, filter_maskloc, 
+                             program_number);
     }
 
     if (program_number == eBlastTypeBlastx && scoring_options->is_ooframe) {
@@ -469,17 +456,14 @@ Int2 BLAST_MainSetUp(EBlastProgramType program_number,
                                       filter_maskloc, lookup_segments);
     }
 
-    if (maskInfo)
+    if (mask)
     {
-        if (program_number == eBlastTypeBlastx || 
-            program_number == eBlastTypeTblastx ||
-            program_number == eBlastTypeRpsTblastn) {
+        if (Blast_QueryIsTranslated(program_number)) {
             /* Filter locations so far are in protein coordinates; 
                convert them back to nucleotide here. */
             BlastMaskLocProteinToDNA(filter_maskloc, query_info);
         }
-        maskInfo->filter_slp = filter_maskloc;
-        maskInfo->mask_at_hash = mask_at_hash;
+        *mask = filter_maskloc;
         filter_maskloc = NULL;
     }
     else 
@@ -488,11 +472,8 @@ Int2 BLAST_MainSetUp(EBlastProgramType program_number,
     status = BlastSetup_ScoreBlkInit(query_blk, query_info, scoring_options, 
                                      program_number, sbpp, scale_factor, 
                                      blast_message);
-    if (status > 0) {
-        return status;
-    }
 
-    return 0;
+    return status;
 }
 
 
@@ -513,8 +494,7 @@ Int2 BLAST_CalcEffLengths (EBlastProgramType program_number,
 
    /* Do nothing for PHI BLAST, because search space is calculated by 
       completely different formulas there. */
-   if (program_number == eBlastTypePhiBlastn || 
-       program_number == eBlastTypePhiBlastp)
+   if (Blast_ProgramIsPhiBlast(program_number))
        return 0;
 
    /* use overriding value from effective lengths options or the real value
@@ -542,14 +522,9 @@ Int2 BLAST_CalcEffLengths (EBlastProgramType program_number,
    else
       db_num_seqs = eff_len_params->real_num_seqs;
    
-   if (program_number != eBlastTypeBlastn) {
-      if (scoring_options->gapped_calculation) {
-         BLAST_GetAlphaBeta(sbp->name,&alpha,&beta,TRUE, 
-            scoring_options->gap_open, scoring_options->gap_extend);
-      }
-   }
-   
-   kbp_ptr = (scoring_options->gapped_calculation ? sbp->kbp_gap : sbp->kbp);
+   /* N.B.: the old code used kbp_gap_std instead of the kbp_gap alias (which
+    * could be kbp_gap_psi), hence we duplicate that behavior here */
+   kbp_ptr = (scoring_options->gapped_calculation ? sbp->kbp_gap_std : sbp->kbp);
    
    for (index = query_info->first_context;
         index <= query_info->last_context;
@@ -567,17 +542,26 @@ Int2 BLAST_CalcEffLengths (EBlastProgramType program_number,
           * blocks are allocated for each sequence (one per strand), but we
           * only need one of them.
           */
-         if (program_number != eBlastTypeBlastn &&
-             scoring_options->gapped_calculation) {
-            BLAST_ComputeLengthAdjustment(kbp->K, kbp->logK,
-                                          alpha/kbp->Lambda, beta,
-                                          query_length, db_length,
-                                          db_num_seqs, &length_adjustment);
+         if (program_number == eBlastTypeBlastn) {
+             Blast_GetNuclAlphaBeta(scoring_options->reward, 
+                                    scoring_options->penalty, 
+                                    scoring_options->gap_open, 
+                                    scoring_options->gap_extend, 
+                                    sbp->kbp_std[index], 
+                                    scoring_options->gapped_calculation,
+                                    &alpha, &beta);
          } else {
-            BLAST_ComputeLengthAdjustment(kbp->K, kbp->logK, 1/kbp->H, 0,
-                                          query_length, db_length,
-                                          db_num_seqs, &length_adjustment);
-         }        
+             BLAST_GetAlphaBeta(sbp->name, &alpha, &beta,
+                                scoring_options->gapped_calculation, 
+                                scoring_options->gap_open, 
+                                scoring_options->gap_extend, 
+                                sbp->kbp_std[index]);
+         }
+         BLAST_ComputeLengthAdjustment(kbp->K, kbp->logK,
+                                       alpha/kbp->Lambda, beta,
+                                       query_length, db_length,
+                                       db_num_seqs, &length_adjustment);
+
          if (eff_len_options->searchsp_eff)
             effective_search_space = eff_len_options->searchsp_eff;
          else
@@ -703,8 +687,7 @@ BlastSeqLoc_RestrictToInterval(BlastSeqLoc* *mask, Int4 from, Int4 to)
          /* Shift the pointer to the next link in chain and free this link. */
          if (last_loc)
             last_loc->next = seqloc->next;
-         sfree(seqloc->ssr);
-         sfree(seqloc);
+         seqloc = BlastSeqLocNodeFree(seqloc);
       } else if (!head_loc) {
          /* First time a mask was found within the range. */
          head_loc = last_loc = seqloc;
@@ -718,32 +701,33 @@ BlastSeqLoc_RestrictToInterval(BlastSeqLoc* *mask, Int4 from, Int4 to)
 }
 
 Int2 
-Blast_SetPHIPatternInfo(EBlastProgramType program, 
-                        SPHIPatternSearchBlk* pattern_blk, 
-                        BLAST_SequenceBlk* query, BlastSeqLoc* lookup_segments, 
-                        BlastQueryInfo* query_info)
+Blast_SetPHIPatternInfo(EBlastProgramType            program,
+                        const SPHIPatternSearchBlk * pattern_blk,
+                        const BLAST_SequenceBlk    * query,
+                        const BlastSeqLoc          * lookup_segments,
+                        BlastQueryInfo             * query_info)
 {
     const Boolean kIsNa = (program == eBlastTypePhiBlastn);
-
-    ASSERT(program == eBlastTypePhiBlastn || program  == eBlastTypePhiBlastp);
+    
+    ASSERT(Blast_ProgramIsPhiBlast(program));
     ASSERT(query_info && pattern_blk);
-
+    
     query_info->pattern_info = SPHIQueryInfoNew();
     
     /* If pattern is not found in query, return failure status. */
     if (!PHIGetPatternOccurrences(pattern_blk, query, lookup_segments, kIsNa,
                                   query_info->pattern_info))
         return -1;
-
+    
     /* Save pattern probability, because it needs to be passed back to
        formatting stage, where lookup table will not be available. */
     query_info->pattern_info->probability = pattern_blk->patternProbability;
-
+    
     /* Save minimal pattern length in the length adjustment field, because 
        that is essentially its meaning. */
     query_info->contexts[0].length_adjustment = 
         pattern_blk->minPatternMatchLength;
-
+    
     return 0;
 }
 
