@@ -1,4 +1,4 @@
-/* $Id: blast_traceback.c,v 1.167 2005/04/27 21:11:35 dondosha Exp $
+/* $Id: blast_traceback.c,v 1.179 2005/12/01 14:47:48 madden Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -6,7 +6,7 @@
  *
  *  This software/database is a "United States Government Work" under the
  *  terms of the United States Copyright Act.  It was written as part of
- *  the author's offical duties as a United States Government employee and
+ *  the author's official duties as a United States Government employee and
  *  thus cannot be copyrighted.  This software/database is freely available
  *  to the public for use. The National Library of Medicine and the U.S.
  *  Government have not placed any restriction on its use or reproduction.
@@ -23,7 +23,6 @@
  *
  * ===========================================================================
  *
- * Author: Ilya Dondoshansky
  *
  */
 
@@ -39,7 +38,7 @@
  *                    for ( all HSP lists )
  *                        Blast_TracebackFromHSPList
  *            else if ( composition based statistics )
- *                Kappa_RedoAlignmentCore
+ *                Blast_RedoAlignmentCore
  *            else
  *                for ( all HSP lists )
  *                    if ( PHI BLAST) 
@@ -51,11 +50,12 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: blast_traceback.c,v 1.167 2005/04/27 21:11:35 dondosha Exp $";
+    "$Id: blast_traceback.c,v 1.179 2005/12/01 14:47:48 madden Exp $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <algo/blast/core/blast_traceback.h>
 #include <algo/blast/core/blast_util.h>
+#include <algo/blast/core/blast_encoding.h>
 #include <algo/blast/core/link_hsps.h>
 #include <algo/blast/core/blast_setup.h>
 #include <algo/blast/core/blast_kappa.h>
@@ -162,11 +162,8 @@ s_HSPListRescaleScores(BlastHSPList* hsp_list, double scale_factor)
    Blast_HSPListSortByScore(hsp_list);
 }
 
-/** Simple macro to swap two numbers. */
-#define SWAP(a, b) {tmp = (a); (a) = (b); (b) = tmp; }
-
-/** Swaps query and subject information in an HSP structure for RPS BLAST 
- * search. This is necessary because query and subject are switched during the 
+/** Swaps insertions and deletions in an edit script for RPS BLAST search. 
+ * This is necessary because query and subject are switched during the 
  * traceback alignment, and must be switched back.
  * @param hsp HSP structure to fix. [in] [out]
  */
@@ -203,29 +200,26 @@ s_BlastHSPListRPSUpdate(EBlastProgramType program, BlastHSPList *hsplist)
    BlastSeg tmp;
 
    /* If this is not an RPS BLAST search, do not do anything. */
-   if (program != eBlastTypeRpsBlast && program != eBlastTypeRpsTblastn)
+   if ( !Blast_ProgramIsRpsBlast(program))
       return;
 
    hsp = hsplist->hsp_array;
    for (i = 0; i < hsplist->hspcnt; i++) {
 
-      /* switch query and subject offsets (which are
-         already in local coordinates) */
-
+      /* switch query and subject (which are already in local coordinates) */
       tmp = hsp[i]->query;
       hsp[i]->query = hsp[i]->subject;
       hsp[i]->subject = tmp;
 
-      /* Change the traceback information to reflect the
-         query and subject sequences getting switched */
-
+      /* Change the traceback information to reflect the query and subject 
+         sequences getting switched */
       s_BlastHSPRPSUpdate(hsp[i]);
 
       /* If query was nucleotide, set context, because it is needed in order 
-	 to pick correct Karlin block for calculating bit scores. There are 
+         to pick correct Karlin block for calculating bit scores. There are 
          6 contexts corresponding to each nucleotide query sequence. */
       if (program == eBlastTypeRpsTblastn) {
-	 hsp[i]->context = FrameToContext(hsp[i]->query.frame);
+          hsp[i]->context = FrameToContext(hsp[i]->query.frame);
       }
    }
    Blast_HSPListSortByScore(hsplist);
@@ -269,9 +263,13 @@ s_HSPListPostTracebackUpdate(EBlastProgramType program_number,
        * computations. 
        */
       double scale_factor = 
-         (program_number == eBlastTypeRpsBlast || 
-          program_number == eBlastTypeRpsTblastn) ? 
-         score_params->scale_factor : 1.0;
+         (Blast_ProgramIsRpsBlast(program_number) ?
+         score_params->scale_factor : 1.0);
+
+      /* For nucleotide search, if match score is = 2, the odd scores
+         are rounded down to the nearest even number. */
+      Blast_HSPListAdjustOddBlastnScores(hsp_list, kGapped, sbp);
+
       Blast_HSPListGetEvalues(query_info, hsp_list, kGapped, sbp, 0,
                               scale_factor);
    }
@@ -313,7 +311,6 @@ Blast_TracebackFromHSPList(EBlastProgramType program_number,
    Int4 q_start, s_start;
    BlastHitSavingOptions* hit_options = hit_params->options;
    BlastScoringOptions* score_options = score_params->options;
-   Int4 context_offset;
    Uint1* translation_buffer = NULL;
    Int4 * frame_offsets   = NULL;
    Int4 * frame_offsets_a = NULL;
@@ -321,8 +318,7 @@ Blast_TracebackFromHSPList(EBlastProgramType program_number,
    const Boolean kIsOutOfFrame = score_options->is_ooframe;
    const Boolean kGreedyTraceback = (ext_options->eTbackExt == eGreedyTbck);
    const Boolean kTranslateSubject = 
-      (program_number == eBlastTypeTblastn ||
-       program_number == eBlastTypeRpsTblastn); 
+        (Blast_SubjectIsTranslated(program_number) || program_number == eBlastTypeRpsTblastn);
    BlastQueryInfo* query_info = query_info_in;
    Int4 offsets[2];
    BlastIntervalTree* tree = NULL;
@@ -360,8 +356,7 @@ Blast_TracebackFromHSPList(EBlastProgramType program_number,
       subject_length = subject_blk->length;
    }
 
-   if (program_number == eBlastTypeRpsBlast || 
-       program_number == eBlastTypeRpsTblastn) {
+   if (Blast_ProgramIsRpsBlast(program_number)) {
       /* Create a local BlastQueryInfo structure for this subject sequence
 	 that has been switched with the query. */
       query_info = BlastMemDup(query_info_in, sizeof(BlastQueryInfo));
@@ -386,20 +381,13 @@ Blast_TracebackFromHSPList(EBlastProgramType program_number,
 
    for (index=0; index < hsp_list->hspcnt; index++) {
       hsp = hsp_array[index];
-      if (program_number == eBlastTypeBlastx || 
-          program_number == eBlastTypeTblastx) {
-         Int4 context = hsp->context - hsp->context % 3;
-         context_offset = query_info->contexts[context].query_offset;
+      if (program_number == eBlastTypeBlastx && kIsOutOfFrame) {
+          Int4 context = hsp->context - hsp->context % 3;
+          Int4 context_offset = query_info->contexts[context].query_offset;
          
-         if (kIsOutOfFrame) {
-            query = query_blk->oof_sequence + CODON_LENGTH + context_offset;
-            query_length = query_info->contexts[context+2].query_offset +
-                query_info->contexts[context+2].query_length - context_offset;
-         } else {
-            query = query_blk->sequence + 
-               query_info->contexts[hsp->context].query_offset;
-            query_length = query_info->contexts[hsp->context].query_length;
-         }
+          query = query_blk->oof_sequence + CODON_LENGTH + context_offset;
+          query_length = query_info->contexts[context+2].query_offset +
+              query_info->contexts[context+2].query_length - context_offset;
       } else {
           query = query_blk->sequence + 
               query_info->contexts[hsp->context].query_offset;
@@ -542,9 +530,7 @@ Blast_TracebackFromHSPList(EBlastProgramType program_number,
    
    /* Remove any HSPs that share a starting or ending diagonal
       with a higher-scoring HSP. */
-   hsp_list->hspcnt = 
-       Blast_CheckHSPsForCommonEndpoints(hsp_list->hsp_array, 
-                                         hsp_list->hspcnt);
+   Blast_HSPListPurgeHSPsWithCommonEndpoints(program_number, hsp_list);
 
    /* Sort HSPs by score again, as the scores might have changed. */
    Blast_HSPListSortByScore(hsp_list);
@@ -613,8 +599,7 @@ s_PHITracebackFromHSPList(EBlastProgramType program_number,
    Int4 q_start, s_start;
    SPHIQueryInfo* pattern_info = NULL;
    
-   if (program_number != eBlastTypePhiBlastn &&
-       program_number != eBlastTypePhiBlastp)
+   if ( !Blast_ProgramIsPhiBlast(program_number))
        return -1;
    
    ASSERT(hsp_list && query_blk && subject_blk && gap_align && sbp &&
@@ -675,14 +660,14 @@ s_PHITracebackFromHSPList(EBlastProgramType program_number,
    return 0;
 }
 
-Uint1 Blast_TracebackGetEncoding(EBlastProgramType program_number) 
+EBlastEncoding Blast_TracebackGetEncoding(EBlastProgramType program_number) 
 {
-   Uint1 encoding;
+   EBlastEncoding retval = eBlastEncodingError;
 
    switch (program_number) {
    case eBlastTypeBlastn:
    case eBlastTypePhiBlastn:
-      encoding = BLASTNA_ENCODING;
+      retval = eBlastEncodingNucleotide;
       break;
    case eBlastTypeBlastp:
    case eBlastTypeRpsBlast:
@@ -690,17 +675,17 @@ Uint1 Blast_TracebackGetEncoding(EBlastProgramType program_number)
    case eBlastTypeRpsTblastn:
    case eBlastTypePsiBlast:
    case eBlastTypePhiBlastp:
-      encoding = BLASTP_ENCODING;
+      retval = eBlastEncodingProtein;
       break;
    case eBlastTypeTblastn:
    case eBlastTypeTblastx:
-      encoding = NCBI4NA_ENCODING;
+      retval = eBlastEncodingNcbi4na;
       break;
    default:
-      encoding = ERROR_ENCODING;
+      retval = eBlastEncodingError;
       break;
    }
-   return encoding;
+   return retval;
 }
 
 /** Delete extra subject sequences hits, if after-traceback hit list size is
@@ -854,7 +839,7 @@ Int2 s_RPSComputeTraceback(EBlastProgramType program_number,
    BlastScoreBlk* sbp;
    Int4 **rpsblast_pssms = NULL;
    Int4 db_seq_start;
-   Uint1 encoding;
+   EBlastEncoding encoding;
    BlastSeqSrcGetSeqArg seq_arg;
    BlastQueryInfo* one_query_info = NULL;
    BLAST_SequenceBlk* one_query = NULL;
@@ -1015,8 +1000,7 @@ BLAST_ComputeTraceback(EBlastProgramType program_number,
 
    results = Blast_HSPResultsNew(query_info->num_queries);
 
-   if (program_number == eBlastTypeRpsBlast ||
-       program_number == eBlastTypeRpsTblastn) {
+   if (Blast_ProgramIsRpsBlast(program_number)) {
        status = 
            s_RPSComputeTraceback(program_number, hsp_stream, seq_src, query, 
                                  query_info, gap_align, score_params, ext_params,
@@ -1027,18 +1011,17 @@ BLAST_ComputeTraceback(EBlastProgramType program_number,
               (ext_params->options->compositionBasedStats == TRUE || 
                ext_params->options->eTbackExt == eSmithWatermanTbck)) {
       status = 
-          Kappa_RedoAlignmentCore(program_number, query, query_info, sbp, 
+          Blast_RedoAlignmentCore(program_number, query, query_info, sbp, 
                                   hsp_stream, seq_src, gen_code_string,
                                   score_params, ext_params, hit_params, 
                                   psi_options, results); 
    } else {
       BlastSeqSrcGetSeqArg seq_arg;
-      Uint1 encoding = Blast_TracebackGetEncoding(program_number);
+      EBlastEncoding encoding = Blast_TracebackGetEncoding(program_number);
       Boolean perform_traceback = 
          (score_params->options->gapped_calculation && 
           (ext_params->options->ePrelimGapExt != eGreedyWithTracebackExt));
-      const Boolean kPhiBlast = (program_number == eBlastTypePhiBlastn ||
-                                 program_number == eBlastTypePhiBlastp);
+      const Boolean kPhiBlast = Blast_ProgramIsPhiBlast(program_number);
 
       memset((void*) &seq_arg, 0, sizeof(seq_arg));
 
@@ -1112,9 +1095,7 @@ BLAST_ComputeTraceback(EBlastProgramType program_number,
 
    /* Eliminate extra hits from results, if preliminary hit list size is larger
       than the final hit list size */
-   if (hit_params->options->hitlist_size < 
-       hit_params->options->prelim_hitlist_size)
-      s_BlastPruneExtraHits(results, hit_params->options->hitlist_size);
+    s_BlastPruneExtraHits(results, hit_params->options->hitlist_size);
 
    *results_out = results;
 
