@@ -1,4 +1,4 @@
-/* $Id: blast_traceback.h,v 1.45 2005/04/27 19:49:35 dondosha Exp $
+/* $Id: blast_traceback.h,v 1.47 2005/11/16 14:31:36 madden Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -6,7 +6,7 @@
  *
  *  This software/database is a "United States Government Work" under the
  *  terms of the United States Copyright Act.  It was written as part of
- *  the author's offical duties as a United States Government employee and
+ *  the author's official duties as a United States Government employee and
  *  thus cannot be copyrighted.  This software/database is freely available
  *  to the public for use. The National Library of Medicine and the U.S.
  *  Government have not placed any restriction on its use or reproduction.
@@ -23,7 +23,6 @@
  *
  * ===========================================================================
  *
- * Author: Ilya Dondoshansky
  *
  */
 
@@ -76,7 +75,7 @@ Blast_TracebackFromHSPList(EBlastProgramType program_number,
  * given a program number.
  */
 NCBI_XBLAST_EXPORT
-Uint1 Blast_TracebackGetEncoding(EBlastProgramType program_number);
+EBlastEncoding Blast_TracebackGetEncoding(EBlastProgramType program_number);
 
 /** Modifies the HSP data after the final gapped alignment.
  * Input includes only data that likely needs modification. This function
