@@ -1,4 +1,4 @@
-/* $Id: blast_util.c,v 1.91 2005/04/27 19:54:57 dondosha Exp $
+/* $Id: blast_util.c,v 1.105 2005/11/16 14:27:04 madden Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -6,7 +6,7 @@
  *
  *  This software/database is a "United States Government Work" under the
  *  terms of the United States Copyright Act.  It was written as part of
- *  the author's offical duties as a United States Government employee and
+ *  the author's official duties as a United States Government employee and
  *  thus cannot be copyrighted.  This software/database is freely available
  *  to the public for use. The National Library of Medicine and the U.S.
  *  Government have not placed any restriction on its use or reproduction.
@@ -34,7 +34,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: blast_util.c,v 1.91 2005/04/27 19:54:57 dondosha Exp $";
+    "$Id: blast_util.c,v 1.105 2005/11/16 14:27:04 madden Exp $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <algo/blast/core/blast_util.h>
@@ -113,10 +113,10 @@ Int2 BlastSeqBlkSetCompressedSequence(BLAST_SequenceBlk* seq_blk,
     return 0;
 }
 
-Int2 BlastSequenceBlkClean(BLAST_SequenceBlk* seq_blk)
+void BlastSequenceBlkClean(BLAST_SequenceBlk* seq_blk)
 {
    if (!seq_blk)
-       return 1;
+       return;
 
    if (seq_blk->sequence_allocated) {
        sfree(seq_blk->sequence);
@@ -131,7 +131,7 @@ Int2 BlastSequenceBlkClean(BLAST_SequenceBlk* seq_blk)
        seq_blk->oof_sequence_allocated = FALSE;
    }
 
-   return 0;
+   return;
 }
 
 BLAST_SequenceBlk* BlastSequenceBlkFree(BLAST_SequenceBlk* seq_blk)
@@ -185,6 +185,8 @@ Int2 BlastProgram2Number(const char *program, EBlastProgramType *number)
 		*number = eBlastTypeRpsTblastn;
     else if (strcasecmp("psiblast", program) == 0)
         *number = eBlastTypePsiBlast;
+    else if (strcasecmp("psitblastn", program) == 0)
+        *number = eBlastTypePsiTblastn;
     else if (strcasecmp("phiblastn", program) == 0)
         *number = eBlastTypePhiBlastn;
     else if (strcasecmp("phiblastp", program) == 0)
@@ -200,10 +202,10 @@ Int2 BlastNumber2Program(EBlastProgramType number, char* *program)
 		return 1;
     
 	switch (number) {
-		case eBlastTypeBlastn: case eBlastTypePhiBlastn:
+		case eBlastTypeBlastn: 
 			*program = strdup("blastn");
 			break;
-		case eBlastTypeBlastp: case eBlastTypePhiBlastp:
+		case eBlastTypeBlastp: 
 			*program = strdup("blastp");
 			break;
 		case eBlastTypeBlastx:
@@ -224,6 +226,15 @@ Int2 BlastNumber2Program(EBlastProgramType number, char* *program)
         case eBlastTypePsiBlast:
             *program = strdup("psiblast");
             break;
+        case eBlastTypePsiTblastn:
+            *program = strdup("psitblastn");
+            break;
+        case eBlastTypePhiBlastp:
+			*program = strdup("phiblastp");
+			break;
+        case eBlastTypePhiBlastn:
+			*program = strdup("phiblastn");
+			break;
         default:
 			*program = strdup("unknown");
 			break;
@@ -232,9 +243,6 @@ Int2 BlastNumber2Program(EBlastProgramType number, char* *program)
 	return 0;
 }
 
-/** Value of the 'X' aminoacid residue in NCBIstdaa encoding. */
-#define X_STDAA 21
-
 /** Translate 3 nucleotides into an amino acid
  * MUST have 'X' as unknown amino acid
  * @param codon 3 values in ncbi4na code
@@ -244,6 +252,7 @@ Int2 BlastNumber2Program(EBlastProgramType number, char* *program)
 static Uint1 
 s_CodonToAA (Uint1* codon, const Uint1* codes)
 {
+   const Uint1 kXResidue = AMINOACID_TO_NCBISTDAA['X'];
    register Uint1 aa = 0, taa;
    register int i, j, k, index0, index1, index2;
    static Uint1 mapping[4] = { 8,     /* T in ncbi4na */
@@ -265,20 +274,20 @@ s_CodonToAA (Uint1* codon, const Uint1* codes)
                         aa = taa;
                      else {
                         if (taa != aa) {
-                           aa = X_STDAA;
+                           aa = kXResidue;
                            break;
                         }
                      }
                   }
-                  if (aa == X_STDAA)
+                  if (aa == kXResidue)
                      break;
                }
             }
-            if (aa == X_STDAA)
+            if (aa == kXResidue)
                break;
          }
       }
-      if (aa == X_STDAA)
+      if (aa == kXResidue)
          break;
    }
    return aa;
@@ -646,7 +655,7 @@ Int2 GetReverseNuclSequence(const Uint1* sequence, Int4 length,
 
 Int1 BLAST_ContextToFrame(EBlastProgramType prog_number, Uint4 context_number)
 {
-   Int1 frame = 127;	/* 127 is used to indicate error */
+   Int1 frame = INT1_MAX;	/* INT1_MAX is used to indicate error */
 
    if (prog_number == eBlastTypeBlastn) {
       if (context_number % NUM_STRANDS == 0)
@@ -655,10 +664,9 @@ Int1 BLAST_ContextToFrame(EBlastProgramType prog_number, Uint4 context_number)
          frame = -1;
    } else if (prog_number == eBlastTypeBlastp   ||
               prog_number == eBlastTypeRpsBlast ||
-              prog_number == eBlastTypePsiBlast ||
               prog_number == eBlastTypeTblastn  ||
-              prog_number == eBlastTypePhiBlastn||
-              prog_number == eBlastTypePhiBlastp) {
+              Blast_ProgramIsPsiBlast(prog_number) ||
+              Blast_ProgramIsPhiBlast(prog_number)) {
       /* Query and subject are protein, no frame. */
       frame = 0;
    } else if (prog_number == eBlastTypeBlastx  ||
@@ -733,10 +741,58 @@ BlastQueryInfo* BlastQueryInfoDup(BlastQueryInfo* query_info)
    return retval;
 }
 
-/** Convert a sequence in ncbi4na or blastna encoding into a packed sequence
- * in ncbi2na encoding. Needed for 2 sequences BLASTn comparison.
+/** Calculates length of the DNA query from the BlastQueryInfo structure that 
+ * contains context information for translated frames for a set of queries.
+ * @param query_info Query information containing data for all contexts [in]
+ * @param query_index Which query to find DNA length for?
+ * @return DNA length of the query, calculated as sum of 3 protein frame lengths, 
+ *         plus 2, because 2 last nucleotide residues do not have a 
+ *         corresponding codon.
  */
-Int2 BLAST_PackDNA(Uint1* buffer, Int4 length, Uint1 encoding, 
+static Int4 
+s_GetTranslatedQueryDNALength(const BlastQueryInfo* query_info, Int4 query_index)
+{
+    Int4 start_context = NUM_FRAMES*query_index;
+    Int4 dna_length = 2;
+    Int4 index;
+ 
+    /* Make sure that query index is within appropriate range, and that this is
+       really a translated search */
+    ASSERT(query_index < query_info->num_queries);
+    ASSERT(start_context < query_info->last_context);
+
+    /* If only reverse strand is searched, then forward strand contexts don't 
+       have lengths information */
+    if (query_info->contexts[start_context].query_length == 0)
+        start_context += 3;
+
+    for (index = start_context; index < start_context + 3; ++index)
+        dna_length += query_info->contexts[index].query_length;
+ 
+    return dna_length;
+}
+
+Int4 BlastQueryInfoGetQueryLength(const BlastQueryInfo* qinfo,
+                                  EBlastProgramType program,
+                                  Int4 query_index)
+{
+    const Uint4 kNumContexts = BLAST_GetNumberOfContexts(program);
+    ASSERT(query_index < qinfo->num_queries);
+
+    if (Blast_QueryIsTranslated(program)) {
+        return s_GetTranslatedQueryDNALength(qinfo, query_index);
+    } else if (program == eBlastTypeBlastn) {
+        Int4 retval = qinfo->contexts[query_index*kNumContexts].query_length;
+        if (retval <= 0) {
+            retval = qinfo->contexts[query_index*kNumContexts+1].query_length;
+        }
+        return retval;
+    } else {
+        return qinfo->contexts[query_index*kNumContexts].query_length;
+    }
+}
+
+Int2 BLAST_PackDNA(const Uint1* buffer, Int4 length, EBlastEncoding encoding, 
                           Uint1** packed_seq)
 {
    Int4 new_length = length/COMPRESSION_RATIO + 1;
@@ -744,9 +800,13 @@ Int2 BLAST_PackDNA(Uint1* buffer, Int4 length, Uint1 encoding,
    Int4 index, new_index;
    Uint1 shift;     /* bit shift to pack bases */
 
+   if ( !new_buffer ) {
+       return -1;
+   }
+
    for (index=0, new_index=0; new_index < new_length-1; 
         ++new_index, index += COMPRESSION_RATIO) {
-      if (encoding == BLASTNA_ENCODING)
+      if (encoding == eBlastEncodingNucleotide)
          new_buffer[new_index] = 
             ((buffer[index]&NCBI2NA_MASK)<<6) | 
             ((buffer[index+1]&NCBI2NA_MASK)<<4) |
@@ -772,7 +832,7 @@ Int2 BLAST_PackDNA(Uint1* buffer, Int4 length, Uint1 encoding,
       case 2: shift = 2; break;
       default: abort();     /* should never happen */
       }
-      if (encoding == BLASTNA_ENCODING)
+      if (encoding == eBlastEncodingNucleotide)
          new_buffer[new_index] |= ((buffer[index]&NCBI2NA_MASK)<<shift);
       else
          new_buffer[new_index] |=
@@ -785,9 +845,9 @@ Int2 BLAST_PackDNA(Uint1* buffer, Int4 length, Uint1 encoding,
 }
 
 Int2 BLAST_InitDNAPSequence(BLAST_SequenceBlk* query_blk, 
-                            BlastQueryInfo* query_info)
+                            const BlastQueryInfo* query_info)
 {
-   Uint1* buffer,* seq,* tmp_seq;
+   Uint1* buffer,* seq = NULL,* tmp_seq;
    Int4 total_length, index, offset, i, context;
    Int4 length[CODON_LENGTH];
    
@@ -889,7 +949,7 @@ s_BlastGetTranslationTable(const Uint1* genetic_code, Boolean reverse_complement
 }
 
 
-Int2 BLAST_GetAllTranslations(const Uint1* nucl_seq, Uint1 encoding,
+Int2 BLAST_GetAllTranslations(const Uint1* nucl_seq, EBlastEncoding encoding,
         Int4 nucl_length, const Uint1* genetic_code,
         Uint1** translation_buffer_ptr, Int4** frame_offsets_ptr,
         Uint1** mixed_seq_ptr)
@@ -902,14 +962,14 @@ Int2 BLAST_GetAllTranslations(const Uint1* nucl_seq, Uint1 encoding,
    Int4* frame_offsets;
    Int2 frame;
    
-   if (encoding != NCBI2NA_ENCODING && encoding != NCBI4NA_ENCODING)
+   if (encoding != eBlastEncodingNcbi2na && encoding != eBlastEncodingNcbi4na)
       return -1;
 
    if ((translation_buffer = 
         (Uint1*) malloc(2*(nucl_length+1)+1)) == NULL)
       return -1;
 
-   if (encoding == NCBI4NA_ENCODING) {
+   if (encoding == eBlastEncodingNcbi4na) {
       /* First produce the reverse strand of the nucleotide sequence */
       GetReverseNuclSequence(nucl_seq, nucl_length, 
                              &nucl_seq_rev);
@@ -924,7 +984,7 @@ Int2 BLAST_GetAllTranslations(const Uint1* nucl_seq, Uint1 encoding,
    
    for (context = 0; context < NUM_FRAMES; ++context) {
       frame = BLAST_ContextToFrame(eBlastTypeBlastx, context);
-      if (encoding == NCBI2NA_ENCODING) {
+      if (encoding == eBlastEncodingNcbi2na) {
          if (frame > 0) {
             length = 
                BLAST_TranslateCompressedSequence(translation_table,
@@ -946,7 +1006,7 @@ Int2 BLAST_GetAllTranslations(const Uint1* nucl_seq, Uint1 encoding,
       frame_offsets[context+1] = offset;
    }
 
-   if (encoding == NCBI4NA_ENCODING) {
+   if (encoding == eBlastEncodingNcbi4na) {
       sfree(nucl_seq_rev);
    } else { 
       free(translation_table);
@@ -1258,12 +1318,12 @@ Blast_SetUpSubjectTranslation(BLAST_SequenceBlk* subject_blk,
    if (!partial_translation) {
       if (is_ooframe) {
          BLAST_GetAllTranslations(subject_blk->sequence_start, 
-            NCBI4NA_ENCODING, subject_blk->length, gen_code_string, 
+            eBlastEncodingNcbi4na, subject_blk->length, gen_code_string, 
             NULL, NULL, &subject_blk->oof_sequence);
          subject_blk->oof_sequence_allocated = TRUE;
       } else {
          BLAST_GetAllTranslations(subject_blk->sequence_start, 
-            NCBI4NA_ENCODING, subject_blk->length, gen_code_string, 
+            eBlastEncodingNcbi4na, subject_blk->length, gen_code_string, 
             translation_buffer_ptr, frame_offsets_ptr, NULL);
       }
    }
@@ -1301,3 +1361,53 @@ BLAST_GetStandardAaProbabilities()
     sbp = BlastScoreBlkFree(sbp);
     return retval;
 }
+
+char* BLAST_StrToUpper(const char* string)
+{
+    char* retval = NULL;        /* the return value */
+    char* p = NULL;             /* auxiliary pointer */
+
+    if ( ! string ) {
+        return NULL;
+    }
+
+    retval = strdup(string);
+    if ( !retval ) {
+        return NULL;
+    }
+
+    for (p = retval; *p != NULLB; p++) {
+        *p = toupper((unsigned char)(*p));
+    }
+    return retval;
+}
+
+unsigned int
+BLAST_GetNumberOfContexts(EBlastProgramType p)
+{
+    unsigned int retval = 0;
+
+    switch (p) {
+    case eBlastTypeBlastn:
+    case eBlastTypePhiBlastn:
+        retval = NUM_STRANDS;
+        break;
+    case eBlastTypeBlastp:
+    case eBlastTypeRpsBlast:
+    case eBlastTypeTblastn: 
+    case eBlastTypePsiBlast:
+    case eBlastTypePsiTblastn:
+    case eBlastTypePhiBlastp:
+        retval = 1;
+        break;
+    case eBlastTypeBlastx:
+    case eBlastTypeTblastx:
+    case eBlastTypeRpsTblastn: 
+        retval = NUM_FRAMES;
+        break;
+    default:
+        break;
+    }
+
+    return retval;
+}
