@@ -1,4 +1,4 @@
-/* $Id: blast_util.h,v 1.63 2004/12/29 17:32:57 camacho Exp $
+/* $Id: blast_util.h,v 1.72 2005/11/16 14:31:36 madden Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -6,7 +6,7 @@
  *
  *  This software/database is a "United States Government Work" under the
  *  terms of the United States Copyright Act.  It was written as part of
- *  the author's offical duties as a United States Government employee and
+ *  the author's official duties as a United States Government employee and
  *  thus cannot be copyrighted.  This software/database is freely available
  *  to the public for use. The National Library of Medicine and the U.S.
  *  Government have not placed any restriction on its use or reproduction.
@@ -23,7 +23,6 @@
  *
  * ===========================================================================
  *
- * Author: Ilya Dondoshansky
  *
  */
 
@@ -35,21 +34,12 @@
 #define __BLAST_UTIL__
 
 #include <algo/blast/core/blast_def.h>
+#include <algo/blast/core/blast_encoding.h>
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-/** Different types of sequence encodings for sequence retrieval from the 
- * BLAST database 
- */
-#define BLASTP_ENCODING 0  /**< NCBIstdaa */
-#define BLASTNA_ENCODING 1 /**< Special encoding for preliminary stage of 
-                              BLAST: permutation of NCBI8na */
-#define NCBI4NA_ENCODING 2 /**< NCBI8na */
-#define NCBI2NA_ENCODING 3 /**< NCBI2na */
-#define ERROR_ENCODING 255 /**< Error value for encoding */
-
 #ifndef IS_residue
 /** Does character encode a residue? */
 #define IS_residue(x) (x <= 250)
@@ -64,7 +54,7 @@ extern "C" {
 
 /** Deallocate memory only for the sequence in the sequence block */
 NCBI_XBLAST_EXPORT
-Int2 BlastSequenceBlkClean(BLAST_SequenceBlk* seq_blk);
+void BlastSequenceBlkClean(BLAST_SequenceBlk* seq_blk);
    
 /** Deallocate memory for a sequence block */
 NCBI_XBLAST_EXPORT
@@ -127,7 +117,8 @@ Int2 BlastSeqBlkSetSequence(BLAST_SequenceBlk* seq_blk,
 
 /** Stores the compressed nucleotide sequence in the sequence block structure
  * for the subject sequence when BLASTing 2 sequences. This sequence should be
- * encoded in NCBI2NA_ENCODING and NOT have sentinel bytes.
+ * encoded in eBlastEncodingNcbi2na and NOT have sentinel bytes (as this 
+ * encoding doesn't allow them).
  * @param seq_blk The sequence block structure to modify [in/out]
  * @param sequence Actual sequence buffer. [in]
  */
@@ -202,7 +193,7 @@ Int2 GetReverseNuclSequence(const Uint1* sequence, Int4 length,
  * @param prog_number Integer corresponding to the BLAST program
  * @param context_number Context number 
  * @return Sequence frame: -1,1 for nucleotides, -3,-2,-1,1,2,3 for 
- * translations, 0 for proteins and 127 in case of unsupported program
+ * translations, 0 for proteins and INT1_MAX in case of unsupported program
 */
 NCBI_XBLAST_EXPORT
 Int1 BLAST_ContextToFrame(EBlastProgramType prog_number, Uint4 context_number);
@@ -223,6 +214,18 @@ BlastQueryInfo* BlastQueryInfoFree(BlastQueryInfo* query_info);
 NCBI_XBLAST_EXPORT
 BlastQueryInfo* BlastQueryInfoDup(BlastQueryInfo* query_info);
 
+/** Obtains the sequence length for a given query in the query, without taking
+ * into consideration any applicable translations 
+ * @param qinfo BlastQueryInfo structure [in]
+ * @param program CORE program type [in]
+ * @param query_index number of the query 
+ * (query_index < BlastQueryInfo::num_queries) [in]
+ * @return the length of the query sequence requested
+ */
+Int4 BlastQueryInfoGetQueryLength(const BlastQueryInfo* qinfo,
+                                  EBlastProgramType program,
+                                  Int4 query_index);
+
 /** Create auxiliary query structures with all data corresponding
  * to a single query sequence within a concatenated set. Allocates the 
  * structures if the pointers are NULL on input; otherwise only changes the
@@ -249,9 +252,19 @@ Int2 Blast_GetOneQueryStructs(BlastQueryInfo** one_query_info_ptr,
                               BLAST_SequenceBlk* query, Int4 query_index);
 
 
+/** Convert a sequence in ncbi4na or blastna encoding into a packed sequence
+ * in ncbi2na encoding. Needed for 2 sequences BLASTn comparison.
+ * @param buffer original sequence data (one base per byte) [in]
+ * @param length length of the sequence data above [in]
+ * @param encoding source encoding of the sequence data above [in]
+ * @param packed_seq output buffer containing compressed sequence. Its length
+ * will be (length/COMPRESSION_RATIO + 1), caller is responsible for
+ * deallocating it [out]
+ * @return 0 in case of success, -1 in case of memory allocation failure
+ */
 NCBI_XBLAST_EXPORT
-Int2 BLAST_PackDNA(Uint1* buffer, Int4 length, Uint1 encoding, 
-                   Uint1** packed_seq);
+Int2 BLAST_PackDNA(const Uint1* buffer, Int4 length, 
+                   EBlastEncoding encoding, Uint1** packed_seq);
 
 /** Initialize the mixed-frame sequence for out-of-frame gapped extension.
  * @param query_blk Sequence block containing the concatenated frames of the 
@@ -261,7 +274,7 @@ Int2 BLAST_PackDNA(Uint1* buffer, Int4 length, Uint1 encoding,
  */
 NCBI_XBLAST_EXPORT
 Int2 BLAST_InitDNAPSequence(BLAST_SequenceBlk* query_blk, 
-                       BlastQueryInfo* query_info);
+                            const BlastQueryInfo* query_info);
 
 /** Translate nucleotide into 6 frames. All frames are put into a 
  * translation buffer, with sentinel NULLB bytes in between.
@@ -279,7 +292,7 @@ Int2 BLAST_InitDNAPSequence(BLAST_SequenceBlk* query_blk,
  * @param mixed_seq_ptr Pointer to buffer for the mixed frame sequence [out]
  */
 NCBI_XBLAST_EXPORT
-Int2 BLAST_GetAllTranslations(const Uint1* nucl_seq, Uint1 encoding,
+Int2 BLAST_GetAllTranslations(const Uint1* nucl_seq, EBlastEncoding encoding,
         Int4 nucl_length, const Uint1* genetic_code, 
         Uint1** translation_buffer_ptr, Int4** frame_offsets_ptr,
         Uint1** mixed_seq_ptr);
@@ -363,6 +376,16 @@ NCBI_XBLAST_EXPORT
 double* 
 BLAST_GetStandardAaProbabilities(void);
 
+/** Returns a copy of the input string with all its characters turned to
+ * uppercase. Useful for saving score matrix names. Caller is responsible for
+ * deallocating return value.
+ * @param string string to copy [in]
+ * @return newly allocated string in upper case or NULL if string is NULL or
+ * out of memory
+ */
+char*
+BLAST_StrToUpper(const char* string);
+
 /** Maximal unpacked subject sequence length for which full translation is 
  * performed up front. 
  */
@@ -388,6 +411,13 @@ Blast_SetUpSubjectTranslation(BLAST_SequenceBlk* subject_blk,
                               Int4** frame_offsets,
                               Boolean* partial_translation);
 
+/** Get the number of contexts for a given program. This corresponds to the
+ * number of translation frames or strands whenever applicable. 
+ * @return 0 on unsupported program, non-zero otherwise
+ */
+NCBI_XBLAST_EXPORT
+unsigned int BLAST_GetNumberOfContexts(EBlastProgramType program);
+
 #ifdef __cplusplus
 }
 #endif
