@@ -1,4 +1,4 @@
-/* $Id: lookup_util.c,v 1.11 2005/03/01 14:00:56 coulouri Exp $
+/* $Id: lookup_util.c,v 1.12 2005/11/16 14:27:04 madden Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -6,7 +6,7 @@
  *
  *  This software/database is a "United States Government Work" under the
  *  terms of the United States Copyright Act.  It was written as part of
- *  the author's offical duties as a United States Government employee and
+ *  the author's official duties as a United States Government employee and
  *  thus cannot be copyrighted.  This software/database is freely available
  *  to the public for use. The National Library of Medicine and the U.S.
  *  Government have not placed any restriction on its use or reproduction.
@@ -30,7 +30,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: lookup_util.c,v 1.11 2005/03/01 14:00:56 coulouri Exp $";
+    "$Id: lookup_util.c,v 1.12 2005/11/16 14:27:04 madden Exp $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <algo/blast/core/lookup_util.h>
