@@ -1,4 +1,4 @@
-/* $Id: lookup_wrap.c,v 1.14 2005/04/27 19:55:29 dondosha Exp $
+/* $Id: lookup_wrap.c,v 1.17 2005/11/16 14:27:04 madden Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -6,7 +6,7 @@
  *
  *  This software/database is a "United States Government Work" under the
  *  terms of the United States Copyright Act.  It was written as part of
- *  the author's offical duties as a United States Government employee and
+ *  the author's official duties as a United States Government employee and
  *  thus cannot be copyrighted.  This software/database is freely available
  *  to the public for use. The National Library of Medicine and the U.S.
  *  Government have not placed any restriction on its use or reproduction.
@@ -37,7 +37,7 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: lookup_wrap.c,v 1.14 2005/04/27 19:55:29 dondosha Exp $";
+    "$Id: lookup_wrap.c,v 1.17 2005/11/16 14:27:04 madden Exp $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <algo/blast/core/lookup_wrap.h>
@@ -49,7 +49,7 @@ static char const rcsid[] =
 Int2 LookupTableWrapInit(BLAST_SequenceBlk* query, 
         const LookupTableOptions* lookup_options,	
         BlastSeqLoc* lookup_segments, BlastScoreBlk* sbp, 
-        LookupTableWrap** lookup_wrap_ptr, BlastRPSInfo *rps_info)
+        LookupTableWrap** lookup_wrap_ptr, const BlastRPSInfo *rps_info)
 {
    LookupTableWrap* lookup_wrap;
 
@@ -62,13 +62,16 @@ Int2 LookupTableWrapInit(BLAST_SequenceBlk* query,
    case AA_LOOKUP_TABLE:
        {
        Int4** matrix = NULL;
-       if (lookup_options->use_pssm) {
+       Boolean has_pssm = FALSE;
+       if (sbp->psi_matrix && sbp->psi_matrix->pssm) {
            matrix = sbp->psi_matrix->pssm->data;
+           has_pssm = TRUE;
        } else {
            matrix = sbp->matrix->data;
        }
        BlastAaLookupNew(lookup_options, (BlastLookupTable* *)
                         &lookup_wrap->lut);
+       ((BlastLookupTable*)lookup_wrap->lut)->use_pssm = has_pssm;
        BlastAaLookupIndexQuery( (BlastLookupTable*) lookup_wrap->lut, matrix, 
                                  query, lookup_segments);
        _BlastAaLookupFinalize((BlastLookupTable*) lookup_wrap->lut);
