@@ -1,4 +1,4 @@
-/* $Id: phi_gapalign.c,v 1.4 2005/04/28 15:42:16 dondosha Exp $
+/* $Id: phi_gapalign.c,v 1.9 2005/11/30 18:25:32 papadopo Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -6,7 +6,7 @@
  *
  *  This software/database is a "United States Government Work" under the
  *  terms of the United States Copyright Act.  It was written as part of
- *  the author's offical duties as a United States Government employee and
+ *  the author's official duties as a United States Government employee and
  *  thus cannot be copyrighted.  This software/database is freely available
  *  to the public for use. The National Library of Medicine and the U.S.
  *  Government have not placed any restriction on its use or reproduction.
@@ -58,13 +58,14 @@
 
 #ifndef SKIP_DOXYGEN_PROCESSING
 static char const rcsid[] = 
-    "$Id: phi_gapalign.c,v 1.4 2005/04/28 15:42:16 dondosha Exp $";
+    "$Id: phi_gapalign.c,v 1.9 2005/11/30 18:25:32 papadopo Exp $";
 #endif /* SKIP_DOXYGEN_PROCESSING */
 
 #include <algo/blast/core/blast_options.h>
 #include <algo/blast/core/blast_def.h>
 #include <algo/blast/core/phi_gapalign.h>
 #include <algo/blast/core/blast_encoding.h>
+#include <algo/blast/core/blast_gapalign.h>
 #include "blast_gapalign_priv.h"
 #include "pattern_priv.h"
 
@@ -85,6 +86,9 @@ typedef enum {
  * @param end2 Length of the second sequence segment [in]
  * @param lowDiag Low diagonal for the alignment [in]
  * @param highDiag High diagonal for the alignment [in]
+ * @param matrix Scoring matrix [in]
+ * @param gapOpen Gap opening cost [in]
+ * @param gapExtend Gap extension cost [in]
  * @param alignScript Stores traceback information. [out]
  */
 static Int4 
@@ -98,7 +102,7 @@ s_Align(Uint1 * seq1, Uint1 * seq2, Int4 end1, Int4 end2, Int4 lowDiag,
                  inclusive*/
     Int4 diagIndex; /*loop index over diagonals*/
 	Int4 leftd, rightd;	/* diagonal indices for CC, DD, CP and DP */
-    BlastGapSmallDP* score_array; /*array for dynamic program information*/
+    BlastGapDP* score_array; /*array for dynamic program information*/
 	Int4 curd;	/* current index for CC, DD CP and DP */
     Int4 i;  /*loop index*/
     Int4 index1; /*index on seq1*/
@@ -106,7 +110,7 @@ s_Align(Uint1 * seq1, Uint1 * seq2, Int4 end1, Int4 end2, Int4 lowDiag,
 	Int4 temp_indel_score = 0; /*placeholder for an indel score */
 	Int4 tempHorScore; /*dual of temp_indel_score for the case where a
                               horizontal edge (insertion) is the last step*/
-	BlastGapSmallDP* score_row = NULL; /*points to a row of CD*/
+	BlastGapDP* score_row = NULL; /*points to a row of CD*/
 	Int4 stateDecoder; /*used to decode the edge information in a state*/
     Int4 initialScore; /*score to initialize dynamic program entries*/
     Int4 *matrixRow; /*row of score matrix*/
@@ -126,7 +130,7 @@ s_Align(Uint1 * seq1, Uint1 * seq2, Int4 end1, Int4 end2, Int4 lowDiag,
 	band = highDiag-lowDiag+1;
 
     /* Allocate array of scores. */
-	score_array = (BlastGapSmallDP*) calloc(band+2, sizeof(BlastGapSmallDP));
+	score_array = (BlastGapDP*) calloc(band+2, sizeof(BlastGapDP));
 
 	state = (Int1 **) malloc(sizeof(Int1 *)*(end1+1));
 	state[0] = (Int1 *) malloc((end1+1)*(band+2));
@@ -380,7 +384,7 @@ s_PHIGetLongPattern(Uint1 *seq, Int4 len, Int4 *start, Int4 *end,
     Int4 wordIndex; /*index over words in pattern*/
     Int4  i;  /*index over seq*/
     Int4 rightMaskOnly; /*rightmost bit that is 1 in the mask only*/
-    Int4 j; /*index over bits in a word*/
+    Int4 j = 0; /*index over bits in a word*/
     Boolean found = FALSE;  /*found match position yet*/
     SLongPatternItems* multiword_items = pattern_blk->multi_word_items;
     Int4 num_words = multiword_items->numWords;
@@ -423,6 +427,9 @@ s_PHIGetLongPattern(Uint1 *seq, Int4 len, Int4 *start, Int4 *end,
     *end = wordIndex*PHI_BITS_PACKED_PER_WORD+j;
 }
 
+/** Maximal possible size of the hits array for one word of pattern. */
+#define MAX_HITS_IN_WORD 64
+
 /** Find pattern occurrences in seq when the pattern description is
  * extra long, report the results back in hitArray
  * @param seq Sequence to find pattern in [in]
@@ -439,12 +446,9 @@ s_PHIGetExtraLongPattern(Uint1 *seq, Int4 len, Int4 *hitArray,
     Int4  twiceHitsOneWord; /*Twice the number of hits against one
                               word of the pattern*/
     Int4  hitIndex; /*index over hits against one word*/
-    Int4 pos; /*keeps track of how many intermediate hits*/
-    Int4 placeIndex; /*index over the number of places in the pattern rep*/
+    Int4 pos = 0; /*keeps track of how many intermediate hits*/
     Int4 hitArray1[PHI_MAX_HIT];
-    const int kMaxHitsInWord = 64; /* Maximal possible size of the hits 
-                                      array for one word of pattern. */
-    Int4 oneWordHitArray[64 /*kMaxHitsInWord*/]; /*hits for one word of 
+    Int4 oneWordHitArray[MAX_HITS_IN_WORD]; /*hits for one word of 
                                             pattern representation*/
     SShortPatternItems* one_word_items = pattern_blk->one_word_items; 
     SLongPatternItems* multiword_items = pattern_blk->multi_word_items; 
@@ -453,41 +457,37 @@ s_PHIGetExtraLongPattern(Uint1 *seq, Int4 len, Int4 *hitArray,
 
     i = 1; 
 
-    hitArray[0] = extra_items->numPlacesInWord[0] - 1;
+    hitArray[0] = extra_items->numPlacesInWord[0];
     for (wordIndex = 1; wordIndex < num_words; wordIndex++) {
         one_word_items->whichPositionPtr = multiword_items->SLL[wordIndex]; 
         one_word_items->match_mask = multiword_items->match_maskL[wordIndex];
         pos = 0;
         for (j = 0; j < i; j += wordIndex) {
+            Int4 lastOffset = hitArray[j+wordIndex-1];
             twiceHitsOneWord = 
                 _PHIBlastFindHitsShort(oneWordHitArray, 
-                    (Uint1*) &seq[hitArray[j]+1], 
-                    MIN(len-hitArray[j]-1, extra_items->spacing[wordIndex-1] +
+                    seq + lastOffset,
+                    MIN(len-lastOffset, extra_items->spacing[wordIndex-1] +
                         extra_items->numPlacesInWord[wordIndex]),
-                                      pattern_blk);
+                    pattern_blk);
             for (hitIndex = 0; hitIndex < twiceHitsOneWord; 
                  hitIndex+= 2, pos+= wordIndex+1) {
-                hitArray1[pos] = hitArray[j]+oneWordHitArray[hitIndex]+1;
-                for (wordIndex2 = 1; wordIndex2 < wordIndex; wordIndex2++)
+                for (wordIndex2 = 0; wordIndex2 < wordIndex; wordIndex2++)
                     hitArray1[pos+wordIndex2] = hitArray[j+wordIndex2];
-                hitArray1[pos+wordIndex] = oneWordHitArray[hitIndex+1];
+                hitArray1[pos+wordIndex2] = hitArray1[pos+wordIndex2-1] +
+                                             oneWordHitArray[hitIndex] + 1;
             }
         }
         i = pos;
         for (j = 0; j < pos; j++) 
             hitArray[j] = hitArray1[j];
     }
-    for (j = 0; j < pos; j+= wordIndex) {
-        for (placeIndex = extra_items->numPlacesInWord[0], i=j+1; 
-             i < j+wordIndex; i++) 
-            placeIndex += hitArray[i]+extra_items->numPlacesInWord[i-j];
-        if (placeIndex != len) 
-            continue;
-        if (j == 0) 
+    for (j = 0; j < pos; j+= num_words) {
+        if (hitArray[j+num_words-1] == len) {
+            for (i = 0; i < num_words; i++) 
+                hitArray[i] = hitArray[i+j];
             return 0;
-        for (i = 0; i < wordIndex; i++) 
-            hitArray[i] = hitArray[i+j];
-        return 0;
+        }
     }
     /* This point should never be reached. */
     return -1;
@@ -498,14 +498,13 @@ s_PHIGetExtraLongPattern(Uint1 *seq, Int4 len, Int4 *hitArray,
  * @param dbSeq Pointer to start of pattern in subject [in]
  * @param lenQuerySeq Length of pattern occurrence in query [in]
  * @param lenDbSeq Length of pattern occurrence in subject [in]
- * alignScript is alignment script; NULL if only score is desired
- * tback will hold adjusted alignment script for return
- * gap_align keeps track of parameters for a gapped alignment
- * multiple is related to significance of the pattern match
- * the overal match score is returned; a variant of the
- * score is passed back in useful_score
+ * @param alignScript Traceback script [out]
+ * @param score_options Scoring options [in]
+ * @param score_matrix Scoring matrix [in]
+ * @param pattern_blk Structure with information about pattern [in]
+ * @return Score for this alignment.
  */
-static Int2 
+static Int4 
 s_PHIBlastAlignPatterns(Uint1 *querySeq, Uint1 *dbSeq, Int4 lenQuerySeq, 
                       Int4 lenDbSeq, GapPrelimEditBlock *alignScript,  
                       const BlastScoringOptions* score_options, 
@@ -525,11 +524,9 @@ s_PHIBlastAlignPatterns(Uint1 *querySeq, Uint1 *dbSeq, Int4 lenQuerySeq,
                                             used as indices*/
     Int4 patternPosQuery, patternPosDb; /*positions in pattern
                             for matches to query and database sequences*/
-    Int4 wordIndex; /*index over words*/
 
-    Int4 placeIndex1, placeIndex2; /*indices over places in pattern*/
+    Int4 placeIndex; /*index over places in pattern*/
     Int4  *hitArray1=NULL, *hitArray2=NULL;
-    Int4 numPlacesInWord[PHI_MAX_WORDS_IN_PATTERN];
     Int4 gap_open; /*gap opening penalty*/
     Int4 gap_extend; /*gap extension penalty*/
     Int4** matrix = score_matrix->data;
@@ -555,9 +552,8 @@ s_PHIBlastAlignPatterns(Uint1 *querySeq, Uint1 *dbSeq, Int4 lenQuerySeq,
         s_PHIGetLongPattern(dbSeq, lenDbSeq, &startDbMatch, 
                             &endDbMatch, pattern_blk);
     } else {
-        Uint1* endQuerySeq = querySeq + lenQuerySeq;
-        Uint1* endDbSeq = dbSeq + lenDbSeq;
-        Int4 num_words = multiword_items->numWords;
+        Int4 QueryWord, DbWord;
+        Int4 QueryVarSize, DbVarSize;
         SExtraLongPatternItems* extra_items = multiword_items->extra_long_items;
 
         hitArray1 = calloc(PHI_MAX_HIT, sizeof(Int4));
@@ -568,41 +564,58 @@ s_PHIBlastAlignPatterns(Uint1 *querySeq, Uint1 *dbSeq, Int4 lenQuerySeq,
                                  pattern_blk);
         s_PHIGetExtraLongPattern(dbSeq, lenDbSeq, hitArray2, pattern_blk);
 
-        for (wordIndex = 0; wordIndex < num_words; wordIndex++) 
-            numPlacesInWord[wordIndex] = 
-                extra_items->numPlacesInWord[wordIndex];
-        for (placeIndex1 = 0, wordIndex = 0; 
-             placeIndex1 < extra_items->highestPlace; placeIndex1++) {
-            if (multiword_items->inputPatternMasked[placeIndex1] < 0) {
-                for (placeIndex2 = placeIndex1-1; placeIndex2 >=0; 
-                     placeIndex2--) {
-                    if (multiword_items->inputPatternMasked[placeIndex2] != 
-                        kMaskAaAlphabetBits) 
-                        break;
-                }
-                numPlacesInWord[wordIndex++]-= placeIndex1-placeIndex2-1;
-                hitArray1[wordIndex] += placeIndex1-placeIndex2-1;
-                hitArray2[wordIndex] += placeIndex1-placeIndex2-1;
-            }
-        }
         queryMatchOffset = dbMatchOffset = 0;
-        for (wordIndex = 0; wordIndex < num_words; wordIndex++) {
-            for (placeIndex1 = 0; placeIndex1 < numPlacesInWord[wordIndex]; 
-                 placeIndex1++) {
+        QueryWord = DbWord = 0;
+        QueryVarSize = DbVarSize = 0;
+
+        for (placeIndex = 0; 
+             placeIndex < extra_items->highestPlace; placeIndex++) {
+
+            Int4 patternWord = 
+                    multiword_items->inputPatternMasked[placeIndex];
+
+            if (patternWord < 0) {
+                QueryVarSize += hitArray1[QueryWord] - 
+                                hitArray1[QueryWord-1] -
+                                extra_items->numPlacesInWord[QueryWord];
+                DbVarSize += hitArray2[DbWord] - 
+                             hitArray2[DbWord-1] -
+                             extra_items->numPlacesInWord[DbWord];
+            }
+            else if (patternWord == kMaskAaAlphabetBits) {
+                QueryVarSize++;
+                DbVarSize++;
+            }
+            else {
+                if (QueryVarSize || DbVarSize) {
+                    if (QueryVarSize == DbVarSize) {
+                        GapPrelimEditBlockAdd(alignScript, eGapAlignSub, 
+                                              QueryVarSize);
+                    }
+                    else {
+                        local_score += s_BandedAlign(querySeq-1, dbSeq-1, 
+                                         QueryVarSize, DbVarSize, 
+                                         kBandLow, kBandHigh, matrix, 
+                                         gap_open, gap_extend, alignScript);
+                    }
+                    queryMatchOffset += QueryVarSize;
+                    querySeq += QueryVarSize;
+                    dbMatchOffset += DbVarSize;
+                    dbSeq += DbVarSize;
+                }
+
                 GapPrelimEditBlockAdd(alignScript, eGapAlignSub, 1);
+                queryMatchOffset++;
                 querySeq++;
+                dbMatchOffset++;
                 dbSeq++;
-                if (querySeq == endQuerySeq || dbSeq == endDbSeq)
-                    break;
-            }
-            if (wordIndex < num_words-1) { 
-                local_score += 
-                    s_BandedAlign(querySeq-1, dbSeq-1, hitArray1[wordIndex+1], 
-                            hitArray2[wordIndex+1], kBandLow, kBandHigh, matrix, 
-                            gap_open, gap_extend, alignScript);
-                querySeq += hitArray1[wordIndex+1];
-                dbSeq += hitArray2[wordIndex+1];
+                QueryVarSize = DbVarSize = 0;
             }
+
+            if (queryMatchOffset + QueryVarSize >= hitArray1[QueryWord])
+                QueryWord++;
+            if (dbMatchOffset + DbVarSize >= hitArray2[DbWord])
+                DbWord++;
         }
 
         sfree(hitArray1);
@@ -893,6 +906,5 @@ Int2 PHIGappedAlignmentWithTraceback(Uint1* query, Uint1* subject,
                                              fwd_prelim_tback);
 
     gap_align->score = score_right + score_left;
-
     return status;
 }
