@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   10/21/98
 *
-* $Revision: 6.63 $
+* $Revision: 6.68 $
 *
 * File Description:  New GenBank flatfile generator
 *
@@ -116,6 +116,8 @@ typedef unsigned long FlgType;
 
 #define DDBJ_VARIANT_FORMAT    32768
 
+#define SPECIAL_GAP_DISPLAY    65536
+
 /* locking behavior for system performance */
 
 typedef unsigned long LckType;
@@ -149,6 +151,7 @@ typedef unsigned long CstType;
 #define ONLY_REVIEW_PUBS         768
 #define NEWEST_PUBS             1024
 #define OLDEST_PUBS             1280
+#define HIDE_ALL_PUBS           1792
 
 #define SHOW_PROT_FTABLE        2048
 #define SHOW_FTABLE_REFS        4096
@@ -157,6 +160,9 @@ typedef unsigned long CstType;
 #define HIDE_GAP_FEATS         16384
 #define HIDE_SEQUENCE          32768
 
+#define EXPANDED_GAP_DISPLAY   65536
+#define HIDE_GO_TERMS         131072
+
 /* opaque pointer for special extensions */
 
 struct XtraData;
