@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_CONNUTIL__H
 #define CONNECT___NCBI_CONNUTIL__H
 
-/*  $Id: ncbi_connutil.h,v 6.42 2005/04/20 15:47:24 lavr Exp $
+/*  $Id: ncbi_connutil.h,v 6.44 2005/11/29 21:32:07 lavr Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -80,6 +80,9 @@
  *       StringToHostPort()
  *       HostPortToString()
  *
+ *    8.CRC32
+ *       CRC32_Update()
+ *
  */
 
 #include <connect/ncbi_buffer.h>
@@ -98,6 +101,15 @@ extern "C" {
 #endif
 
 
+typedef enum {
+    eURL_Unspec = 0,
+    eURL_Https,
+    eURL_Http,
+    eURL_File,
+    eURL_Ftp
+} EURLScheme;
+
+
 typedef enum {
     eReqMethod_Any = 0,
     eReqMethod_Post,
@@ -116,14 +128,18 @@ typedef enum {
  * ATTENTION:  Do NOT fill out this structure (SConnNetInfo) "from scratch"!
  *             Instead, use ConnNetInfo_Create() described below to create
  *             it, and then fix (hard-code) some fields, if really necessary.
+ * NOTE:       "scheme", "user" and "pass" are reserved (unused) fields.
  */
 typedef struct {
     char           client_host[256]; /* effective client hostname            */
+    EURLScheme     scheme;           /* only pre-defined types (limited)     */
+    char           user[128];        /* username (if specified)              */
+    char           pass[128];        /* password (if any, clear text!!!)     */
     char           host[256];        /* host to connect to                   */
     unsigned short port;             /* port to connect to, host byte order  */
     char           path[1024];       /* service: path(e.g. to  a CGI script) */
     char           args[1024];       /* service: args(e.g. for a CGI script) */
-    EReqMethod     req_method;       /* method to use in the request         */
+    EReqMethod     req_method;       /* method to use in the request (HTTP)  */
     STimeout*      timeout;          /* ptr to i/o tmo (infinite if NULL)    */
     unsigned short max_try;          /* max. # of attempts to connect (>= 1) */
     char           http_proxy_host[256]; /* hostname of HTTP proxy server    */
@@ -135,7 +151,7 @@ typedef struct {
     int/*bool*/    lb_disable;       /* to disable local load-balancing      */
     const char*    http_user_header; /* user header to add to HTTP request   */
 
-    /* the following field(s) are for the internal use only! */
+    /* the following field(s) are for the internal use only -- don't touch!  */
     int/*bool*/    http_proxy_adjusted;
     STimeout       tmo;              /* default storage for finite timeout   */
     const char*    service;          /* service for which this info created  */
@@ -686,6 +702,17 @@ extern NCBI_XCONNECT_EXPORT size_t HostPortToString
  );
 
 
+/* Calculate/Update CRC32
+ * Return the checksum updated according to the contents of the block
+ * pointed to by "ptr" and having "count" bytes in it.
+ */
+extern NCBI_XCONNECT_EXPORT unsigned int CRC32_Update
+(unsigned int checksum,  /* Checksum to update (start with 0) */
+ const void*  ptr,       /* Block of data                     */
+ size_t       count      /* Size of data                      */
+ );
+
+
 #ifdef __cplusplus
 }  /* extern "C" */
 #endif
@@ -697,6 +724,12 @@ extern NCBI_XCONNECT_EXPORT size_t HostPortToString
 /*
  * --------------------------------------------------------------------------
  * $Log: ncbi_connutil.h,v $
+ * Revision 6.44  2005/11/29 21:32:07  lavr
+ * Reserve SConnNetInfo::scheme, user, and pass for future use
+ *
+ * Revision 6.43  2005/11/29 19:51:51  lavr
+ * +CRC32 (pure C interface; not a ZIP version)
+ *
  * Revision 6.42  2005/04/20 15:47:24  lavr
  * DEF_CONN_REQ_METHOD changed to ANY
  *
