@@ -1,4 +1,4 @@
-/*  $Id: ncbi_http_connector.c,v 6.65 2005/03/08 16:23:13 lavr Exp $
+/*  $Id: ncbi_http_connector.c,v 6.68 2005/11/21 21:09:31 lavr Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -291,6 +291,7 @@ static EIO_Status s_ConnectAndSend(SHttpConnector* uuu,int/*bool*/ drop_unread)
         SOCK_Abort(uuu->sock);
         s_DropConnection(uuu, 0/*no wait*/);
         if (!s_Adjust(uuu, &null, drop_unread)) {
+            uuu->can_connect = eCC_None;
             status = eIO_Closed;
             break;
         }
@@ -492,8 +493,10 @@ static EIO_Status s_PreRead(SHttpConnector* uuu,
         /* HTTP header read error; disconnect and try to use another server */
         SOCK_Abort(uuu->sock);
         s_DropConnection(uuu, 0/*no wait*/);
-        if (!s_Adjust(uuu, &redirect, drop_unread))
+        if (!s_Adjust(uuu, &redirect, drop_unread)) {
+            uuu->can_connect = eCC_None;
             break;
+        }
         assert(redirect == 0);
     }
     assert(redirect == 0);
@@ -589,7 +592,7 @@ static void s_FlushAndDisconnect(SHttpConnector* uuu,
                                  const STimeout* timeout,
                                  int/*bool*/     close)
 {
-    size_t w_size = BUF_Size(uuu->w_buf);
+    size_t w_size;
 
     /* store timeouts for later use */
     if (timeout) {
@@ -611,6 +614,7 @@ static void s_FlushAndDisconnect(SHttpConnector* uuu,
     assert(!uuu->sock);
 
     /* clear pending output data, if any */
+    w_size = BUF_Size(uuu->w_buf);
     if (w_size  &&  BUF_Read(uuu->w_buf, 0, w_size) != w_size) {
         CORE_LOG(eLOG_Error, "[HTTP]  Cannot drop output buffer");
         assert(0);
@@ -796,6 +800,7 @@ static EIO_Status s_VT_Flush
 (CONNECTOR       connector,
  const STimeout* timeout)
 {
+    const STimeout zero = {0, 0};
     SHttpConnector* uuu = (SHttpConnector*) connector->handle;
 
     /* The real flush will be performed on the first "READ" (or "CLOSE"),
@@ -807,7 +812,10 @@ static EIO_Status s_VT_Flush
     } else
         uuu->w_timeout  = timeout;
 
-    return eIO_Success;
+    assert(connector->meta);
+    return (!(uuu->flags & fHCC_Flushable)  ||  !connector->meta->wait
+            ? eIO_Success
+            : connector->meta->wait(connector->meta->c_wait, eIO_Read, &zero));
 }
 
 
@@ -975,6 +983,15 @@ extern CONNECTOR HTTP_CreateConnectorEx
 /*
  * --------------------------------------------------------------------------
  * $Log: ncbi_http_connector.c,v $
+ * Revision 6.68  2005/11/21 21:09:31  lavr
+ * Fix compilation error
+ *
+ * Revision 6.67  2005/11/21 21:04:05  lavr
+ * Fix double flush error at Close
+ *
+ * Revision 6.66  2005/08/12 16:11:21  lavr
+ * Implement fHCC_Flushable
+ *
  * Revision 6.65  2005/03/08 16:23:13  lavr
  * Replace an assert() with verify() to remove release-mode unused var
  *
