@@ -1,4 +1,4 @@
-/*  $Id: ncbi_service.c,v 6.61 2005/04/25 18:47:29 lavr Exp $
+/*  $Id: ncbi_service.c,v 6.68 2005/08/31 19:25:16 lavr Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -81,15 +81,6 @@ char* SERV_ServiceName(const char* service)
 }
 
 
-SERV_ITER SERV_OpenSimple(const char* service)
-{
-    SConnNetInfo* net_info = ConnNetInfo_Create(service);
-    SERV_ITER iter = SERV_Open(service, fSERV_Any, 0, net_info);
-    ConnNetInfo_Destroy(net_info);
-    return iter;
-}
-
-
 static int/*bool*/ s_AddSkipInfo(SERV_ITER iter, SSERV_Info* info)
 {
     if (info->type == fSERV_Firewall) {
@@ -109,9 +100,9 @@ static int/*bool*/ s_AddSkipInfo(SERV_ITER iter, SSERV_Info* info)
             }
         }
     }
-    if (iter->n_skip == iter->n_max_skip) {
+    if (iter->n_skip == iter->a_skip) {
         SSERV_Info** temp;
-        size_t n = iter->n_max_skip + 10;
+        size_t n = iter->a_skip + 10;
 
         if (iter->skip)
             temp = (SSERV_Info**) realloc(iter->skip, sizeof(*temp) * n);
@@ -121,7 +112,7 @@ static int/*bool*/ s_AddSkipInfo(SERV_ITER iter, SSERV_Info* info)
             return 0;
 
         iter->skip = temp;
-        iter->n_max_skip = n;
+        iter->a_skip = n;
     }
     iter->skip[iter->n_skip++] = info;
     return 1;
@@ -133,48 +124,49 @@ static SERV_ITER s_Open(const char* service, TSERV_Type types,
                         const SConnNetInfo* net_info,
                         const SSERV_Info* const skip[], size_t n_skip,
                         SSERV_Info** info, HOST_INFO* host_info,
-                        int/*bool*/ external, unsigned int origin,
+                        unsigned/*bool*/ ismask, unsigned/*bool*/ external,
                         const char* arg, const char* val)
 {
-    const char* s = s_ServiceName(service, 0);
+    const char* s = ismask ? strdup(service) : s_ServiceName(service, 0);
     const SSERV_VTable* op;
     SERV_ITER iter;
     
     if (!s || !*s || !(iter = (SERV_ITER) malloc(sizeof(*iter))))
         return 0;
 
-    iter->service        = s;
-    iter->current        = 0;
-    iter->types          = types;
-    iter->preferred_host = preferred_host == SERV_LOCALHOST
-        ? SOCK_gethostbyname(0) : preferred_host;
-    iter->preference = 0.01*(preference < 0.0   ? 0.0   :
-                             preference > 100.0 ? 100.0 : preference);
-    iter->n_skip         = iter->n_max_skip = 0;
-    iter->skip           = 0;
-    iter->last           = 0;
-    iter->op             = 0;
-    iter->data           = 0;
-    iter->external       = external;
-    iter->origin         = origin;
-    iter->arg            = arg;
-    iter->arglen         = arg ? strlen(arg) : 0;
-    iter->val            = val;
-    iter->vallen         = val ? strlen(val) : 0;
+    iter->name     = s;
+    iter->type     = types;
+    iter->host     = (preferred_host == SERV_LOCALHOST
+                      ? SOCK_gethostbyname(0) : preferred_host);
+    iter->pref     = 0.01*(preference < 0.0   ? 0.0   :
+                           preference > 100.0 ? 100.0 : preference);
+    iter->n_skip   = 0;
+    iter->a_skip   = 0;
+    iter->skip     = 0;
+    iter->last     = 0;
+    iter->op       = 0;
+    iter->data     = 0;
+    iter->mask     = ismask;
+    iter->external = external;
+    iter->arg      = arg;
+    iter->arglen   = arg ? strlen(arg) : 0;
+    iter->val      = val;
+    iter->vallen   = val ? strlen(val) : 0;
 
     if (n_skip) {
         size_t i;
         for (i = 0; i < n_skip; i++) {
-            size_t   skipinfolen = SERV_SizeOfInfo(skip[i]);
-            SSERV_Info* skipinfo = (SSERV_Info*) malloc(skipinfolen);
-            if (!skipinfo) {
+            size_t skipinfolen = SERV_SizeOfInfo(skip[i]);
+            SSERV_Info*   temp = (SSERV_Info*) malloc(skipinfolen + 1);
+            if (!temp) {
                 SERV_Close(iter);
                 return 0;
             }
-            memcpy(skipinfo, skip[i], skipinfolen);
-            skipinfo->time = (TNCBI_Time)(-1);
-            if (!s_AddSkipInfo(iter, skipinfo)) {
-                free(skipinfo);
+            *((char*) temp + skipinfolen) = '\0';
+            memcpy(temp, skip[i], skipinfolen);
+            temp->time = (TNCBI_Time)(-1);
+            if (!s_AddSkipInfo(iter, temp)) {
+                free(temp);
                 SERV_Close(iter);
             }
         }
@@ -189,9 +181,9 @@ static SERV_ITER s_Open(const char* service, TSERV_Type types,
         }
     } else {
         if (net_info->stateless)
-            iter->types |= fSERV_StatelessOnly;
+            iter->type |= fSERV_StatelessOnly;
         if (net_info->firewall)
-            iter->types |= fSERV_Firewall;
+            iter->type |= fSERV_Firewall;
         if ((net_info->lb_disable ||
              !(op = SERV_LBSMD_Open(iter, info, host_info))) &&
             !(op = SERV_DISPD_Open(iter, net_info, info, host_info))) {
@@ -206,6 +198,26 @@ static SERV_ITER s_Open(const char* service, TSERV_Type types,
 }
 
 
+SERV_ITER SERV_OpenSimple(const char* service)
+{
+    SConnNetInfo* net_info = ConnNetInfo_Create(service);
+    SERV_ITER iter = SERV_Open(service, fSERV_Any, 0, net_info);
+    ConnNetInfo_Destroy(net_info);
+    return iter;
+}
+
+
+SERV_ITER SERV_Open(const char* service,
+                    TSERV_Type types, unsigned int preferred_host,
+                    const SConnNetInfo* net_info)
+{
+    return s_Open(service, types, preferred_host, 0.0/*preference*/,
+                  net_info, 0/*skip*/, 0/*n_skip*/, 0/*info*/, 0/*host_info*/,
+                  0/*not mask*/, 0/*not external*/,
+                  0/*arg*/, 0/*val*/);
+}
+
+
 SERV_ITER SERV_OpenEx(const char* service,
                       TSERV_Type types, unsigned int preferred_host,
                       const SConnNetInfo* net_info,
@@ -213,18 +225,20 @@ SERV_ITER SERV_OpenEx(const char* service,
 {
     return s_Open(service, types, preferred_host, 0.0/*preference*/,
                   net_info, skip, n_skip, 0/*info*/, 0/*host_info*/,
-                  0/*not external*/, 0/*origin*/, 0/*arg*/, 0/*val*/);
+                  0/*not mask*/, 0/*not external*/,
+                  0/*arg*/, 0/*val*/);
 }
 
 
 SERV_ITER SERV_OpenP(const char* service, TSERV_Type types,
                      unsigned int preferred_host, double preference,
                      int/*bool*/ external, SConnNetInfo* net_info,
-                     unsigned int origin, const char* arg, const char* val)
+                     const char* arg, const char* val)
 {
     return s_Open(service, types, preferred_host, preference,
-                  net_info, 0/*skip*/, 0/*n_skip*/, 0/*info*/, 0/*hinfo*/,
-                  external, origin, arg, val);
+                  net_info, 0/*skip*/, 0/*n_skip*/, 0/*info*/, 0/*host_info*/,
+                  strpbrk(service, "?*") != 0, external,
+                  arg, val);
 }
 
 
@@ -232,14 +246,13 @@ static SSERV_Info* s_GetInfo(const char* service, TSERV_Type types,
                              unsigned int preferred_host, double preference,
                              const SConnNetInfo* net_info,
                              const SSERV_Info* const skip[], size_t n_skip,
-                             HOST_INFO* host_info,
-                             int/*bool*/ external, unsigned int origin,
+                             HOST_INFO* host_info, unsigned /*bool*/ external,
                              const char* arg, const char* val)
 {
     SSERV_Info* info = 0;
     SERV_ITER iter= s_Open(service, types, preferred_host, preference,
                            net_info, skip, n_skip, &info, host_info,
-                           external, origin, arg, val);
+                           0/*not mask*/, external, arg, val);
     if (iter && !info && iter->op && iter->op->GetNextInfo)
         info = (*iter->op->GetNextInfo)(iter, host_info);
     SERV_Close(iter);
@@ -247,6 +260,16 @@ static SSERV_Info* s_GetInfo(const char* service, TSERV_Type types,
 }
 
 
+SSERV_Info* SERV_GetInfo(const char* service, TSERV_Type types,
+                         unsigned int preferred_host,
+                         const SConnNetInfo* net_info)
+{
+    return s_GetInfo(service, types, preferred_host, 0.0/*preference*/,
+                     net_info, 0/*skip*/, 0/*n_skip*/, 0/*host_info*/,
+                     0/*not external*/, 0/*arg*/, 0/*val*/);
+}
+
+
 SSERV_Info* SERV_GetInfoEx(const char* service, TSERV_Type types,
                            unsigned int preferred_host,
                            const SConnNetInfo* net_info,
@@ -255,18 +278,18 @@ SSERV_Info* SERV_GetInfoEx(const char* service, TSERV_Type types,
 {
     return s_GetInfo(service, types, preferred_host, 0.0/*preference*/,
                      net_info, skip, n_skip, host_info,
-                     0/*not external*/, 0/*origin*/, 0/*arg*/, 0/*val*/);
+                     0/*not external*/, 0/*arg*/, 0/*val*/);
 }
 
 
 SSERV_Info* SERV_GetInfoP(const char* service, TSERV_Type types,
                           unsigned int preferred_host, double preference,
                           int/*bool*/ external, SConnNetInfo* net_info,
-                          unsigned int origin, const char* arg,const char* val)
+                          const char* arg,const char* val)
 {
     return s_GetInfo(service, types, preferred_host, preference,
                      net_info, 0/*skip*/, 0/*n_skip*/, 0/*host_info*/,
-                     external, origin, arg, val);
+                     external, arg, val);
 }
 
 
@@ -293,35 +316,48 @@ static void s_SkipSkip(SERV_ITER iter)
 }
 
 
-const SSERV_Info* SERV_GetNextInfoEx(SERV_ITER iter, HOST_INFO* host_info)
+static const SSERV_Info* s_GetNextInfo(SERV_ITER iter,
+                                       HOST_INFO* host_info)
 {
     SSERV_Info* info = 0;
-
-    if (iter && iter->op) {
-        /* First, remove all outdated entries from our skip list */
-        s_SkipSkip(iter);
-        /* Next, obtain a fresh entry from the actual mapper */
-        if (iter->op->GetNextInfo &&
-            (info = (*iter->op->GetNextInfo)(iter, host_info)) != 0 &&
-            !s_AddSkipInfo(iter, info)) {
-            free(info);
-            info = 0;
-        }
-        iter->last = info;
+    assert(iter && iter->op);
+    /* First, remove all outdated entries from our skip list */
+    s_SkipSkip(iter);
+    /* Next, obtain a fresh entry from the actual mapper */
+    if (iter->op->GetNextInfo &&
+        (info = (*iter->op->GetNextInfo)(iter, host_info)) != 0 &&
+        !s_AddSkipInfo(iter, info)) {
+        free(info);
+        info = 0;
     }
+    iter->last = info;
     return info;
 }
 
 
+const SSERV_Info* SERV_GetNextInfoEx(SERV_ITER iter, HOST_INFO* host_info)
+{
+    return iter && iter->op ? s_GetNextInfo(iter, host_info) : 0;
+}
+
+
+const SSERV_Info* SERV_GetNextInfo(SERV_ITER iter)
+{
+    return iter && iter->op ? s_GetNextInfo(iter, 0) : 0;
+}
+
+
 const char* SERV_MapperName(SERV_ITER iter)
 {
     return iter && iter->op ? iter->op->name : 0;
 }
 
 
-const char* SERV_GetCurrentName(SERV_ITER iter)
+const char* SERV_CurrentName(SERV_ITER iter)
 {
-    return iter->current ? iter->current : iter->service;
+    const char* name =
+        iter->last? (const char*) iter->last + SERV_SizeOfInfo(iter->last) : 0;
+    return name  &&  *name ? name : iter->name;
 }
 
 
@@ -333,21 +369,11 @@ int/*bool*/ SERV_Penalize(SERV_ITER iter, double fine)
 }
 
 
-char* SERV_GetConfig(void)
-{
-    return SERV_LBSMD_GetConfig();
-}
-
-
 void SERV_Reset(SERV_ITER iter)
 {
     size_t i;
     if (!iter)
         return;
-    if (iter->current) {
-        free((void*) iter->current);
-        iter->current = 0;
-    }
     for (i = 0; i < iter->n_skip; i++)
         free(iter->skip[i]);
     iter->n_skip = 0;
@@ -366,8 +392,8 @@ void SERV_Close(SERV_ITER iter)
         (*iter->op->Close)(iter);
     if (iter->skip)
         free(iter->skip);
-    if (iter->service)
-        free((void*) iter->service);
+    if (iter->name)
+        free((void*) iter->name);
     free(iter);
 }
 
@@ -415,14 +441,14 @@ int/*bool*/ SERV_Update(SERV_ITER iter, const char* text)
 }
 
 
-char* SERV_PrintEx(SERV_ITER iter, const SConnNetInfo* referrer)
+char* SERV_Print(SERV_ITER iter, const SConnNetInfo* referrer)
 {
     static const char referrer_header[] = "Referer: "/*standard misspelling*/;
     static const char client_revision[] = "Client-Revision: %hu.%hu\r\n";
     static const char accepted_types[] = "Accepted-Server-Types:";
     static const char server_count[] = "Server-Count: all\r\n";
     char buffer[128], *str;
-    TSERV_Type types, t;
+    TSERV_Type type, t;
     size_t buflen, i;
     BUF buf = 0;
 
@@ -440,7 +466,7 @@ char* SERV_PrintEx(SERV_ITER iter, const SConnNetInfo* referrer)
             const char* host = referrer->host;
             const char* path = referrer->path;
             const char* args = referrer->args;
-            const char* service = iter->service;
+            const char* name = iter->name;
             char port[8];
             if (referrer->port && referrer->port != DEF_CONN_PORT)
                 sprintf(port, ":%hu", referrer->port);
@@ -452,7 +478,7 @@ char* SERV_PrintEx(SERV_ITER iter, const SConnNetInfo* referrer)
                 !BUF_Write(&buf, port, strlen(port))                         ||
                 !BUF_Write(&buf, path, strlen(path))                         ||
                 !BUF_Write(&buf, "?service=", 9)                             ||
-                !BUF_Write(&buf, service, strlen(service))                   ||
+                !BUF_Write(&buf, name, strlen(name))                         ||
                 (args[0] && (!BUF_Write(&buf, "&", 1)                        ||
                              !BUF_Write(&buf, args, strlen(args))))          ||
                 !BUF_Write(&buf, "\r\n", 2)) {
@@ -461,12 +487,12 @@ char* SERV_PrintEx(SERV_ITER iter, const SConnNetInfo* referrer)
             }
         } else if (refer && strcasecmp(iter->op->name, "LBSMD") == 0) {
             const char* host = referrer->client_host;
-            const char* service = iter->service;
+            const char* name = iter->name;
             if (!BUF_Write(&buf, referrer_header, sizeof(referrer_header)-1) ||
                 !BUF_Write(&buf, "lbsm://", 7)                               ||
                 !BUF_Write(&buf, host, strlen(host))                         ||
-                !BUF_Write(&buf, "/dispatch?service=", 18)                   ||
-                !BUF_Write(&buf, service, strlen(service))                   ||
+                !BUF_Write(&buf, "/lbsmd?service=", 15)                      ||
+                !BUF_Write(&buf, name, strlen(name))                         ||
                 !BUF_Write(&buf, "\r\n", 2)) {
                 BUF_Destroy(buf);
                 return 0;
@@ -475,9 +501,11 @@ char* SERV_PrintEx(SERV_ITER iter, const SConnNetInfo* referrer)
         /* Form accepted server types */
         buflen = sizeof(accepted_types) - 1;
         memcpy(buffer, accepted_types, buflen);
-        types = (TSERV_Type) (iter->types & ~fSERV_StatelessOnly);
+        type = (TSERV_Type) (iter->type & ~(fSERV_Promiscuous |
+                                            fSERV_ReverseDns  |
+                                            fSERV_StatelessOnly));
         for (t = 1; t; t <<= 1) {
-            if (types & t) {
+            if (type & t) {
                 const char* name = SERV_TypeStr((ESERV_Type) t);
                 size_t namelen = strlen(name);
                 if (!namelen || buflen + 1 + namelen + 2 >= sizeof(buffer))
@@ -495,8 +523,8 @@ char* SERV_PrintEx(SERV_ITER iter, const SConnNetInfo* referrer)
                 return 0;
             }
         }
-        /* how many for the dispatcher to send to us */
-        if (iter->preference) {
+        /* how many server-infos for the dispatcher to send to us */
+        if (iter->pref) {
             if (!BUF_Write(&buf, server_count, sizeof(server_count) - 1)) {
                 BUF_Destroy(buf);
                 return 0;
@@ -539,15 +567,6 @@ char* SERV_PrintEx(SERV_ITER iter, const SConnNetInfo* referrer)
 }
 
 
-#ifdef SERV_Print
-#  undef SERV_Print
-#endif
-char* SERV_Print(SERV_ITER iter)
-{
-    return SERV_PrintEx(iter, 0);
-}
-
-
 /*
  * Note parameters' ranges here:
  * 0.0 <= pref <= 1.0
@@ -574,6 +593,27 @@ double SERV_Preference(double pref, double gap, unsigned int n)
 /*
  * --------------------------------------------------------------------------
  * $Log: ncbi_service.c,v $
+ * Revision 6.68  2005/08/31 19:25:16  lavr
+ * Fix number of bytes written in new referer
+ *
+ * Revision 6.67  2005/08/31 19:02:17  lavr
+ * Change LBSMD referer HTTP tag
+ *
+ * Revision 6.66  2005/07/11 18:49:11  lavr
+ * Hashed preference generation algorithm retired (proven to fail often)
+ *
+ * Revision 6.65  2005/07/11 18:16:00  lavr
+ * Revised to allow wildcard searches thru service iterator
+ *
+ * Revision 6.64  2005/07/06 18:54:54  lavr
+ * +enum ESERV_SpecialType to hold special server type bits (instead of macros)
+ *
+ * Revision 6.63  2005/07/06 18:28:20  lavr
+ * Eliminate macro calls -- replaced with real function calls
+ *
+ * Revision 6.62  2005/05/04 16:14:48  lavr
+ * -SERV_GetConfig()
+ *
  * Revision 6.61  2005/04/25 18:47:29  lavr
  * Private API to accept SConnNetInfo* for network dispatching to work too
  *
@@ -596,7 +636,7 @@ double SERV_Preference(double pref, double gap, unsigned int n)
  * SERV_ITER::type renamed into SERV_ITER::types to reflect its bitmask nature
  *
  * Revision 6.54  2004/07/01 16:28:19  lavr
- * +SERV_PrintEx(): "Referer:" tag impelemented
+ * +SERV_Print(): "Referer:" tag impelemented
  *
  * Revision 6.53  2004/06/14 16:37:09  lavr
  * Allow no more than one firewall server info in the skip list
