@@ -1,7 +1,7 @@
 #ifndef CONNECT___NCBI_SERVICE__H
 #define CONNECT___NCBI_SERVICE__H
 
-/*  $Id: ncbi_service.h,v 6.36 2005/03/05 21:04:11 lavr Exp $
+/*  $Id: ncbi_service.h,v 6.41 2005/10/03 15:55:22 lavr Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -30,8 +30,6 @@
  *
  * File Description:
  *   Top-level API to resolve NCBI service name to the server meta-address.
- *   More elaborate documentation could be found in:
- *   http://www.ncbi.nlm.nih.gov/IEB/ToolBox/CPP_DOC/libs/conn.html#ref_ServiceAPI
  *
  */
 
@@ -39,9 +37,9 @@
 #include <connect/ncbi_host_info.h>
 
 
-/* Revision 6.100 */
+/* Revision 6.200 */
 #define SERV_CLIENT_REVISION_MAJOR  6
-#define SERV_CLIENT_REVISION_MINOR  101
+#define SERV_CLIENT_REVISION_MINOR  200
 
 
 /** @addtogroup ServiceSupport
@@ -89,10 +87,14 @@ extern NCBI_XCONNECT_EXPORT SERV_ITER SERV_OpenSimple
 #define SERV_LOCALHOST  ((unsigned int)(~0UL))
 #define SERV_ANYHOST    0           /* default, may be used as just 0 in code*/
 
-/* Can be combined in types to get even dead services (not off ones!) */
-#define fSERV_Promiscuous 0x80000000
-/* Do reverse DNS translation of the resulting info */
-#define fSERV_ReverseDns  0x40000000
+
+/* Special "type" bit values that may be combined with server types */
+typedef enum {
+    /* Do reverse DNS translation of the resulting info */
+    fSERV_ReverseDns  = 0x40000000,
+    /* Allows to get even dead services (not off ones!) */
+    fSERV_Promiscuous = 0x20000000
+} ESERV_SpecialType;
 
 
 extern NCBI_XCONNECT_EXPORT SERV_ITER SERV_OpenEx
@@ -104,8 +106,13 @@ extern NCBI_XCONNECT_EXPORT SERV_ITER SERV_OpenEx
  size_t              n_skip         /* number of servers in preceding array  */
  );
 
-#define SERV_Open(service, types, preferred_host, net_info) \
-        SERV_OpenEx(service, types, preferred_host, net_info, 0, 0)
+
+extern NCBI_XCONNECT_EXPORT SERV_ITER SERV_Open
+(const char*         service,
+ TSERV_Type          types,
+ unsigned int        preferred_host,
+ const SConnNetInfo* net_info
+);
 
 
 /* Get the next server meta-address, optionally accompanied by host
@@ -128,10 +135,13 @@ extern NCBI_XCONNECT_EXPORT SERV_ITER SERV_OpenEx
  */
 extern NCBI_XCONNECT_EXPORT const SSERV_Info* SERV_GetNextInfoEx
 (SERV_ITER           iter,          /* handle obtained via 'SERV_Open*' call */
- HOST_INFO*          host_info      /* ptr to store host info to [may be 0]  */
+ HOST_INFO*          host_info      /* ptr to store host info at [may be 0]  */
  );
 
-#define SERV_GetNextInfo(iter)  SERV_GetNextInfoEx(iter, 0)
+
+extern NCBI_XCONNECT_EXPORT const SSERV_Info* SERV_GetNextInfo
+(SERV_ITER           iter
+ );
 
 
 /* This is a 'fast track' routine equivalent to creation of an iterator
@@ -149,12 +159,16 @@ extern NCBI_XCONNECT_EXPORT SSERV_Info* SERV_GetInfoEx
  const SConnNetInfo* net_info,      /* connection information                */
  const SSERV_Info*   const skip[],  /* array of servers NOT to select        */
  size_t              n_skip,        /* number of servers in preceding array  */
- HOST_INFO*          host_info      /* ptr to store host info to [may be 0]  */
+ HOST_INFO*          host_info      /* ptr to store host info at [may be 0]  */
  );
 
 
-#define SERV_GetInfo(service, types, preferred_host, net_info) \
-    SERV_GetInfoEx(service, types, preferred_host, net_info, 0, 0, 0)
+extern NCBI_XCONNECT_EXPORT SSERV_Info* SERV_GetInfo
+(const char*         service,
+ TSERV_Type          types,
+ unsigned int        preferred_host,
+ const SConnNetInfo* net_info
+ );
 
 
 /* Penalize server returned last from SERV_GetNextInfo[Ex]().
@@ -182,17 +196,6 @@ extern NCBI_XCONNECT_EXPORT void SERV_Close
  );
 
 
-/* Set message hook procedure for messages originating from NCBI network
- * dispatcher (if used).  Any hook will be called not more than once.
- * If no hook is installed, warning message will be generated in the
- * standard log file, not more than once.
- */
-
-typedef void (*FDISP_MessageHook)(const char* message);
-
-extern NCBI_XCONNECT_EXPORT void DISP_SetMessageHook(FDISP_MessageHook);
-
-
 #ifdef __cplusplus
 }  /* extern "C" */
 #endif
@@ -204,6 +207,21 @@ extern NCBI_XCONNECT_EXPORT void DISP_SetMessageHook(FDISP_MessageHook);
 /*
  * --------------------------------------------------------------------------
  * $Log: ncbi_service.h,v $
+ * Revision 6.41  2005/10/03 15:55:22  lavr
+ * Change ESERV_SpecialType::fSERV_Promiscuous to fit "signed int"
+ *
+ * Revision 6.40  2005/09/02 14:28:18  lavr
+ * Bump client revision number to 6.200 for use with patched PubFetch
+ *
+ * Revision 6.39  2005/07/06 18:54:44  lavr
+ * +enum ESERV_SpecialType to hold special server type bits (instead of macros)
+ *
+ * Revision 6.38  2005/07/06 18:27:58  lavr
+ * Eliminate macro calls
+ *
+ * Revision 6.37  2005/05/04 16:13:57  lavr
+ * DISP_SetMessageHook() moved to <connect/ncbi_service_misc.h>
+ *
  * Revision 6.36  2005/03/05 21:04:11  lavr
  * +fSERV_ReverseDns
  *
