@@ -1,4 +1,4 @@
-/*  $Id: test_ncbi_heapmgr.c,v 6.12 2003/08/25 14:58:10 lavr Exp $
+/*  $Id: test_ncbi_heapmgr.c,v 6.14 2005/07/11 18:24:46 lavr Exp $
  * ===========================================================================
  *
  *                            PUBLIC DOMAIN NOTICE
@@ -73,7 +73,8 @@ int main(void)
 
     CORE_SetLOGFILE(stderr, 0/*false*/);
     for (j = 1; j <= 3; j++) {
-        srand((int)time(0));
+        g_NCBI_ConnectRandomSeed = (int) time(0) + NCBI_CONNECT_SRAND_ADDEND;
+        srand(g_NCBI_ConnectRandomSeed);
         CORE_LOGF(eLOG_Note, ("Creating heap %d", j));
         if (!(heap = HEAP_Create(0, 0, 4096, s_Expand, 0)))
             CORE_LOG(eLOG_Error, "Cannot create heap");
@@ -165,6 +166,12 @@ int main(void)
 /*
  * --------------------------------------------------------------------------
  * $Log: test_ncbi_heapmgr.c,v $
+ * Revision 6.14  2005/07/11 18:24:46  lavr
+ * Spell ADDEND
+ *
+ * Revision 6.13  2005/05/02 16:12:32  lavr
+ * Use global random seed
+ *
  * Revision 6.12  2003/08/25 14:58:10  lavr
  * Adjust test to take advantage of modified API
  *
