@@ -1,4 +1,4 @@
-/* $Id: blast_driver.c,v 1.92 2005/04/27 20:01:00 dondosha Exp $
+/* $Id: blast_driver.c,v 1.102 2005/10/21 19:32:45 papadopo Exp $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
@@ -32,10 +32,10 @@ Author: Ilya Dondoshansky
 Contents: Main function for running BLAST
 
 ******************************************************************************
- * $Revision: 1.92 $
+ * $Revision: 1.102 $
  * */
 
-static char const rcsid[] = "$Id: blast_driver.c,v 1.92 2005/04/27 20:01:00 dondosha Exp $";
+static char const rcsid[] = "$Id: blast_driver.c,v 1.102 2005/10/21 19:32:45 papadopo Exp $";
 
 #include <ncbi.h>
 #include <sqnutils.h>
@@ -162,10 +162,10 @@ static Args myargs[] = {
    { "Use greedy algorithm for gapped extensions:\n      0 no, 1 one-step, "
      "2 two-step, 3 two-step with ungapped", /* ARG_GREEDY */
      "0", NULL, NULL, FALSE, 'g', ARG_INT, 0.0, 0, NULL},
-   { "Gap open penalty (default: non-affine if greedy; 5 if dyn. prog.)", 
-     "0", NULL, NULL, FALSE, 'G', ARG_INT, 0.0, 0, NULL}, /* ARG_GAPOPEN */
-   { "Gap extension penalty (default: non-affine if greedy; 2 otherwise)",
-     "0", NULL, NULL, FALSE, 'E', ARG_INT, 0.0, 0, NULL}, /* ARG_GAPEXT */
+   { "Gap open penalty (-1 means default: non-affine if greedy; 5 if dyn. prog.)", 
+     "-1", NULL, NULL, FALSE, 'G', ARG_INT, 0.0, 0, NULL}, /* ARG_GAPOPEN */
+   { "Gap extension penalty (-1 means default: non-affine if greedy; 2 otherwise)",
+     "-1", NULL, NULL, FALSE, 'E', ARG_INT, 0.0, 0, NULL}, /* ARG_GAPEXT */
    { "Frame shift penalty for out-of-frame gapping (blastx, tblastn only)",
      "0", NULL, NULL, FALSE, 'h', ARG_INT, 0.0, 0, NULL}, /* ARG_FRAMESHIFT */
    { "X dropoff value for gapped alignment (in bits) (zero invokes default "
@@ -247,7 +247,6 @@ s_FillOptions(SBlastOptions* options)
    Boolean greedy_with_ungapped = FALSE;
    Boolean is_gapped = FALSE;
    EBlastProgramType program_number = options->program;
-   Boolean use_pssm = FALSE;
 
    /* The following options are for blastn only */
    if (program_number == eBlastTypeBlastn) {
@@ -268,7 +267,7 @@ s_FillOptions(SBlastOptions* options)
 
    BLAST_FillLookupTableOptions(lookup_options, program_number, mb_lookup,
       myargs[ARG_THRESHOLD].intvalue, (Int2)myargs[ARG_WORDSIZE].intvalue, 
-      variable_wordsize, use_pssm);
+      variable_wordsize);
    /* Fill the rest of the lookup table options */
    lookup_options->mb_template_length = 
       (Uint1) myargs[ARG_TEMPL_LEN].intvalue;
@@ -373,7 +372,7 @@ s_ParseIntervalLocationArgument(char* arg, Int4* from_ptr, Int4* to_ptr)
    *from_ptr = from;
    *to_ptr = to;
 
-   if (from > to)
+   if (to > 0 && from > to)
       return -1;
    else
       return 0;
@@ -391,9 +390,9 @@ Int2 Nlm_Main(void)
    Int2 status = 0;
    SeqLoc* lcase_mask = NULL;
    SeqLoc* query_slp = NULL;
-   FILE *infp, *outfp;
+   FILE *infp, *outfp = NULL;
    SBlastOptions* options = NULL;
-   BlastFormattingOptions* format_options = NULL;
+   BlastFormattingInfo* format_info = NULL;
    Int2 ctr = 1;
    Int4 num_queries=0;
    int tabular_output = FALSE;
@@ -403,6 +402,7 @@ Int2 Nlm_Main(void)
    Blast_SummaryReturn* sum_returns = NULL;
    Boolean phi_blast = FALSE;
    ValNode* phivnps = NULL;
+   Blast_SummaryReturn* full_sum_returns = NULL;
 
    if (! GetArgs (buf, NUMARG, myargs))
       return (1);
@@ -439,8 +439,7 @@ Int2 Nlm_Main(void)
                   program_number == eBlastTypeTblastx ||
                   program_number == eBlastTypePhiBlastn);
 
-   phi_blast = (program_number == eBlastTypePhiBlastn ||
-                program_number == eBlastTypePhiBlastp);
+   phi_blast = Blast_ProgramIsPhiBlast(program_number);
 
    if ((dbname = myargs[ARG_DB].strvalue) == NULL) {
       Int4 letters_read;
@@ -463,7 +462,8 @@ Int2 Nlm_Main(void)
 
       letters_read = 
          BLAST_GetQuerySeqLoc(infp2, db_is_na, 0, 0, s_from, s_to, NULL, 
-                              &subject_slp, &ctr, NULL, FALSE);
+                              &subject_slp, &ctr, NULL, FALSE,
+                              myargs[ARG_DBGENCODE].intvalue);
 
       if (letters_read <= 0)
       {
@@ -475,77 +475,69 @@ Int2 Nlm_Main(void)
 
    s_FillOptions(options);
 
-   tabular_output = myargs[ARG_TABULAR].intvalue;
-       
-   if (tabular_output) {
-      if ((outfp = FileOpen(myargs[ARG_OUT].strvalue, "w")) == NULL) {
-         ErrPostEx(SEV_FATAL, 1, 0, "blast: Unable to open output file %s\n", 
-                   myargs[ARG_OUT].strvalue);
-        return (1);
-      }
-   } else {
-      if ((status = BlastFormattingOptionsNew(program_number, 
-                       myargs[ARG_OUT].strvalue, 
-                       myargs[ARG_DESCRIPTIONS].intvalue, 
-                       myargs[ARG_ALIGNMENTS].intvalue, 
-                       myargs[ARG_FORMAT].intvalue, &format_options)) != 0)
-         return status;
-
-      if (myargs[ARG_HTML].intvalue) {
-         format_options->html = TRUE;
-         format_options->align_options += TXALIGN_HTML;
-         format_options->print_options += TXALIGN_HTML;
-      }
-
-      if (myargs[ARG_SHOWGI].intvalue) {
-         format_options->align_options += TXALIGN_SHOW_GI;
-         format_options->print_options += TXALIGN_SHOW_GI;
-      }
-
-      if (dbname) {
-         BLAST_PrintOutputHeader(format_options, 
-            (Boolean)myargs[ARG_GREEDY].intvalue, blast_program, dbname, !db_is_na);
-      }
-
-     if (myargs[ARG_UNGAPPED].intvalue != 0) 
-         format_options->print_options += TXALIGN_SHOW_NO_OF_SEGS;
-
-   }
-
    if ((infp = FileOpen(myargs[ARG_QUERY].strvalue, "r")) == NULL) {
       ErrPostEx(SEV_FATAL, 1, 0, "blast: Unable to open input file %s\n", 
                 myargs[ARG_QUERY].strvalue);
       return (1);
    }
 
-   if (tabular_output) {
-      believe_defline = TRUE;
-   }
-
    if (s_ParseIntervalLocationArgument(myargs[ARG_QUERY_LOC].strvalue,
                                        &q_from, &q_to)) {
          ErrPostEx(SEV_FATAL, 1, 0, "Invalid query sequence location\n");
          return -1;
    }
 
+   tabular_output = myargs[ARG_TABULAR].intvalue;
+   
+   /* For on-the-fly tabular and for ASN.1 output, the believe query defline 
+      option must be set to true, otherwise it should be false. */
+   believe_defline = 
+       (tabular_output || myargs[ARG_FORMAT].intvalue == eAlignViewAsnText ||
+        myargs[ARG_FORMAT].intvalue == eAlignViewAsnBinary);
+
+
+   if (tabular_output) {
+      if ((outfp = FileOpen(myargs[ARG_OUT].strvalue, "w")) == NULL) {
+         ErrPostEx(SEV_FATAL, 1, 0, "blast: Unable to open output file %s\n", 
+                   myargs[ARG_OUT].strvalue);
+        return (1);
+      }
+   } else {
+       BlastFormattingInfoNew(myargs[ARG_FORMAT].intvalue, options, 
+                              blast_program, dbname, 
+                              myargs[ARG_OUT].strvalue, &format_info);
+
+       BlastFormattingInfoSetUpOptions(format_info, 
+                                       myargs[ARG_DESCRIPTIONS].intvalue, 
+                                       myargs[ARG_ALIGNMENTS].intvalue,
+                                       (Boolean) myargs[ARG_HTML].intvalue,
+                                       (Boolean) myargs[ARG_GREEDY].intvalue,
+                                       (Boolean) myargs[ARG_SHOWGI].intvalue,
+                                       believe_defline);
+
+       if (dbname)
+           BLAST_PrintOutputHeader(format_info);
+   }
+
    /* Get the query (queries), loop if necessary. */
    while (1) {
        SeqAlign* seqalign = NULL;
        SeqLoc* filter_loc=NULL;	/* All masking locations */
-       Boolean mask_at_hash = FALSE;
        SeqLoc* repeat_mask = NULL; /* Repeat mask locations */
        Int4 letters_read;
 
        if ((Boolean)myargs[ARG_LCASE].intvalue) {
            letters_read = 
                BLAST_GetQuerySeqLoc(infp, query_is_na, 
-                   (Uint1)myargs[ARG_STRAND].intvalue, 0, q_from, q_to, &lcase_mask,
-                   &query_slp, &ctr, &num_queries, believe_defline);
+                   (Uint1)myargs[ARG_STRAND].intvalue, 10000, q_from, q_to, &lcase_mask,
+                   &query_slp, &ctr, &num_queries, believe_defline,
+                   myargs[ARG_GENCODE].intvalue);
        } else {
            letters_read = 
                BLAST_GetQuerySeqLoc(infp, query_is_na,
                    (Uint1)myargs[ARG_STRAND].intvalue, 0, q_from, q_to, NULL, 
-                   &query_slp, &ctr, &num_queries, believe_defline);
+                   &query_slp, &ctr, &num_queries, believe_defline,
+                   myargs[ARG_GENCODE].intvalue);
        }
 
        /* If there is no sequence data left in the input file, break out of 
@@ -596,13 +588,12 @@ Int2 Nlm_Main(void)
            status = 
                Blast_DatabaseSearch(query_slp, dbname, lcase_mask, options, 
                                     tf_data, &seqalign, &filter_loc, 
-                                    &mask_at_hash, sum_returns);
+                                    sum_returns);
        } else {
            status = 
                Blast_TwoSeqLocSetsAdvanced(query_slp, subject_slp, lcase_mask, 
                                            options, tf_data, &seqalign, 
-                                           &filter_loc, &mask_at_hash, 
-                                           sum_returns);
+                                           &filter_loc, sum_returns);
        }
 
        /* Deallocate the data structure used for tabular formatting. */
@@ -613,7 +604,7 @@ Int2 Nlm_Main(void)
 
        /* If masking was done for lookup table only, free the masking locations,
           because they will not be used for formatting. */
-       if (mask_at_hash)
+       if (SBlastOptionsGetMaskAtHash(options))
            filter_loc = Blast_ValNodeMaskListFree(filter_loc);
 
        /* Post warning or error messages, no matter what the search status was. */
@@ -622,9 +613,8 @@ Int2 Nlm_Main(void)
        if (!status) {
            if (phi_blast) {
                status =
-                   PHIBlastFormatResults(program_number, phivnps, dbname,
-                                          query_slp, format_options, 
-                                          sum_returns);
+                   PHIBlastFormatResults(phivnps, query_slp, format_info, 
+                                         sum_returns);
                phivnps = PHIBlastResultsFree(phivnps);
            } else if (!tabular_output) {
                if (myargs[ARG_ASNOUT].strvalue) {
@@ -636,40 +626,39 @@ Int2 Nlm_Main(void)
                
                /* Format the results */
                status = 
-                   BLAST_FormatResults(seqalign, dbname, blast_program, 
-                                       num_queries, query_slp, 
-                                       filter_loc, format_options, 
-                                       (Boolean)myargs[ARG_FRAMESHIFT].intvalue, 
-                                       NULL, sum_returns);
+                   BLAST_FormatResults(seqalign, num_queries, query_slp, 
+                                       filter_loc, format_info, sum_returns);
                
                seqalign = SeqAlignSetFree(seqalign);
            }
 
-           Blast_PrintOutputFooter(program_number, format_options, dbname, 
-                                   sum_returns);
        }
-       /* Clean the summary returns substructures. */
+       /* Update the cumulative summary returns structure and clean the returns
+          substructures for the current search iteration. */
+       Blast_SummaryReturnUpdate(sum_returns, &full_sum_returns);
        Blast_SummaryReturnClean(sum_returns);
        filter_loc = Blast_ValNodeMaskListFree(filter_loc);
        query_slp = SeqLocSetFree(query_slp);
    } /* End loop on sets of queries */
    
-   subject_slp = SeqLocSetFree(subject_slp);
-   options = SBlastOptionsFree(options);
-   sum_returns = Blast_SummaryReturnFree(sum_returns);
-
-   if (!tabular_output) { 
-      if(format_options->html && myargs[ARG_FORMAT].intvalue < 7) {
-         fprintf(format_options->outfp, "</PRE>\n</BODY>\n</HTML>\n");
-      }
-      BlastFormattingOptionsFree(format_options);
-   } else {
-      FileClose(outfp);
-   }
-
    if (infp)
       FileClose(infp);
    
+   subject_slp = SeqLocSetFree(subject_slp);
+
+   /* Print the footer with cumulative summary information. */
+   Blast_PrintOutputFooter(format_info, full_sum_returns);
+
+   sum_returns = Blast_SummaryReturnFree(sum_returns);
+   full_sum_returns = Blast_SummaryReturnFree(full_sum_returns);
+
+   if (!tabular_output) 
+       format_info = BlastFormattingInfoFree(format_info);       
+   else
+       FileClose(outfp);
+
+   options = SBlastOptionsFree(options);
+
    return status;
 }
 
