@@ -1,5 +1,5 @@
-static char const rcsid[] = "$Id: megablast.c,v 6.158 2005/04/27 14:55:09 papadopo Exp $";
-/* $Id: megablast.c,v 6.158 2005/04/27 14:55:09 papadopo Exp $
+static char const rcsid[] = "$Id: megablast.c,v 6.174 2005/10/31 14:15:10 madden Exp $";
+/* $Id: megablast.c,v 6.174 2005/10/31 14:15:10 madden Exp $
 **************************************************************************
 *                                                                         *
 *                             COPYRIGHT NOTICE                            *
@@ -26,6 +26,55 @@ static char const rcsid[] = "$Id: megablast.c,v 6.158 2005/04/27 14:55:09 papado
 *                                                                         *
 ************************************************************************** 
  * $Log: megablast.c,v $
+ * Revision 6.174  2005/10/31 14:15:10  madden
+ * Call SBlastOptionsSetRewardPenaltyAndGapCosts
+ *
+ * Revision 6.173  2005/10/17 14:07:13  madden
+ * Use -1 rather than zero for unset gap parameters
+ *
+ * Revision 6.172  2005/09/16 14:09:45  madden
+ * Print out more informative message when Blast_DatabaseSearch has non-zero return if available
+ *
+ * Revision 6.171  2005/08/29 14:45:34  camacho
+ * From Ilya Dondoshansky:
+ * Retrieve mask_at_hash option from the SBlastOptions structure instead of
+ * passing as argument in search API calls
+ *
+ * Revision 6.170  2005/08/22 19:22:56  madden
+ * Check return value of Blast_DatabaseSearch, call ErrPostEx if non-zero
+ *
+ * Revision 6.169  2005/08/18 14:19:24  madden
+ * Set cutoff_score for new engine if specified on command-line
+ *
+ * Revision 6.168  2005/08/15 18:50:35  madden
+ * Remove unused variable
+ *
+ * Revision 6.167  2005/08/09 21:36:54  dondosha
+ * Added extra argument in calls to BLAST_GetQuerySeqLoc
+ *
+ * Revision 6.166  2005/08/05 22:28:44  dondosha
+ * Pass number of CPUs option to the engine
+ *
+ * Revision 6.165  2005/06/09 17:10:03  dondosha
+ * Validate options before proceeding with search
+ *
+ * Revision 6.164  2005/06/08 20:23:24  dondosha
+ * Previous change wrong - rolled back
+ *
+ * Revision 6.162  2005/06/02 20:45:05  dondosha
+ * 1. Print footer only once after the loop over sets of queries;
+ * 2. Format XML output in a valid XML;
+ * 3. Use BlastFormattingInfo structure for formatting.
+ *
+ * Revision 6.161  2005/05/20 18:57:51  camacho
+ * Update to use new signature to BLAST_FillLookupTableOptions
+ *
+ * Revision 6.160  2005/05/02 17:44:07  coulouri
+ * bring default evalue and output type in line with other blast programs
+ *
+ * Revision 6.159  2005/05/02 17:00:28  coulouri
+ * change default to new engine
+ *
  * Revision 6.158  2005/04/27 14:55:09  papadopo
  * change signature of BlastFillHitSavingOptions
  *
@@ -528,6 +577,7 @@ static char const rcsid[] = "$Id: megablast.c,v 6.158 2005/04/27 14:55:09 papado
 #include <algo/blast/api/blast_api.h>
 #include <algo/blast/api/blast_seq.h>
 #include <algo/blast/api/repeats_filter.h>
+#include <algo/blast/core/blast_setup.h>
 #endif
 
 #define DEFLINE_BUF 255
@@ -581,7 +631,6 @@ MegaBlastPrintEndpoints(VoidPtr ptr)
    BLAST_HSPPtr hsp; 
    Int2 context;
    Char context_sign;
-   Uint4 header_index = 0;
    Int4 subject_gi, score;
    FILE *fp = (FILE *) search->output;
 
@@ -611,7 +660,7 @@ MegaBlastPrintEndpoints(VoidPtr ptr)
            !StringICmp(db_tag->db, "TI")) && 
           db_tag->tag->id != 0) {
          subject_buffer = (CharPtr) Malloc(16);
-         sprintf(subject_buffer, "%ld", db_tag->tag->id);
+         sprintf(subject_buffer, "%ld", (long) db_tag->tag->id);
       } else {
          subject_buffer = StringTokMT(subject_descr, " \t", &subject_descr);
          subject_descr = subject_buffer;
@@ -680,9 +729,8 @@ MegaBlastPrintEndpoints(VoidPtr ptr)
             query_buffer = StringTokMT(title, " ", &title);
          else {
             Int4 query_gi;
-            Boolean numeric_query_id =
-               GetAccessionFromSeqId(query_bsp->id, &query_gi,
-                                     &query_buffer);
+            GetAccessionFromSeqId(query_bsp->id, &query_gi,
+                                  &query_buffer);
          }  
          BioseqUnlock(query_bsp);
       } else {
@@ -720,7 +768,7 @@ MegaBlastPrintEndpoints(VoidPtr ptr)
       hsp->subject.end += s_shift;
 
       if (numeric_sip_type)
-	 fprintf(fp, "'%ld'=='%c%s' (%d %d %d %d) %d\n", subject_gi, 
+	 fprintf(fp, "'%ld'=='%c%s' (%d %d %d %d) %d\n", (long) subject_gi, 
 		 context_sign, query_buffer, hsp->subject.offset, q_start, 
 		 hsp->subject.end, q_end, score);
       else 
@@ -859,9 +907,8 @@ MegaBlastPrintSegments(VoidPtr ptr)
             query_buffer = StringTokMT(title, " ", &title);
          else {
             Int4 query_gi;
-            Boolean numeric_query_id =
-               GetAccessionFromSeqId(query_bsp->id, &query_gi,
-                                     &query_buffer);
+            GetAccessionFromSeqId(query_bsp->id, &query_gi,
+                                  &query_buffer);
          }  
          BioseqUnlock(query_bsp);
       } else {
@@ -876,7 +923,7 @@ MegaBlastPrintSegments(VoidPtr ptr)
 
       if (numeric_sip_type)
 	 sprintf(buffer, "\n#'>%ld'=='%c%s' (%d %d %d %d) %d\na {\n  s %d\n  b %d %d\n  e %d %d\n", 
-	      subject_gi, strand, query_buffer, 
+	      (long) subject_gi, strand, query_buffer, 
 	      s_start, q_start, s_end, q_end, score, score, 
 	      s_start, q_start, s_end, q_end);
       else 
@@ -1030,7 +1077,7 @@ static Args myargs [] = {
   { "Query File", 
 	NULL, NULL, NULL, FALSE, 'i', ARG_FILE_IN, 0.0, 0, NULL}, /* ARG_QUERY */
   { "Expectation value", 
-	"1000000.0", NULL, NULL, FALSE, 'e', ARG_FLOAT, 0.0, 0, NULL},/* ARG_EVALUE */
+	"10.0", NULL, NULL, FALSE, 'e', ARG_FLOAT, 0.0, 0, NULL},/* ARG_EVALUE */
   { "alignment view options:\n0 = pairwise,\n1 = query-anchored showing identities,\n2 = query-anchored no identities,\n3 = flat query-anchored, show identities,\n4 = flat query-anchored, no identities,\n5 = query-anchored no identities and blunt ends,\n6 = flat query-anchored, no identities and blunt ends,\n7 = XML Blast output,\n8 = tabular, \n9 tabular with comment lines,\n10 ASN, text\n11 ASN, binary", 
         "0", NULL, NULL, FALSE, 'm', ARG_INT, 0.0, 0, NULL},       /* ARG_FORMAT */
   { "BLAST report Output File", 
@@ -1050,7 +1097,7 @@ static Args myargs [] = {
   { "Number of database sequence to show alignments for (B)",
         "250", NULL, NULL, FALSE, 'b', ARG_INT, 0.0, 0, NULL},     /* ARG_ALIGNMENTS */
   { "Type of output:\n0 - alignment endpoints and score,\n1 - all ungapped segments endpoints,\n2 - traditional BLAST output,\n3 - tab-delimited one line format",
-        "0", NULL, NULL, FALSE, 'D', ARG_INT, 0.0, 0, NULL},       /* ARG_OUTTYPE */
+        "2", NULL, NULL, FALSE, 'D', ARG_INT, 0.0, 0, NULL},       /* ARG_OUTTYPE */
   { "Number of processors to use",
         "1", NULL, NULL, FALSE, 'a', ARG_INT, 0.0, 0, NULL},       /* ARG_THREADS */
   { "ASN.1 SeqAlign file; must be used in conjunction with -D2 option", 
@@ -1071,10 +1118,10 @@ static Args myargs [] = {
         "F", NULL, NULL, FALSE, 'T', ARG_BOOLEAN, 0.0, 0, NULL},   /* ARG_HTML */
   { "Restrict search of database to list of GI's",
 	NULL, NULL, NULL, TRUE, 'l', ARG_STRING, 0.0, 0, NULL},    /* ARG_GILIST */
-  { "Cost to open a gap (zero invokes default behavior)",          /* ARG_GAPOPEN */
-        "0", NULL, NULL, FALSE, 'G', ARG_INT, 0.0, 0, NULL},
-  { "Cost to extend a gap (zero invokes default behavior)",        /* ARG_GAPEXT */
-        "0", NULL, NULL, FALSE, 'E', ARG_INT, 0.0, 0, NULL},
+  { "Cost to open a gap (-1 invokes default behavior)",          /* ARG_GAPOPEN */
+        "-1", NULL, NULL, FALSE, 'G', ARG_INT, 0.0, 0, NULL},
+  { "Cost to extend a gap (-1 invokes default behavior)",        /* ARG_GAPEXT */
+        "-1", NULL, NULL, FALSE, 'E', ARG_INT, 0.0, 0, NULL},
   { "Minimal hit score to report (0 for default behavior)",        /* ARG_MINSCORE */
         "0", NULL, NULL, FALSE, 's', ARG_INT, 0.0, 0, NULL},
   { "Masked query output, must be used in conjunction with -D 2 option", 
@@ -1108,8 +1155,8 @@ static Args myargs [] = {
 	"0", NULL, NULL, FALSE, 'H', ARG_INT, 0.0, 0, NULL},        /* ARG_MAXHSP */
 #endif
 #if MB_ALLOW_NEW
-  {"Force use of old engine",
-        "T", NULL, NULL, TRUE, 'V', ARG_BOOLEAN, 0.0, 0, NULL}    /* ARG_FORCE_OLD */
+  {"Force use of the legacy BLAST engine",
+        "F", NULL, NULL, TRUE, 'V', ARG_BOOLEAN, 0.0, 0, NULL}    /* ARG_FORCE_OLD */
 #endif
 
 };
@@ -1119,7 +1166,7 @@ static Args myargs [] = {
 static Int2 Main_old (void)
  
 {
-   AsnIoPtr aip, xml_aip;
+   AsnIoPtr aip, xml_aip = NULL;
    BioseqPtr query_bsp, PNTR query_bsp_array;
    BioSourcePtr source;
    BLAST_MatrixPtr matrix;
@@ -1223,8 +1270,10 @@ static Int2 Main_old (void)
 	show_gi = (Boolean) myargs[ARG_SHOWGIS].intvalue;
 	options->penalty = myargs[ARG_MISMATCH].intvalue;
 	options->reward = myargs[ARG_MATCH].intvalue;
-	options->gap_open = myargs[ARG_GAPOPEN].intvalue;
-	options->gap_extend = myargs[ARG_GAPEXT].intvalue;
+        if (myargs[ARG_GAPOPEN].intvalue >= 0)
+	    options->gap_open = myargs[ARG_GAPOPEN].intvalue;
+        if (myargs[ARG_GAPEXT].intvalue >= 0)
+	    options->gap_extend = myargs[ARG_GAPEXT].intvalue;
 
 	if (options->gap_open == 0 && options->reward % 2 == 0 && 
 	    options->gap_extend == options->reward / 2 - options->penalty)
@@ -1745,16 +1794,12 @@ GetLambdaFast(const BlastScoringOptions* scoring_options)
 #if MB_ALLOW_NEW
 /** Fills all the options structures with user defined values. Uses the 
  * myargs global structure obtained from GetArgs.
- * @param lookup_options Lookup table options [in]
- * @param query_setup_options Query options [in]
- * @param word_options Initial word processing options [in]
- * @param ext_options Extension options [in]
- * @param hit_options Hit saving options [out]
- * @param score_options Scoring options [out]
- * @param eff_len_options Effective length options [out]
+ * @param options Wrapper for all search options [in] [out]
+ * @param sum_returns Returns structure for passing back error 
+ *                    messages [in] [out]
  */
 static Int2 
-BLAST_FillOptions(SBlastOptions* options)
+BLAST_FillOptions(SBlastOptions* options, Blast_SummaryReturn* sum_returns)
 {
    LookupTableOptions* lookup_options = options->lookup_options;
    QuerySetUpOptions* query_setup_options = options->query_options; 
@@ -1762,20 +1807,27 @@ BLAST_FillOptions(SBlastOptions* options)
    BlastExtensionOptions* ext_options = options->ext_options;
    BlastHitSavingOptions* hit_options = options->hit_options ;
    BlastScoringOptions* score_options = options->score_options;
-   BlastEffectiveLengthsOptions* eff_len_options = options->eff_len_options;
    Boolean ag_blast = FALSE, variable_wordsize = FALSE, mb_lookup = TRUE;
    Boolean greedy=TRUE; /* greedy alignment should be done. */
    double lambda=0;
-   Int2 status;
-   const Uint1 program_number = eBlastTypeBlastn; 
+   const EBlastProgramType kProgram = eBlastTypeBlastn; 
+   Int2 status = 0;
 
    if (myargs[ARG_DYNAMIC].intvalue != 0) 
        greedy = FALSE; 
 
-   BLAST_FillScoringOptions(score_options, program_number, 
+   options->num_cpus = myargs[ARG_THREADS].intvalue;
+
+   BLAST_FillScoringOptions(score_options, kProgram, 
                 greedy, myargs[ARG_MISMATCH].intvalue, myargs[ARG_MATCH].intvalue,
                 NULL, myargs[ARG_GAPOPEN].intvalue, myargs[ARG_GAPEXT].intvalue);
 
+   SBlastOptionsSetRewardPenaltyAndGapCosts(options,
+        myargs[ARG_MATCH].intvalue,
+        myargs[ARG_MISMATCH].intvalue,
+        MAX(0, myargs[ARG_GAPOPEN].intvalue),
+        MAX(0, myargs[ARG_GAPEXT].intvalue));
+
    /* Use to "unscale" x-dropoff since they were never properly scaled in the old code but are in the new
       code, so we wish to keep megablast running like the old code without adding this "bug" back to the main 
       engine. The returned Lambda is also increased by 2% to make sure the Xdrop value we use is at least as
@@ -1794,8 +1846,8 @@ BLAST_FillOptions(SBlastOptions* options)
          variable_wordsize = FALSE;
    }
 
-   BLAST_FillLookupTableOptions(lookup_options, program_number, mb_lookup,
-      0, myargs[ARG_WORDSIZE].intvalue, variable_wordsize, FALSE);
+   BLAST_FillLookupTableOptions(lookup_options, kProgram, mb_lookup,
+      0, myargs[ARG_WORDSIZE].intvalue, variable_wordsize);
    /* Fill the rest of the lookup table options */
    lookup_options->mb_template_length = 
       (Uint1) myargs[ARG_TEMPL_LEN].intvalue;
@@ -1805,14 +1857,14 @@ BLAST_FillOptions(SBlastOptions* options)
    if (myargs[ARG_EVERYBASE].intvalue)
       lookup_options->full_byte_scan = FALSE;
    
-   BLAST_FillQuerySetUpOptions(query_setup_options, program_number, 
+   BLAST_FillQuerySetUpOptions(query_setup_options, kProgram, 
       myargs[ARG_FILTER].strvalue, myargs[ARG_STRAND].intvalue);
 
-   BLAST_FillInitialWordOptions(word_options, program_number, 
+   BLAST_FillInitialWordOptions(word_options, kProgram, 
       greedy, myargs[ARG_WINDOW].intvalue,
       lambda*myargs[ARG_XDROP_UNGAPPED].intvalue/NCBIMATH_LN2);
 
-   BLAST_FillExtensionOptions(ext_options, program_number, (greedy ? 1 : 0), 
+   BLAST_FillExtensionOptions(ext_options, kProgram, (greedy ? 1 : 0), 
       lambda*myargs[ARG_XDROP].intvalue/NCBIMATH_LN2, 
       lambda*myargs[ARG_XDROP_FINAL].intvalue/NCBIMATH_LN2);
 
@@ -1822,9 +1874,10 @@ BLAST_FillOptions(SBlastOptions* options)
       myargs[ARG_EVALUE].floatvalue, 
       MAX(myargs[ARG_DESCRIPTIONS].intvalue, 
           myargs[ARG_ALIGNMENTS].intvalue),
-      score_options->gapped_calculation,
-      0);
- 
+      score_options->gapped_calculation, 0);
+
+   if (myargs[ARG_MINSCORE].intvalue)
+        hit_options->cutoff_score = myargs[ARG_MINSCORE].intvalue;
    
    word_options->ungapped_extension = TRUE;
    /* For discontiguous megablast we need to use a less drastic method of elimination redundant hits (at least to
@@ -1843,32 +1896,34 @@ BLAST_FillOptions(SBlastOptions* options)
       eff_len_options->searchsp_eff = (Int8) myargs[ARG_SEARCHSP].floatvalue; 
    }
 */
+   /* Validate the options. */
+   status = BLAST_ValidateOptions(kProgram, ext_options, score_options, 
+                                  lookup_options, word_options, hit_options, 
+                                  &sum_returns->error);
 
-   return 0;
+   return status;
 }
 
 static Int2 Main_new(void)
 {
    const Boolean query_is_na = TRUE; 
-   const Boolean db_is_na = TRUE;
    Boolean believe_query = FALSE;
    char* program_name = "blastn";
    Int2 status = 0;
    Int4 start=0, end=0;   /* start and end of sequence to be searched as specified by ARG_QUERYLOC */
    SeqLoc* lcase_mask = NULL;
-   SeqLoc* filter_loc=NULL;	/* All masking locations */
    SeqLoc* query_slp = NULL;
    FILE *infp=NULL, *outfp=NULL;
    SBlastOptions* options = NULL;
-   BlastFormattingOptions* format_options;
    Int2 ctr = 1;
    Int4 num_queries_total=0;  /* total number of queries read. */
    Boolean tabular_output = FALSE;
    BlastTabularFormatData* tf_data = NULL;
-   int num_threads;
    char* dbname = myargs[ARG_DB].strvalue;
+   BlastFormattingInfo* format_info = NULL;
    Blast_SummaryReturn* sum_returns = NULL;
-   Int4 align_view;
+   Blast_SummaryReturn* full_sum_returns = NULL;
+   EAlignView align_view = eAlignViewMax;
 
    if (myargs[ARG_OUTTYPE].intvalue == 3)
        tabular_output = TRUE;
@@ -1876,7 +1931,11 @@ static Int2 Main_new(void)
    sum_returns = Blast_SummaryReturnNew();
    status = SBlastOptionsNew(program_name, &options, sum_returns);
 
+   if (!status)
+       status = BLAST_FillOptions(options, sum_returns);
+
    if (status) {
+       options = SBlastOptionsFree(options);
        if (sum_returns->error) {
            Blast_SummaryReturnsPostError(sum_returns);
            sum_returns = Blast_SummaryReturnFree(sum_returns);
@@ -1884,21 +1943,19 @@ static Int2 Main_new(void)
        return -1;
    }
 
-   BLAST_FillOptions(options);
+   if (myargs[ARG_FORMAT].intvalue >= eAlignViewPairwise &&
+       myargs[ARG_FORMAT].intvalue < eAlignViewMax)
+       align_view = (EAlignView) myargs[ARG_FORMAT].intvalue;
+   else 
+       ErrPostEx(SEV_FATAL, 1, 0, "Unsupported value for the -m option");
 
-   align_view = myargs[ARG_FORMAT].intvalue;
-
-   /* If tabular output is requested, set believe_query to TRUE,
-      to guarantee meaningful sequence ids in the output. */
-   if (tabular_output)
-       believe_query = TRUE;
-   else
-       believe_query = (Boolean) myargs[ARG_BELIEVEQUERY].intvalue;
+   believe_query = (Boolean) myargs[ARG_BELIEVEQUERY].intvalue;
 
    /* If ASN.1 output is requested and believe_query is not set to TRUE,
       exit with an error. */
    if (!believe_query && (myargs[ARG_ASNOUT].strvalue ||
-                          align_view == 10 || align_view == 11)) {
+                          align_view == eAlignViewAsnText || 
+                          align_view == eAlignViewAsnBinary)) {
        ErrPostEx(SEV_FATAL, 1, 0,
                  "-J option must be TRUE to produce ASN.1 output; before "
                  "changing -J to TRUE please also ensure that all query "
@@ -1907,31 +1964,30 @@ static Int2 Main_new(void)
    }
 
    if (!tabular_output) {
-      if ((status = BlastFormattingOptionsNew(options->program, 
-                       myargs[ARG_OUT].strvalue, 
-                       myargs[ARG_DESCRIPTIONS].intvalue, 
-                       myargs[ARG_ALIGNMENTS].intvalue, 
-                       align_view, &format_options)) != 0)
-         return status;
-      format_options->html = (Boolean) myargs[ARG_HTML].intvalue;
-      if (myargs[ARG_SHOWGIS].intvalue == 0)
-      { /* These are default in new api, so we turn off if not requested. */
-           format_options->align_options -= TXALIGN_SHOW_GI;
-           format_options->print_options -= TXALIGN_SHOW_GI;
-      }
+       BlastFormattingInfoNew(myargs[ARG_FORMAT].intvalue, options, 
+                              program_name, dbname, 
+                              myargs[ARG_OUT].strvalue, &format_info);
 
-      if (dbname) {
-         BLAST_PrintOutputHeader(format_options, TRUE, program_name, dbname, 
-                                 !db_is_na);
-      }
-   }
-   else
-   { /* tabular output requires raw FILE*. */
+       /* Pass TRUE for the "is megablast" argument. Since megablast is always
+          gapped, pass FALSE for the "is ungapped" argument. */
+       BlastFormattingInfoSetUpOptions(format_info, 
+                                       myargs[ARG_DESCRIPTIONS].intvalue, 
+                                       myargs[ARG_ALIGNMENTS].intvalue,
+                                       (Boolean) myargs[ARG_HTML].intvalue,
+                                       TRUE,
+                                       (Boolean) myargs[ARG_SHOWGIS].intvalue,
+                                       believe_query);
+
+       if (dbname)
+           BLAST_PrintOutputHeader(format_info);
+   } else { /* tabular output requires raw FILE*. */
        if ((outfp = FileOpen(myargs[ARG_OUT].strvalue, "w")) == NULL) {
             ErrPostEx(SEV_FATAL, 1, 0, "blast: Unable to open output file %s\n", 
                 myargs[ARG_OUT].strvalue);
             return (1);
        }
+       /* If tabular output is requested, automatically set believe_query to
+          TRUE, to guarantee meaningful sequence ids in the output. */
        believe_query = TRUE;
    }
 
@@ -1945,13 +2001,10 @@ static Int2 Main_new(void)
 
    Megablast_GetLoc(myargs[ARG_QUERYLOC].strvalue, &start, &end);
 
-   num_threads = myargs[ARG_THREADS].intvalue;
-
    /* Get the query (queries), loop if necessary. */
    while (1) {
        SeqAlign* seqalign = NULL;
        SeqLoc* filter_loc=NULL;	/* All masking locations */
-       Boolean mask_at_hash;
        SeqLoc* repeat_mask = NULL; /* Repeat mask locations */
        Int4 num_queries; /* Number of queries read this time. */
        Int4  letters_read;  /* number of letters (bases/residues) read. */
@@ -1959,11 +2012,11 @@ static Int2 Main_new(void)
       if ((Boolean)myargs[ARG_LCASE].intvalue) {
          letters_read = BLAST_GetQuerySeqLoc(infp, query_is_na, 
                    myargs[ARG_STRAND].intvalue, myargs[ARG_MAXQUERY].intvalue, start, end,
-                   &lcase_mask, &query_slp, &ctr, &num_queries, believe_query);
+                   &lcase_mask, &query_slp, &ctr, &num_queries, believe_query, 0);
       } else {
          letters_read = BLAST_GetQuerySeqLoc(infp, query_is_na,
                    myargs[ARG_STRAND].intvalue, myargs[ARG_MAXQUERY].intvalue, start, end, 
-                   NULL, &query_slp, &ctr, &num_queries, believe_query);
+                   NULL, &query_slp, &ctr, &num_queries, believe_query, 0);
       }
 
       if (letters_read == 0)
@@ -1998,10 +2051,15 @@ static Int2 Main_new(void)
           lcase_mask = ValNodeLink(&lcase_mask, repeat_mask);
       
        /* The main search is here. */
-      status = 
-          Blast_DatabaseSearch(query_slp, dbname, lcase_mask, options, tf_data,
-                               &seqalign, &filter_loc, &mask_at_hash, 
-                               sum_returns);
+      if((status = Blast_DatabaseSearch(query_slp, dbname, lcase_mask, options, tf_data,
+                               &seqalign, &filter_loc, sum_returns)) != 0)
+      {
+           if (sum_returns && sum_returns->error)
+               ErrPostEx(SEV_ERROR, 1, 0, sum_returns->error->message);
+           else
+               ErrPostEx(SEV_ERROR, 1, 0, "Non-zero return from Blast_DatabaseSearch\n");
+           return status;
+      }
 
       /* Deallocate the data structure used for tabular formatting. */
       BlastTabularFormatDataFree(tf_data);
@@ -2011,7 +2069,7 @@ static Int2 Main_new(void)
       
       /* If masking was done for lookup table only, free the masking locations,
          because they will not be used for formatting. */
-      if (mask_at_hash)
+      if (SBlastOptionsGetMaskAtHash(options))
           filter_loc = Blast_ValNodeMaskListFree(filter_loc);
 
       /* Post warning or error messages, no matter what the search status was. */
@@ -2027,29 +2085,33 @@ static Int2 Main_new(void)
           /* Format the results; note that seqalign and filter locations 
              are freed inside. */
           status = 
-              BLAST_FormatResults(seqalign, myargs[ARG_DB].strvalue, 
-                                  program_name, num_queries, query_slp, 
-                                  filter_loc, format_options, FALSE, NULL, 
-                                  sum_returns);
+              BLAST_FormatResults(seqalign, num_queries, query_slp, 
+                                  filter_loc, format_info, sum_returns);
           
           seqalign = SeqAlignSetFree(seqalign);
 
-          Blast_PrintOutputFooter(options->program, format_options, dbname, 
-                                  sum_returns);
       }
+      /* Update the cumulative summary returns structure and clean the returns
+         substructures for the current search iteration. */
+      Blast_SummaryReturnUpdate(sum_returns, &full_sum_returns);
       Blast_SummaryReturnClean(sum_returns);
       filter_loc = Blast_ValNodeMaskListFree(filter_loc);
       query_slp = SeqLocSetFree(query_slp);
    } /* End loop on sets of queries */
    
-   options = SBlastOptionsFree(options);
+   if (infp)
+      FileClose(infp);
+   
+   Blast_PrintOutputFooter(format_info, full_sum_returns);
+
    sum_returns = Blast_SummaryReturnFree(sum_returns);
+   full_sum_returns = Blast_SummaryReturnFree(full_sum_returns);
 
    if (!tabular_output)
    {
-      if (align_view < 7 && myargs[ARG_LOGINFO].intvalue)
-            BlastPrintLogReport(format_options, num_queries_total);
-      format_options = BlastFormattingOptionsFree(format_options);
+       if (align_view < eAlignViewXml && myargs[ARG_LOGINFO].intvalue)
+           BlastPrintLogReport(format_info);
+       format_info = BlastFormattingInfoFree(format_info);       
    }
    else
    {
@@ -2059,9 +2121,8 @@ static Int2 Main_new(void)
       FileClose(outfp);
    }
 
-   if (infp)
-      FileClose(infp);
-   
+   options = SBlastOptionsFree(options);
+
    return status;
 }
 #endif
