@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   1/22/95
 *
-* $Revision: 6.33 $
+* $Revision: 6.49 $
 *
 * File Description: 
 *
@@ -85,11 +85,13 @@ extern ValNodePtr AddStringToValNodeChain (ValNodePtr head, CharPtr str, Uint1 c
 #define FEATURE_FORM_BLOCK          \
   DESCRIPTOR_FORM_BLOCK             \
   DialoG          commonRadio;      \
-  GrouP           commonSubGrp [5]; \
+  GrouP           commonSubGrp [8]; \
   Int2            commonPage;       \
   ButtoN          partial;          \
   ButtoN          exception;        \
   ButtoN          pseudo;           \
+  DialoG          experiment;       \
+  DialoG          inference;        \
   TexT            comment;          \
   TexT            title;            \
   TexT            exceptText;       \
@@ -112,6 +114,9 @@ extern ValNodePtr AddStringToValNodeChain (ValNodePtr head, CharPtr str, Uint1 c
   DialoG          dbxrefs;          \
   DialoG          gbquals;          \
   DialoG          usrobjext;        \
+  TexT            featid;           \
+  TexT            fidxref;          \
+  ButtoN          leave_dlg_up;     \
   UserObjectPtr   goTermUserObj;
 
 typedef struct descform {
@@ -137,7 +142,9 @@ extern void StdInitFeatFormProc (ForM f);
 extern void StdFeatFormCleanupProc (GraphiC g, VoidPtr data);
 extern void StdFeatFormAcceptButtonProc (ButtoN b);
 
+/*
 extern void ExtendGeneFeatIfOnMRNA (Uint2 entityID, SeqEntryPtr sep);
+*/
 
 extern OMUserDataPtr ItemAlreadyHasEditor (Uint2 entityID, Uint2 itemID, Uint2 itemtype, Uint2 procid);
 extern Int2 LIBCALLBACK StdVibrantEditorMsgFunc (OMMsgStructPtr ommsp);
@@ -191,6 +198,23 @@ extern DialoG CreateIntervalEditorDialogEx (GrouP h, CharPtr title, Uint2 rows,
                                             Boolean useBar, Boolean showPartials,
                                             Boolean allowGaps, FeatureFormPtr ffp,
                                             IntEdPartialProc proc);
+                                            
+extern DialoG CreateIntervalEditorDialogExEx (GrouP h, CharPtr title, Uint2 rows,
+                                              Int2 spacing, SeqEntryPtr sep,
+                                              Boolean nucsOK, Boolean protsOK,
+                                              Boolean useBar, Boolean showPartials,
+                                              Boolean allowGaps, FeatureFormPtr ffp,
+                                              IntEdPartialProc proc, 
+                                              Boolean use_aln, Boolean show_seqid,
+                                              TaglistCallback tlp_callback, 
+                                              Pointer callback_data,
+                                              Boolean allow_nulls_in_list);
+
+extern void DisplayErrorMessages (CharPtr title, ValNodePtr err_list);
+
+extern Boolean AdjustFromForGap (Int4Ptr p_from, SeqAlignPtr salp, Int4 aln_len, Int4 aln_row);
+extern Boolean AdjustToForGap (Int4Ptr p_to, SeqAlignPtr salp, Int4 aln_row);
+                                            
 extern void SetSequenceAndStrandForIntervalPage (DialoG d);                                            
 extern void StdFeatIntEdPartialCallback (FeatureFormPtr ffp, Boolean partial5, Boolean partial3, Boolean order);
 
@@ -323,20 +347,27 @@ extern DialoG FeatCitEditDialog (GrouP parent, Uint2 entityID);
 typedef struct modalacceptcancel
 {
   Boolean accepted;
-  Boolean cancelled;  
+  Boolean cancelled; 
+  Boolean third_option; 
 } ModalAcceptCancelData, PNTR ModalAcceptCancelPtr;
 
 extern void ModalAcceptButton (ButtoN b);
 extern void ModalCancelButton (ButtoN b);
+extern void ModalThirdOptionButton (ButtoN b);
 
 typedef void (*TableDisplayDblClick) PROTO((PoinT, CharPtr, CharPtr, Pointer));
+typedef Boolean (*TableDisplayLeftInRed) PROTO ((Int4, ValNodePtr, Pointer));
 extern DialoG TableDisplayDialog (GrouP parent, Int4 width, Int4 height,
                                   Int4 frozen_header, Int4 frozen_left,
                                   TableDisplayDblClick dbl_click,
-                                  Pointer dbl_click_data);
+                                  Pointer dbl_click_data,
+                                  TableDisplayLeftInRed,
+                                  Pointer left_in_red_data);
 extern ValNodePtr FreeTableDisplayRowList (ValNodePtr row_list);
 extern void PrintTableDisplayRowListToFile (ValNodePtr row_list, FILE *fp);
 extern ValNodePtr CopyTableDisplayRowList (ValNodePtr row_list);
+extern CharPtr GetRowListCellText (ValNodePtr row_list, Int4 row, Int4 column);
+extern FonT GetTableDisplayDefaultFont (void);
 
 typedef  void  (*Nlm_ChangeNotifyProc) PROTO ((Pointer));
 
@@ -356,6 +387,16 @@ extern DialoG SelectionDialog
  ValNodePtr               choice_list,
  Int2                     list_height);
 
+extern DialoG SelectionDialogEx 
+(GrouP h,
+ Nlm_ChangeNotifyProc     change_notify,
+ Pointer                  change_userdata,
+ Boolean                  allow_multi,
+ CharPtr                  err_msg,
+ ValNodePtr               choice_list,
+ Int2                     list_height,
+ Boolean                  force_list);
+
 /* This function should free just the data associated with the ValNode */
 typedef void (*FreeValNodeProc) PROTO ((ValNodePtr));
 /* This function should copy just this ValNode, not the chain */
@@ -376,6 +417,7 @@ extern DialoG ValNodeSelectionDialogEx
  Nlm_ChangeNotifyProc     change_notify,
  Pointer                  change_userdata,
  Boolean                  allow_multi,
+ Boolean                  force_list, 
  RemapValNodeProc         remap_vn_proc);
 extern DialoG ValNodeSelectionDialog
 (GrouP h,
@@ -402,6 +444,16 @@ extern CharPtr ValNodeStringName (ValNodePtr vnp);
 extern void ValNodeSimpleDataFree (ValNodePtr vnp);
 extern ValNodePtr ValNodeStringCopy (ValNodePtr vnp);
 extern Boolean ValNodeChoiceMatch (ValNodePtr vnp1, ValNodePtr vnp2);
+extern Boolean ValNodeStringMatch (ValNodePtr vnp1, ValNodePtr vnp2);
+
+extern DialoG SequenceSelectionDialog 
+(GrouP h,
+ Nlm_ChangeNotifyProc     change_notify,
+ Pointer                  change_userdata,
+ Boolean                  allow_multi,
+ Boolean                  show_nucs,
+ Boolean                  show_prots,
+ Uint2                    entityID);
 
 
 #ifdef __cplusplus
