@@ -29,7 +29,7 @@
 *
 * Version Creation Date:   2/5/97
 *
-* $Revision: 6.85 $
+* $Revision: 6.87 $
 *
 * File Description: 
 *
@@ -1017,6 +1017,9 @@ static void PopulateFlatFile (BioseqViewPtr bvp, FmtType format, FlgType flags)
       case 6 :
         custom |= ONLY_REVIEW_PUBS;
         break;
+      case 7 :
+        custom |= HIDE_ALL_PUBS;
+        break;
       default :
         break;
     }
@@ -1684,6 +1687,9 @@ static void PopulateAsnOrXML (BioseqViewPtr bvp, CharPtr outmode, Boolean doGbse
         case 6 :
           custom |= ONLY_REVIEW_PUBS;
           break;
+        case 7 :
+          custom |= HIDE_ALL_PUBS;
+          break;
         default :
           break;
       }
@@ -2235,7 +2241,7 @@ static void ExportFeatureTable (BioseqViewPtr bvp, CharPtr filename, CharPtr dfa
 static CharPtr asnconfirmmsg =
 "'Export' saves only the targeted portion of the record.\n\
 Use 'Save' to save the entire record to a file, so that\n\
-it can be read back in.  Proceed with export?";
+it can be submitted to the database.  Proceed with export?";
 
 static void ExportAsnAfterConfirming (BioseqViewPtr bvp, CharPtr filename, CharPtr dfault)
 
